@@ -2,6 +2,7 @@
 -- |
 -- Module      :  Distribution.PackageDescription.Check
 -- Copyright   :  Lennart Kolmodin 2008
+-- License     :  BSD3
 --
 -- Maintainer  :  cabal-devel@haskell.org
 -- Portability :  portable
@@ -14,41 +15,11 @@
 -- different kinds of check so we can see which ones are appropriate to report
 -- in different situations. This code gets uses when configuring a package when
 -- we consider only basic problems. The higher standard is uses when when
--- preparing a source tarball and by hackage when uploading new packages. The
+-- preparing a source tarball and by Hackage when uploading new packages. The
 -- reason for this is that we want to hold packages that are expected to be
 -- distributed to a higher standard than packages that are only ever expected
 -- to be used on the author's own environment.
 
-{- All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions are
-met:
-
-    * Redistributions of source code must retain the above copyright
-      notice, this list of conditions and the following disclaimer.
-
-    * Redistributions in binary form must reproduce the above
-      copyright notice, this list of conditions and the following
-      disclaimer in the documentation and/or other materials provided
-      with the distribution.
-
-    * Neither the name of Isaac Jones nor the names of other
-      contributors may be used to endorse or promote products derived
-      from this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. -}
-
 module Distribution.PackageDescription.Check (
         -- * Package Checking
         PackageCheck(..),
@@ -69,12 +40,14 @@ import Control.Monad
          ( filterM, liftM )
 import qualified System.Directory as System
          ( doesFileExist, doesDirectoryExist )
+import qualified Data.Map as Map
 
 import Distribution.PackageDescription
 import Distribution.PackageDescription.Configuration
          ( flattenPackageDescription, finalizePackageDescription )
 import Distribution.Compiler
-         ( CompilerFlavor(..), buildCompilerFlavor, CompilerId(..) )
+         ( CompilerFlavor(..), buildCompilerFlavor, CompilerId(..)
+         , unknownCompilerInfo, AbiTag(..) )
 import Distribution.System
          ( OS(..), Arch(..), buildPlatform )
 import Distribution.License
@@ -102,7 +75,8 @@ import Text.PrettyPrint ((<>), (<+>))
 
 import qualified Language.Haskell.Extension as Extension (deprecatedExtensions)
 import Language.Haskell.Extension
-         ( Language(UnknownLanguage), knownLanguages, Extension(..), KnownExtension(..) )
+         ( Language(UnknownLanguage), knownLanguages
+         , Extension(..), KnownExtension(..) )
 import System.FilePath
          ( (</>), takeExtension, isRelative, isAbsolute
          , splitDirectories,  splitPath )
@@ -128,13 +102,13 @@ data PackageCheck =
      | PackageBuildWarning { explanation :: String }
 
        -- | An issue that might not be a problem for the package author but
-       -- might be annoying or determental when the package is distributed to
+       -- might be annoying or detrimental when the package is distributed to
        -- users. We should encourage distributed packages to be free from these
        -- issues, but occasionally there are justifiable reasons so we cannot
        -- ban them entirely.
      | PackageDistSuspicious { explanation :: String }
 
-       -- | An issue that is ok in the author's environment but is almost
+       -- | An issue that is OK in the author's environment but is almost
        -- certain to be a portability problem for other environments. We can
        -- quite legitimately refuse to publicly distribute packages with these
        -- problems.
@@ -148,7 +122,8 @@ check :: Bool -> PackageCheck -> Maybe PackageCheck
 check False _  = Nothing
 check True  pc = Just pc
 
-checkSpecVersion :: PackageDescription -> [Int] -> Bool -> PackageCheck -> Maybe PackageCheck
+checkSpecVersion :: PackageDescription -> [Int] -> Bool -> PackageCheck
+                 -> Maybe PackageCheck
 checkSpecVersion pkg specver cond pc
   | specVersion pkg >= Version specver [] = Nothing
   | otherwise                             = check cond pc
@@ -160,7 +135,7 @@ checkSpecVersion pkg specver cond pc
 
 -- | Check for common mistakes and problems in package descriptions.
 --
--- This is the standard collection of checks covering all apsects except
+-- This is the standard collection of checks covering all aspects except
 -- for checks that require looking at files within the package. For those
 -- see 'checkPackageFiles'.
 --
@@ -218,7 +193,8 @@ checkSanity pkg =
         ++ ". The name of every executable, test suite, and benchmark section in"
         ++ " the package must be unique."
   ]
-  --TODO: check for name clashes case insensitively: windows file systems cannot cope.
+  --TODO: check for name clashes case insensitively: windows file systems cannot
+  --cope.
 
   ++ maybe []  (checkLibrary    pkg) (library pkg)
   ++ concatMap (checkExecutable pkg) (executables pkg)
@@ -240,17 +216,34 @@ checkSanity pkg =
     duplicateNames = dups $ exeNames ++ testNames ++ bmNames
 
 checkLibrary :: PackageDescription -> Library -> [PackageCheck]
-checkLibrary _pkg lib =
+checkLibrary pkg lib =
   catMaybes [
 
     check (not (null moduleDuplicates)) $
-       PackageBuildWarning $
+       PackageBuildImpossible $
             "Duplicate modules in library: "
          ++ commaSep (map display moduleDuplicates)
+
+    -- check use of required-signatures/exposed-signatures sections
+  , checkVersion [1,21] (not (null (requiredSignatures lib))) $
+      PackageDistInexcusable $
+           "To use the 'required-signatures' field the package needs to specify "
+        ++ "at least 'cabal-version: >= 1.21'."
+
+  , checkVersion [1,21] (not (null (exposedSignatures lib))) $
+      PackageDistInexcusable $
+           "To use the 'exposed-signatures' field the package needs to specify "
+        ++ "at least 'cabal-version: >= 1.21'."
   ]
 
   where
-    moduleDuplicates = dups (libModules lib)
+    checkVersion :: [Int] -> Bool -> PackageCheck -> Maybe PackageCheck
+    checkVersion ver cond pc
+      | specVersion pkg >= Version ver []      = Nothing
+      | otherwise                              = check cond pc
+
+    moduleDuplicates = dups (libModules lib ++
+                             map moduleReexportName (reexportedModules lib))
 
 checkExecutable :: PackageDescription -> Executable -> [PackageCheck]
 checkExecutable pkg exe =
@@ -275,7 +268,7 @@ checkExecutable pkg exe =
         ++ "To use this feature you must specify 'cabal-version: >= 1.18'."
 
   , check (not (null moduleDuplicates)) $
-       PackageBuildWarning $
+       PackageBuildImpossible $
             "Duplicate modules in executable '" ++ exeName exe ++ "': "
          ++ commaSep (map display moduleDuplicates)
   ]
@@ -301,7 +294,7 @@ checkTestSuite pkg test =
       _ -> Nothing
 
   , check (not $ null moduleDuplicates) $
-      PackageBuildWarning $
+      PackageBuildImpossible $
            "Duplicate modules in test suite '" ++ testName test ++ "': "
         ++ commaSep (map display moduleDuplicates)
 
@@ -359,7 +352,7 @@ checkBenchmark pkg bm =
       _ -> Nothing
 
   , check (not $ null moduleDuplicates) $
-      PackageBuildWarning $
+      PackageBuildImpossible $
            "Duplicate modules in benchmark '" ++ benchmarkName bm ++ "': "
         ++ commaSep (map display moduleDuplicates)
 
@@ -458,7 +451,7 @@ checkFields pkg =
   , check (null (synopsis pkg) && not (null (description pkg))) $
       PackageDistSuspicious "No 'synopsis' field."
 
-    --TODO: recommend the bug reports url, author and homepage fields
+    --TODO: recommend the bug reports URL, author and homepage fields
     --TODO: recommend not using the stability field
     --TODO: recommend specifying a source repo
 
@@ -502,10 +495,13 @@ checkLicense :: PackageDescription -> [PackageCheck]
 checkLicense pkg =
   catMaybes [
 
-    check (license pkg == AllRightsReserved) $
+    check (license pkg == UnspecifiedLicense) $
       PackageDistInexcusable
-        "The 'license' field is missing or specified as AllRightsReserved."
+        "The 'license' field is missing."
 
+  , check (license pkg == AllRightsReserved) $
+      PackageDistSuspicious
+        "The 'license' is AllRightsReserved. Is that really what you want?"
   , case license pkg of
       UnknownLicense l -> Just $
         PackageBuildWarning $
@@ -530,10 +526,11 @@ checkLicense pkg =
           ++ "version then please file a ticket."
       _ -> Nothing
 
-  , check (license pkg `notElem` [AllRightsReserved, PublicDomain]
+  , check (license pkg `notElem` [ AllRightsReserved
+                                 , UnspecifiedLicense, PublicDomain]
            -- AllRightsReserved and PublicDomain are not strictly
            -- licenses so don't need license files.
-        && null (licenseFile pkg)) $
+        && null (licenseFiles pkg)) $
       PackageDistSuspicious "A 'license-file' is not specified."
   ]
   where
@@ -592,7 +589,7 @@ checkGhcOptions pkg =
   catMaybes [
 
     check has_WerrorWall $
-      PackageDistInexcusable $
+      PackageDistSuspicious $
            "'ghc-options: -Wall -Werror' makes the package very easy to "
         ++ "break with future GHC versions because new GHC versions often "
         ++ "add new warnings. Use just 'ghc-options: -Wall' instead."
@@ -624,9 +621,12 @@ checkGhcOptions pkg =
       PackageDistInexcusable $
         "'ghc-options: -fhpc' is not appropriate for a distributed package."
 
-  , check (any ("-d" `isPrefixOf`) all_ghc_options) $
+    -- -dynamic is not a debug flag
+  , check (any (\opt -> "-d" `isPrefixOf` opt && opt /= "-dynamic")
+           all_ghc_options) $
       PackageDistInexcusable $
-        "'ghc-options: -d*' debug flags are not appropriate for a distributed package."
+        "'ghc-options: -d*' debug flags are not appropriate "
+        ++ "for a distributed package."
 
   , checkFlags ["-prof"] $
       PackageBuildWarning $
@@ -636,37 +636,43 @@ checkGhcOptions pkg =
 
   , checkFlags ["-o"] $
       PackageBuildWarning $
-        "'ghc-options: -o' is not needed. The output files are named automatically."
+           "'ghc-options: -o' is not needed. "
+        ++ "The output files are named automatically."
 
   , checkFlags ["-hide-package"] $
       PackageBuildWarning $
-           "'ghc-options: -hide-package' is never needed. Cabal hides all packages."
+      "'ghc-options: -hide-package' is never needed. "
+      ++ "Cabal hides all packages."
 
   , checkFlags ["--make"] $
       PackageBuildWarning $
-        "'ghc-options: --make' is never needed. Cabal uses this automatically."
+      "'ghc-options: --make' is never needed. Cabal uses this automatically."
 
   , checkFlags ["-main-is"] $
       PackageDistSuspicious $
-           "'ghc-options: -main-is' is not portable."
+      "'ghc-options: -main-is' is not portable."
 
   , checkFlags ["-O0", "-Onot"] $
       PackageDistSuspicious $
-        "'ghc-options: -O0' is not needed. Use the --disable-optimization configure flag."
+      "'ghc-options: -O0' is not needed. "
+      ++ "Use the --disable-optimization configure flag."
 
   , checkFlags [ "-O", "-O1"] $
       PackageDistInexcusable $
-           "'ghc-options: -O' is not needed. Cabal automatically adds the '-O' flag. "
-        ++ "Setting it yourself interferes with the --disable-optimization flag."
+      "'ghc-options: -O' is not needed. "
+      ++ "Cabal automatically adds the '-O' flag. "
+      ++ "Setting it yourself interferes with the --disable-optimization flag."
 
   , checkFlags ["-O2"] $
       PackageDistSuspicious $
-           "'ghc-options: -O2' is rarely needed. Check that it is giving a real benefit "
-        ++ "and not just imposing longer compile times on your users."
+      "'ghc-options: -O2' is rarely needed. "
+      ++ "Check that it is giving a real benefit "
+      ++ "and not just imposing longer compile times on your users."
 
   , checkFlags ["-split-objs"] $
       PackageBuildWarning $
-        "'ghc-options: -split-objs' is not needed. Use the --enable-split-objs configure flag."
+        "'ghc-options: -split-objs' is not needed. "
+        ++ "Use the --enable-split-objs configure flag."
 
   , checkFlags ["-optl-Wl,-s", "-optl-s"] $
       PackageDistInexcusable $
@@ -678,7 +684,18 @@ checkGhcOptions pkg =
 
   , checkFlags ["-fglasgow-exts"] $
       PackageDistSuspicious $
-        "Instead of 'ghc-options: -fglasgow-exts' it is preferable to use the 'extensions' field."
+        "Instead of 'ghc-options: -fglasgow-exts' it is preferable to use "
+        ++ "the 'extensions' field."
+
+  , checkProfFlags ["-auto-all"] $
+      PackageDistSuspicious $
+        "'ghc-prof-options: -auto-all' is fine during development, but "
+        ++ "not recommended in a distributed package. "
+
+  , checkProfFlags ["-fprof-auto"] $
+      PackageDistSuspicious $
+        "'ghc-prof-options: -fprof-auto' is fine during development, but "
+        ++ "not recommended in a distributed package. "
 
   , check ("-threaded" `elem` lib_ghc_options) $
       PackageDistSuspicious $
@@ -712,49 +729,57 @@ checkGhcOptions pkg =
                            && ("-Wall"   `elem` opts || "-W" `elem` opts)
     has_Werror     = any (\opts -> "-Werror" `elem` opts) ghc_options
 
-    ghc_options = [ strs | bi <- allBuildInfo pkg
-                         , (GHC, strs) <- options bi ]
-    all_ghc_options = concat ghc_options
+    (ghc_options, ghc_prof_options) =
+      unzip . map (\bi -> (hcOptions GHC bi, hcProfOptions GHC bi))
+      $ (allBuildInfo pkg)
+    all_ghc_options      = concat ghc_options
+    all_ghc_prof_options = concat ghc_prof_options
     lib_ghc_options = maybe [] (hcOptions GHC . libBuildInfo) (library pkg)
 
-    checkFlags :: [String] -> PackageCheck -> Maybe PackageCheck
-    checkFlags flags = check (any (`elem` flags) all_ghc_options)
+    checkFlags,checkProfFlags :: [String] -> PackageCheck -> Maybe PackageCheck
+    checkFlags     flags = doCheckFlags flags all_ghc_options
+    checkProfFlags flags = doCheckFlags flags all_ghc_prof_options
+
+    doCheckFlags   flags opts = check (any (`elem` flags) opts)
 
     ghcExtension ('-':'f':name) = case name of
-      "allow-overlapping-instances"    -> Just (EnableExtension  OverlappingInstances)
-      "no-allow-overlapping-instances" -> Just (DisableExtension OverlappingInstances)
-      "th"                             -> Just (EnableExtension  TemplateHaskell)
-      "no-th"                          -> Just (DisableExtension TemplateHaskell)
-      "ffi"                            -> Just (EnableExtension  ForeignFunctionInterface)
-      "no-ffi"                         -> Just (DisableExtension ForeignFunctionInterface)
-      "fi"                             -> Just (EnableExtension  ForeignFunctionInterface)
-      "no-fi"                          -> Just (DisableExtension ForeignFunctionInterface)
-      "monomorphism-restriction"       -> Just (EnableExtension  MonomorphismRestriction)
-      "no-monomorphism-restriction"    -> Just (DisableExtension MonomorphismRestriction)
-      "mono-pat-binds"                 -> Just (EnableExtension  MonoPatBinds)
-      "no-mono-pat-binds"              -> Just (DisableExtension MonoPatBinds)
-      "allow-undecidable-instances"    -> Just (EnableExtension  UndecidableInstances)
-      "no-allow-undecidable-instances" -> Just (DisableExtension UndecidableInstances)
-      "allow-incoherent-instances"     -> Just (EnableExtension  IncoherentInstances)
-      "no-allow-incoherent-instances"  -> Just (DisableExtension IncoherentInstances)
-      "arrows"                         -> Just (EnableExtension  Arrows)
-      "no-arrows"                      -> Just (DisableExtension Arrows)
-      "generics"                       -> Just (EnableExtension  Generics)
-      "no-generics"                    -> Just (DisableExtension Generics)
-      "implicit-prelude"               -> Just (EnableExtension  ImplicitPrelude)
-      "no-implicit-prelude"            -> Just (DisableExtension ImplicitPrelude)
-      "implicit-params"                -> Just (EnableExtension  ImplicitParams)
-      "no-implicit-params"             -> Just (DisableExtension ImplicitParams)
-      "bang-patterns"                  -> Just (EnableExtension  BangPatterns)
-      "no-bang-patterns"               -> Just (DisableExtension BangPatterns)
-      "scoped-type-variables"          -> Just (EnableExtension  ScopedTypeVariables)
-      "no-scoped-type-variables"       -> Just (DisableExtension ScopedTypeVariables)
-      "extended-default-rules"         -> Just (EnableExtension  ExtendedDefaultRules)
-      "no-extended-default-rules"      -> Just (DisableExtension ExtendedDefaultRules)
+      "allow-overlapping-instances"    -> enable  OverlappingInstances
+      "no-allow-overlapping-instances" -> disable OverlappingInstances
+      "th"                             -> enable  TemplateHaskell
+      "no-th"                          -> disable TemplateHaskell
+      "ffi"                            -> enable  ForeignFunctionInterface
+      "no-ffi"                         -> disable ForeignFunctionInterface
+      "fi"                             -> enable  ForeignFunctionInterface
+      "no-fi"                          -> disable ForeignFunctionInterface
+      "monomorphism-restriction"       -> enable  MonomorphismRestriction
+      "no-monomorphism-restriction"    -> disable MonomorphismRestriction
+      "mono-pat-binds"                 -> enable  MonoPatBinds
+      "no-mono-pat-binds"              -> disable MonoPatBinds
+      "allow-undecidable-instances"    -> enable  UndecidableInstances
+      "no-allow-undecidable-instances" -> disable UndecidableInstances
+      "allow-incoherent-instances"     -> enable  IncoherentInstances
+      "no-allow-incoherent-instances"  -> disable IncoherentInstances
+      "arrows"                         -> enable  Arrows
+      "no-arrows"                      -> disable Arrows
+      "generics"                       -> enable  Generics
+      "no-generics"                    -> disable Generics
+      "implicit-prelude"               -> enable  ImplicitPrelude
+      "no-implicit-prelude"            -> disable ImplicitPrelude
+      "implicit-params"                -> enable  ImplicitParams
+      "no-implicit-params"             -> disable ImplicitParams
+      "bang-patterns"                  -> enable  BangPatterns
+      "no-bang-patterns"               -> disable BangPatterns
+      "scoped-type-variables"          -> enable  ScopedTypeVariables
+      "no-scoped-type-variables"       -> disable ScopedTypeVariables
+      "extended-default-rules"         -> enable  ExtendedDefaultRules
+      "no-extended-default-rules"      -> disable ExtendedDefaultRules
       _                                -> Nothing
-    ghcExtension "-cpp"             = Just (EnableExtension CPP)
+    ghcExtension "-cpp"             = enable CPP
     ghcExtension _                  = Nothing
 
+    enable  e = Just (EnableExtension e)
+    disable e = Just (DisableExtension e)
+
 checkCCOptions :: PackageDescription -> [PackageCheck]
 checkCCOptions pkg =
   catMaybes [
@@ -859,6 +884,7 @@ checkPaths pkg =
       ++ [ (path, "data-dir")        | path <- [dataDir      pkg]]
       ++ concat
          [    [ (path, "c-sources")        | path <- cSources        bi ]
+           ++ [ (path, "js-sources")       | path <- jsSources       bi ]
            ++ [ (path, "install-includes") | path <- installIncludes bi ]
            ++ [ (path, "hs-source-dirs")   | path <- hsSourceDirs    bi ]
          | bi <- allBuildInfo pkg ]
@@ -869,7 +895,7 @@ checkPaths pkg =
         ++ [ (path, "extra-lib-dirs")   | path <- extraLibDirs    bi ]
       | bi <- allBuildInfo pkg ]
 
---TODO: check sets of paths that would be interpreted differently between unix
+--TODO: check sets of paths that would be interpreted differently between Unix
 -- and windows, ie case-sensitive or insensitive. Things that might clash, or
 -- conversely be distinguished.
 
@@ -931,6 +957,22 @@ checkCabalVersion pkg =
         ++ "different modules then list the other ones in the "
         ++ "'other-languages' field."
 
+    -- check use of reexported-modules sections
+  , checkVersion [1,21]
+    (maybe False (not.null.reexportedModules) (library pkg)) $
+      PackageDistInexcusable $
+           "To use the 'reexported-module' field the package needs to specify "
+        ++ "at least 'cabal-version: >= 1.21'."
+
+    -- check use of thinning and renaming
+  , checkVersion [1,21] (not (null depsUsingThinningRenamingSyntax)) $
+      PackageDistInexcusable $
+           "The package uses "
+        ++ "thinning and renaming in the 'build-depends' field: "
+        ++ commaSep (map display depsUsingThinningRenamingSyntax)
+        ++ ". To use this new syntax, the package needs to specify at least"
+        ++ "'cabal-version: >= 1.21'."
+
     -- check use of default-extensions field
     -- don't need to do the equivalent check for other-extensions
   , checkVersion [1,10] (any (not . null) (buildInfoField defaultExtensions)) $
@@ -965,7 +1007,7 @@ checkCabalVersion pkg =
            "The package uses wildcard syntax in the 'build-depends' field: "
         ++ commaSep (map display depsUsingWildcardSyntax)
         ++ ". To use this new syntax the package need to specify at least "
-        ++ "'cabal-version: >= 1.6'. Alternatively, if broader compatability "
+        ++ "'cabal-version: >= 1.6'. Alternatively, if broader compatibility "
         ++ "is important then use: " ++ commaSep
            [ display (Dependency name (eliminateWildcardSyntax versionRange))
            | Dependency name versionRange <- depsUsingWildcardSyntax ]
@@ -985,7 +1027,7 @@ checkCabalVersion pkg =
            "The package uses wildcard syntax in the 'tested-with' field: "
         ++ commaSep (map display testedWithUsingWildcardSyntax)
         ++ ". To use this new syntax the package need to specify at least "
-        ++ "'cabal-version: >= 1.6'. Alternatively, if broader compatability "
+        ++ "'cabal-version: >= 1.6'. Alternatively, if broader compatibility "
         ++ "is important then use: " ++ commaSep
            [ display (Dependency name (eliminateWildcardSyntax versionRange))
            | Dependency name versionRange <- testedWithUsingWildcardSyntax ]
@@ -996,7 +1038,7 @@ checkCabalVersion pkg =
            "Using wildcards like "
         ++ commaSep (map quote $ take 3 dataFilesUsingGlobSyntax)
         ++ " in the 'data-files' field requires 'cabal-version: >= 1.6'. "
-        ++ "Alternatively if you require compatability with earlier Cabal "
+        ++ "Alternatively if you require compatibility with earlier Cabal "
         ++ "versions then list all the files explicitly."
 
     -- check use of "extra-source-files: mk/*.in" syntax
@@ -1006,7 +1048,7 @@ checkCabalVersion pkg =
         ++ commaSep (map quote $ take 3 extraSrcFilesUsingGlobSyntax)
         ++ " in the 'extra-source-files' field requires "
         ++ "'cabal-version: >= 1.6'. Alternatively if you require "
-        ++ "compatability with earlier Cabal versions then list all the files "
+        ++ "compatibility with earlier Cabal versions then list all the files "
         ++ "explicitly."
 
     -- check use of "source-repository" section
@@ -1022,7 +1064,7 @@ checkCabalVersion pkg =
            "Unfortunately the license " ++ quote (display (license pkg))
         ++ " messes up the parser in earlier Cabal versions so you need to "
         ++ "specify 'cabal-version: >= 1.4'. Alternatively if you require "
-        ++ "compatability with earlier Cabal versions then use 'OtherLicense'."
+        ++ "compatibility with earlier Cabal versions then use 'OtherLicense'."
 
     -- check for new language extensions
   , checkVersion [1,2,3] (not (null mentionedExtensionsThatNeedCabal12)) $
@@ -1031,7 +1073,7 @@ checkCabalVersion pkg =
         ++ commaSep (map (quote . display) mentionedExtensionsThatNeedCabal12)
         ++ " break the parser in earlier Cabal versions so you need to "
         ++ "specify 'cabal-version: >= 1.2.3'. Alternatively if you require "
-        ++ "compatability with earlier Cabal versions then you may be able to "
+        ++ "compatibility with earlier Cabal versions then you may be able to "
         ++ "use an equivalent compiler-specific flag."
 
   , checkVersion [1,4] (not (null mentionedExtensionsThatNeedCabal14)) $
@@ -1040,7 +1082,7 @@ checkCabalVersion pkg =
         ++ commaSep (map (quote . display) mentionedExtensionsThatNeedCabal14)
         ++ " break the parser in earlier Cabal versions so you need to "
         ++ "specify 'cabal-version: >= 1.4'. Alternatively if you require "
-        ++ "compatability with earlier Cabal versions then you may be able to "
+        ++ "compatibility with earlier Cabal versions then you may be able to "
         ++ "use an equivalent compiler-specific flag."
   ]
   where
@@ -1101,9 +1143,19 @@ checkCabalVersion pkg =
     depsUsingWildcardSyntax = [ dep | dep@(Dependency _ vr) <- buildDepends pkg
                                     , usesWildcardSyntax vr ]
 
-    testedWithUsingWildcardSyntax = [ Dependency (PackageName (display compiler)) vr
-                                    | (compiler, vr) <- testedWith pkg
-                                    , usesWildcardSyntax vr ]
+    -- XXX: If the user writes build-depends: foo with (), this is
+    -- indistinguishable from build-depends: foo, so there won't be an
+    -- error even though there should be
+    depsUsingThinningRenamingSyntax =
+      [ name
+      | bi <- allBuildInfo pkg
+      , (name, rns) <- Map.toList (targetBuildRenaming bi)
+      , rns /= ModuleRenaming True [] ]
+
+    testedWithUsingWildcardSyntax =
+      [ Dependency (PackageName (display compiler)) vr
+      | (compiler, vr) <- testedWith pkg
+      , usesWildcardSyntax vr ]
 
     usesWildcardSyntax :: VersionRange -> Bool
     usesWildcardSyntax =
@@ -1123,7 +1175,8 @@ checkCabalVersion pkg =
         intersectVersionRanges unionVersionRanges id
 
     compatLicenses = [ GPL Nothing, LGPL Nothing, AGPL Nothing, BSD3, BSD4
-                     , PublicDomain, AllRightsReserved, OtherLicense ]
+                     , PublicDomain, AllRightsReserved
+                     , UnspecifiedLicense, OtherLicense ]
 
     mentionedExtensions = [ ext | bi <- allBuildInfo pkg
                                 , ext <- allExtensions bi ]
@@ -1168,7 +1221,7 @@ checkCabalVersion pkg =
 
 -- | A variation on the normal 'Text' instance, shows any ()'s in the original
 -- textual syntax. We need to show these otherwise it's confusing to users when
--- we complain of their presense but do not pretty print them!
+-- we complain of their presence but do not pretty print them!
 --
 displayRawVersionRange :: VersionRange -> String
 displayRawVersionRange =
@@ -1183,8 +1236,10 @@ displayRawVersionRange =
      (\v   -> (Disp.text ">=" <> disp v                   , 0))
      (\v   -> (Disp.text "<=" <> disp v                   , 0))
      (\v _ -> (Disp.text "==" <> dispWild v               , 0))
-     (\(r1, p1) (r2, p2) -> (punct 2 p1 r1 <+> Disp.text "||" <+> punct 2 p2 r2 , 2))
-     (\(r1, p1) (r2, p2) -> (punct 1 p1 r1 <+> Disp.text "&&" <+> punct 1 p2 r2 , 1))
+     (\(r1, p1) (r2, p2) ->
+       (punct 2 p1 r1 <+> Disp.text "||" <+> punct 2 p2 r2 , 2))
+     (\(r1, p1) (r2, p2) ->
+       (punct 1 p1 r1 <+> Disp.text "&&" <+> punct 1 p2 r2 , 1))
      (\(r,  _ )          -> (Disp.parens r, 0)) -- parens
 
   where
@@ -1237,7 +1292,8 @@ checkPackageVersions pkg =
     -- using no package index and the current platform.
     finalised = finalizePackageDescription
                               [] (const True) buildPlatform
-                              (CompilerId buildCompilerFlavor (Version [] []))
+                              (unknownCompilerInfo
+                                (CompilerId buildCompilerFlavor (Version [] [])) NoAbiTag)
                               [] pkg
     baseDependency = case finalised of
       Right (pkg', _) | not (null baseDeps) ->
@@ -1298,7 +1354,7 @@ checkConditionals pkg =
 -- ------------------------------------------------------------
 
 -- | Sanity check things that requires IO. It looks at the files in the
--- package and expects to find the package unpacked in at the given filepath.
+-- package and expects to find the package unpacked in at the given file path.
 --
 checkPackageFiles :: PackageDescription -> FilePath -> IO [PackageCheck]
 checkPackageFiles pkg root = checkPackageContent checkFilesIO pkg
@@ -1328,30 +1384,30 @@ checkPackageContent :: Monad m => CheckPackageContentOps m
                     -> PackageDescription
                     -> m [PackageCheck]
 checkPackageContent ops pkg = do
-  licenseError    <- checkLicenseExists   ops pkg
+  licenseErrors   <- checkLicensesExist   ops pkg
   setupError      <- checkSetupExists     ops pkg
   configureError  <- checkConfigureExists ops pkg
   localPathErrors <- checkLocalPathsExist ops pkg
   vcsLocation     <- checkMissingVcsInfo  ops pkg
 
-  return $ catMaybes [licenseError, setupError, configureError]
+  return $ licenseErrors
+        ++ catMaybes [setupError, configureError]
         ++ localPathErrors
         ++ vcsLocation
 
-checkLicenseExists :: Monad m => CheckPackageContentOps m
+checkLicensesExist :: Monad m => CheckPackageContentOps m
                    -> PackageDescription
-                   -> m (Maybe PackageCheck)
-checkLicenseExists ops pkg
-  | null (licenseFile pkg) = return Nothing
-  | otherwise = do
-    exists <- doesFileExist ops file
-    return $ check (not exists) $
-      PackageBuildWarning $
-           "The 'license-file' field refers to the file " ++ quote file
-        ++ " which does not exist."
-
+                   -> m [PackageCheck]
+checkLicensesExist ops pkg = do
+    exists <- mapM (doesFileExist ops) (licenseFiles pkg)
+    return
+      [ PackageBuildWarning $
+           "The '" ++ fieldname ++ "' field refers to the file "
+        ++ quote file ++ " which does not exist."
+      | (file, False) <- zip (licenseFiles pkg) exists ]
   where
-    file = licenseFile pkg
+    fieldname | length (licenseFiles pkg) == 1 = "license-file"
+              | otherwise                      = "license-files"
 
 checkSetupExists :: Monad m => CheckPackageContentOps m
                  -> PackageDescription
@@ -1370,7 +1426,8 @@ checkConfigureExists ops PackageDescription { buildType = Just Configure } = do
   exists <- doesFileExist ops "configure"
   return $ check (not exists) $
     PackageBuildWarning $
-      "The 'build-type' is 'Configure' but there is no 'configure' script."
+      "The 'build-type' is 'Configure' but there is no 'configure' script. "
+      ++ "You probably need to run 'autoreconf -i' to generate it."
 checkConfigureExists _ _ = return Nothing
 
 checkLocalPathsExist :: Monad m => CheckPackageContentOps m
