@@ -1,9 +1,11 @@
+{-# LANGUAGE CPP #-}
 -- -fno-warn-deprecations for use of Map.foldWithKey
 {-# OPTIONS_GHC -fno-warn-deprecations #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Distribution.PackageDescription.Configuration
 -- Copyright   :  Thomas Schilling, 2007
+-- License     :  BSD3
 --
 -- Maintainer  :  cabal-devel@haskell.org
 -- Portability :  portable
@@ -14,36 +16,6 @@
 -- 'PackageDescription's. It has code for working with the tree of conditions
 -- and resolving or flattening conditions.
 
-{- All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions are
-met:
-
-    * Redistributions of source code must retain the above copyright
-      notice, this list of conditions and the following disclaimer.
-
-    * Redistributions in binary form must reproduce the above
-      copyright notice, this list of conditions and the following
-      disclaimer in the documentation and/or other materials provided
-      with the distribution.
-
-    * Neither the name of Isaac Jones nor the names of other
-      contributors may be used to endorse or promote products derived
-      from this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. -}
-
 module Distribution.PackageDescription.Configuration (
     finalizePackageDescription,
     flattenPackageDescription,
@@ -75,6 +47,8 @@ import Distribution.System
          ( Platform(..), OS, Arch )
 import Distribution.Simple.Utils
          ( currentDir, lowercase )
+import Distribution.Simple.Compiler
+         ( CompilerInfo(..) )
 
 import Distribution.Text
          ( Text(parse) )
@@ -86,7 +60,9 @@ import Data.Char ( isAlphaNum )
 import Data.Maybe ( catMaybes, maybeToList )
 import Data.Map ( Map, fromListWith, toList )
 import qualified Data.Map as Map
+#if __GLASGOW_HASKELL__ < 710
 import Data.Monoid
+#endif
 
 ------------------------------------------------------------------------------
 
@@ -124,18 +100,25 @@ simplifyCondition cond i = fv . walk $ cond
       COr c1 c2  -> fv' c1 ++ fv' c2
       CAnd c1 c2 -> fv' c1 ++ fv' c2
 
--- | Simplify a configuration condition using the os and arch names.  Returns
+-- | Simplify a configuration condition using the OS and arch names.  Returns
 --   the names of all the flags occurring in the condition.
-simplifyWithSysParams :: OS -> Arch -> CompilerId -> Condition ConfVar
+simplifyWithSysParams :: OS -> Arch -> CompilerInfo -> Condition ConfVar
                       -> (Condition FlagName, [FlagName])
-simplifyWithSysParams os arch (CompilerId comp compVer) cond = (cond', flags)
+simplifyWithSysParams os arch cinfo cond = (cond', flags)
   where
     (cond', flags) = simplifyCondition cond interp
     interp (OS os')    = Right $ os' == os
     interp (Arch arch') = Right $ arch' == arch
-    interp (Impl comp' vr) = Right $ comp' == comp
-                                  && compVer `withinRange` vr
-    interp (Flag  f)   = Left f
+    interp (Impl comp vr)
+      | matchImpl (compilerInfoId cinfo) = Right True
+      | otherwise = case compilerInfoCompat cinfo of
+          -- fixme: treat Nothing as unknown, rather than empty list once we
+          --        support partial resolution of system parameters
+          Nothing     -> Right False
+          Just compat -> Right (any matchImpl compat)
+          where
+            matchImpl (CompilerId c v) = comp == c && v `withinRange` vr
+    interp (Flag f) = Left f
 
 -- TODO: Add instances and check
 --
@@ -237,7 +220,7 @@ resolveWithFlags ::
         -- ^ Domain for each flag name, will be tested in order.
   -> OS      -- ^ OS as returned by Distribution.System.buildOS
   -> Arch    -- ^ Arch as returned by Distribution.System.buildArch
-  -> CompilerId -- ^ Compiler flavour + version
+  -> CompilerInfo  -- ^ Compiler information
   -> [Dependency]  -- ^ Additional constraints
   -> [CondTree ConfVar [Dependency] PDTagged]
   -> ([Dependency] -> DepTestRslt [Dependency])  -- ^ Dependency test function.
@@ -488,17 +471,19 @@ instance Monoid PDTagged where
 --
 finalizePackageDescription ::
      FlagAssignment  -- ^ Explicitly specified flag assignments
-  -> (Dependency -> Bool) -- ^ Is a given depenency satisfiable from the set of available packages?
-                          -- If this is unknown then use True.
+  -> (Dependency -> Bool) -- ^ Is a given dependency satisfiable from the set of
+                          -- available packages?  If this is unknown then use
+                          -- True.
   -> Platform      -- ^ The 'Arch' and 'OS'
-  -> CompilerId    -- ^ Compiler + Version
+  -> CompilerInfo  -- ^ Compiler information
   -> [Dependency]  -- ^ Additional constraints
   -> GenericPackageDescription
   -> Either [Dependency]
             (PackageDescription, FlagAssignment)
              -- ^ Either missing dependencies or the resolved package
              -- description along with the flag assignments chosen.
-finalizePackageDescription userflags satisfyDep (Platform arch os) impl constraints
+finalizePackageDescription userflags satisfyDep
+        (Platform arch os) impl constraints
         (GenericPackageDescription pkg flags mlib0 exes0 tests0 bms0) =
     case resolveFlags of
       Right ((mlib, exes', tests', bms'), targetSet, flagVals) ->
@@ -540,9 +525,10 @@ finalizePackageDescription userflags satisfyDep (Platform arch os) impl constrai
                       | manual -> [b]
                       | otherwise -> [b, not b]
     --flagDefaults = map (\(n,x:_) -> (n,x)) flagChoices
-    check ds     = if all satisfyDep ds
-                   then DepOk
-                   else MissingDeps $ filter (not . satisfyDep) ds
+    check ds     = let missingDeps = filter (not . satisfyDep) ds
+                   in if null missingDeps
+                      then DepOk
+                      else MissingDeps missingDeps
 
 {-
 let tst_p = (CondNode [1::Int] [Distribution.Package.Dependency "a" AnyVersion] [])
