@@ -1,44 +1,17 @@
 {-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE DeriveGeneric #-}
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Language.Haskell.Extension
 -- Copyright   :  Isaac Jones 2003-2004
+-- License     :  BSD3
 --
 -- Maintainer  :  libraries@haskell.org
 -- Portability :  portable
 --
 -- Haskell language dialects and extensions
 
-{- All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions are
-met:
-
-    * Redistributions of source code must retain the above copyright
-      notice, this list of conditions and the following disclaimer.
-
-    * Redistributions in binary form must reproduce the above
-      copyright notice, this list of conditions and the following
-      disclaimer in the documentation and/or other materials provided
-      with the distribution.
-
-    * Neither the name of Isaac Jones nor the names of other
-      contributors may be used to endorse or promote products derived
-      from this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. -}
-
 module Language.Haskell.Extension (
         Language(..),
         knownLanguages,
@@ -54,8 +27,10 @@ import qualified Distribution.Compat.ReadP as Parse
 import qualified Text.PrettyPrint as Disp
 import qualified Data.Char as Char (isAlphaNum)
 import Data.Array (Array, accumArray, bounds, Ix(inRange), (!))
+import Distribution.Compat.Binary (Binary)
 import Data.Data (Data)
 import Data.Typeable (Typeable)
+import GHC.Generics (Generic)
 
 -- ------------------------------------------------------------
 -- * Language
@@ -78,7 +53,9 @@ data Language =
 
   -- | An unknown language, identified by its name.
   | UnknownLanguage String
-  deriving (Show, Read, Eq, Typeable, Data)
+  deriving (Generic, Show, Read, Eq, Typeable, Data)
+
+instance Binary Language
 
 knownLanguages :: [Language]
 knownLanguages = [Haskell98, Haskell2010]
@@ -106,7 +83,7 @@ classifyLanguage = \str -> case lookup str langTable of
 -- Note: if you add a new 'KnownExtension':
 --
 -- * also add it to the Distribution.Simple.X.languageExtensions lists
---   (where X is each compiler: GHC, JHC, Hugs, NHC)
+--   (where X is each compiler: GHC, JHC, LHC, UHC, HaskellSuite)
 --
 -- | This represents language extensions beyond a base 'Language' definition
 -- (such as 'Haskell98') that are supported by some implementations, usually
@@ -126,7 +103,9 @@ data Extension =
   -- pragma.
   | UnknownExtension String
 
-  deriving (Show, Read, Eq, Typeable, Data)
+  deriving (Generic, Show, Read, Eq, Ord, Typeable, Data)
+
+instance Binary Extension
 
 data KnownExtension =
 
@@ -152,11 +131,11 @@ data KnownExtension =
   -- * <http://www.haskell.org/ghc/docs/latest/html/users_guide/type-class-extensions.html#instance-overlap>
   | IncoherentInstances
 
-  -- | /(deprecated)/ Allows recursive bindings in @do@ blocks, using the @rec@
+  -- | /(deprecated)/ Allow recursive bindings in @do@ blocks, using the @rec@
   -- keyword. See also 'RecursiveDo'.
   | DoRec
 
-  -- | Allows recursive bindings using @mdo@, a variant of @do@.
+  -- | Allow recursive bindings using @mdo@, a variant of @do@.
   -- @DoRec@ provides a different, preferred syntax.
   --
   -- * <http://www.haskell.org/ghc/docs/latest/html/users_guide/syntax-extns.html#recursive-do-notation>
@@ -533,13 +512,13 @@ data KnownExtension =
   -- and Josef Svenningsson, from ICFP '04.
   | RegularPatterns
 
-  -- | Enables the use of tuple sections, e.g. @(, True)@ desugars into
+  -- | Enable the use of tuple sections, e.g. @(, True)@ desugars into
   -- @\x -> (x, True)@.
   --
   -- * <http://www.haskell.org/ghc/docs/latest/html/users_guide/syntax-extns.html#tuple-sections>
   | TupleSections
 
-  -- | Allows GHC primops, written in C--, to be imported into a Haskell
+  -- | Allow GHC primops, written in C--, to be imported into a Haskell
   -- file.
   | GHCForeignImportPrim
 
@@ -657,7 +636,7 @@ data KnownExtension =
   -- * <http://www.haskell.org/haskellwiki/GHC/Data_Parallel_Haskell>
   | ParallelArrays
 
-  -- | Enable explicit role annotations, like in (@data T a\@R@).
+  -- | Enable explicit role annotations, like in (@type role Foo representational representational@).
   --
   -- * <http://www.haskell.org/ghc/docs/latest/html/users_guide/roles.html>
   | RoleAnnotations
@@ -668,7 +647,7 @@ data KnownExtension =
   -- * <http://www.haskell.org/ghc/docs/latest/html/users_guide/type-class-extensions.html#overloaded-lists>
   | OverloadedLists
 
-  -- | Enables case expressions that have no alternatives. Also applies to lambda-case expressions if they are enabled.
+  -- | Enable case expressions that have no alternatives. Also applies to lambda-case expressions if they are enabled.
   --
   -- * <http://www.haskell.org/ghc/docs/latest/html/users_guide/syntax-extns.html#empty-case>
   | EmptyCase
@@ -684,12 +663,17 @@ data KnownExtension =
   -- * <http://www.haskell.org/ghc/docs/latest/html/users_guide/syntax-extns.html#negative-literals>
   | NegativeLiterals
 
-  -- | Allows the use of floating literal syntax for all instances of 'Num', including 'Int' and 'Integer'.
+  -- | Allow the use of binary integer literal syntax (e.g. @0b11001001@ to denote @201@).
+  --
+  -- * <http://www.haskell.org/ghc/docs/latest/html/users_guide/syntax-extns.html#binary-literals>
+  | BinaryLiterals
+
+  -- | Allow the use of floating literal syntax for all instances of 'Num', including 'Int' and 'Integer'.
   --
   -- * <http://www.haskell.org/ghc/docs/latest/html/users_guide/syntax-extns.html#num-decimals>
   | NumDecimals
 
-  -- | Enables support for type classes with no type parameter.
+  -- | Enable support for type classes with no type parameter.
   --
   -- * <http://www.haskell.org/ghc/docs/latest/html/users_guide/type-class-extensions.html#nullary-type-classes>
   | NullaryTypeClasses
@@ -704,7 +688,35 @@ data KnownExtension =
   -- * <http://www.haskell.org/ghc/docs/latest/html/users_guide/other-type-extensions.html#ambiguity>
   | AllowAmbiguousTypes
 
-  deriving (Show, Read, Eq, Enum, Bounded, Typeable, Data)
+  -- | Enable @foreign import javascript@.
+  | JavaScriptFFI
+
+  -- | Allow giving names to and abstracting over patterns.
+  --
+  -- * <http://www.haskell.org/ghc/docs/latest/html/users_guide/other-type-extensions.html#pattern-synonyms>
+  | PatternSynonyms
+
+  -- | Allow anonymous placeholders (underscore) inside type signatures.  The
+  -- type inference engine will generate a message describing the type inferred
+  -- at the hole's location.
+  --
+  -- * <http://www.haskell.org/ghc/docs/latest/html/users_guide/other-type-extensions.html#partial-type-signatures>
+  | PartialTypeSignatures
+
+  -- | Allow named placeholders written with a leading underscore inside type
+  -- signatures.  Wildcards with the same name unify to the same type.
+  --
+  -- * <http://www.haskell.org/ghc/docs/latest/html/users_guide/other-type-extensions.html#named-wildcards>
+  | NamedWildCards
+
+  -- | Enable @deriving@ for any class.
+  --
+  -- * <http://www.haskell.org/ghc/docs/latest/html/users_guide/other-type-extensions.html#derive-any-class>
+  | DeriveAnyClass
+
+  deriving (Generic, Show, Read, Eq, Ord, Enum, Bounded, Typeable, Data)
+
+instance Binary KnownExtension
 
 {-# DEPRECATED knownExtensions
    "KnownExtension is an instance of Enum and Bounded, use those instead." #-}
@@ -721,7 +733,7 @@ deprecatedExtensions =
   ]
 -- NOTE: when adding deprecated extensions that have new alternatives
 -- we must be careful to make sure that the deprecation messages are
--- valid. We must not recomend aliases that cannot be used with older
+-- valid. We must not recommend aliases that cannot be used with older
 -- compilers, perhaps by adding support in Cabal to translate the new
 -- name to the old one for older compilers. Otherwise we are in danger
 -- of the scenario in ticket #689.
