@@ -11,6 +11,6 @@ dir = "PackageTests" </> "BenchmarkExeV10"
 
 checkBenchmark :: FilePath -> Test
 checkBenchmark ghcPath = TestCase $ do
-    let spec = PackageSpec dir ["--enable-benchmarks"]
+    let spec = PackageSpec dir Nothing ["--enable-benchmarks"]
     buildResult <- cabal_build spec ghcPath
     assertBuildSucceeded buildResult
