@@ -8,8 +8,16 @@ import Test.HUnit
 
 suite :: FilePath -> FilePath -> Test
 suite ghcPath ghcPkgPath = TestCase $ do
-    let spec = PackageSpec ("PackageTests" </> "BuildDeps" </> "InternalLibrary3") []
-    let specTI = PackageSpec (directory spec </> "to-install") []
+    let spec = PackageSpec
+            { directory = "PackageTests" </> "BuildDeps" </> "InternalLibrary3"
+            , configOpts = []
+            , distPref = Nothing
+            }
+    let specTI = PackageSpec
+            { directory = directory spec </> "to-install"
+            , configOpts = []
+            , distPref = Nothing
+            }
 
     unregister "InternalLibrary3" ghcPkgPath
     iResult <- cabal_install specTI ghcPath
@@ -18,7 +26,7 @@ suite ghcPath ghcPkgPath = TestCase $ do
     assertBuildSucceeded bResult
     unregister "InternalLibrary3"ghcPkgPath
 
-    (_, _, output) <- run (Just $ directory spec) (directory spec </> "dist" </> "build" </> "lemon" </> "lemon") []
+    (_, _, output) <- run (Just $ directory spec) (directory spec </> "dist" </> "build" </> "lemon" </> "lemon") [] []
     C.appendFile (directory spec </> "test-log.txt") (C.pack $ "\ndist/build/lemon/lemon\n"++output)
     assertEqual "executable should have linked with the internal library" "myLibFunc internal" (concat $ lines output)
 
