@@ -37,7 +37,7 @@ module Distribution.Client.Dependency (
     applySandboxInstallPolicy,
 
     -- ** Extra policy options
-    dontUpgradeBasePackage,
+    dontUpgradeNonUpgradeablePackages,
     hideBrokenInstalledPackages,
     upgradeDependencies,
     reinstallTargets,
@@ -56,6 +56,7 @@ module Distribution.Client.Dependency (
     hideInstalledPackagesSpecificByInstalledPackageId,
     hideInstalledPackagesSpecificBySourcePackageId,
     hideInstalledPackagesAllVersions,
+    removeUpperBounds
   ) where
 
 import Distribution.Client.Dependency.TopDown
@@ -63,6 +64,7 @@ import Distribution.Client.Dependency.TopDown
 import Distribution.Client.Dependency.Modular
          ( modularResolver, SolverConfig(..) )
 import qualified Distribution.Client.PackageIndex as PackageIndex
+import Distribution.Simple.PackageIndex (InstalledPackageIndex)
 import qualified Distribution.Simple.PackageIndex as InstalledPackageIndex
 import qualified Distribution.Client.InstallPlan as InstallPlan
 import Distribution.Client.InstallPlan (InstallPlan)
@@ -71,7 +73,8 @@ import Distribution.Client.Types
          , SourcePackage(..) )
 import Distribution.Client.Dependency.Types
          ( PreSolver(..), Solver(..), DependencyResolver, PackageConstraint(..)
-         , PackagePreferences(..), InstalledPreference(..)
+         , debugPackageConstraint
+         , AllowNewer(..), PackagePreferences(..), InstalledPreference(..)
          , PackagesPreferenceDefault(..)
          , Progress(..), foldProgress )
 import Distribution.Client.Sandbox.Types
@@ -81,11 +84,16 @@ import qualified Distribution.InstalledPackageInfo as Installed
 import Distribution.Package
          ( PackageName(..), PackageId, Package(..), packageName, packageVersion
          , InstalledPackageId, Dependency(Dependency))
+import qualified Distribution.PackageDescription as PD
+         ( PackageDescription(..), GenericPackageDescription(..)
+         , Library(..), Executable(..), TestSuite(..), Benchmark(..), CondTree)
+import Distribution.PackageDescription (BuildInfo(targetBuildDepends))
+import Distribution.PackageDescription.Configuration (mapCondTree)
 import Distribution.Version
          ( Version(..), VersionRange, anyVersion, thisVersion, withinRange
-         , simplifyVersionRange )
+         , removeUpperBound, simplifyVersionRange )
 import Distribution.Compiler
-         ( CompilerId(..), CompilerFlavor(..) )
+         ( CompilerId(..), CompilerInfo(..), CompilerFlavor(..) )
 import Distribution.System
          ( Platform )
 import Distribution.Simple.Utils
@@ -95,7 +103,7 @@ import Distribution.Text
 import Distribution.Verbosity
          ( Verbosity )
 
-import Data.List (maximumBy, foldl')
+import Data.List (maximumBy, foldl', intercalate)
 import Data.Maybe (fromMaybe)
 import qualified Data.Map as Map
 import qualified Data.Set as Set
@@ -114,7 +122,7 @@ data DepResolverParams = DepResolverParams {
        depResolverConstraints       :: [PackageConstraint],
        depResolverPreferences       :: [PackagePreference],
        depResolverPreferenceDefault :: PackagesPreferenceDefault,
-       depResolverInstalledPkgIndex :: InstalledPackageIndex.PackageIndex,
+       depResolverInstalledPkgIndex :: InstalledPackageIndex,
        depResolverSourcePkgIndex    :: PackageIndex.PackageIndex SourcePackage,
        depResolverReorderGoals      :: Bool,
        depResolverIndependentGoals  :: Bool,
@@ -124,6 +132,14 @@ data DepResolverParams = DepResolverParams {
        depResolverMaxBackjumps      :: Maybe Int
      }
 
+debugDepResolverParams :: DepResolverParams -> String
+debugDepResolverParams p =
+     "targets: " ++ intercalate ", " (map display (depResolverTargets p))
+  ++ "\nconstraints: "
+  ++   concatMap (("\n  " ++) . debugPackageConstraint) (depResolverConstraints p)
+  ++ "\npreferences: "
+  ++   concatMap (("\n  " ++) . debugPackagePreference) (depResolverPreferences p)
+  ++ "\nstrategy: " ++ show (depResolverPreferenceDefault p)
 
 -- | A package selection preference for a particular package.
 --
@@ -139,7 +155,16 @@ data PackagePreference =
      -- | If we prefer versions of packages that are already installed.
    | PackageInstalledPreference PackageName InstalledPreference
 
-basicDepResolverParams :: InstalledPackageIndex.PackageIndex
+-- | Provide a textual representation of a package preference
+-- for debugging purposes.
+--
+debugPackagePreference :: PackagePreference -> String
+debugPackagePreference (PackageVersionPreference   pn vr) =
+  display pn ++ " " ++ display (simplifyVersionRange vr)
+debugPackagePreference (PackageInstalledPreference pn ip) =
+  display pn ++ " " ++ show ip
+
+basicDepResolverParams :: InstalledPackageIndex
                        -> PackageIndex.PackageIndex SourcePackage
                        -> DepResolverParams
 basicDepResolverParams installedPkgIndex sourcePkgIndex =
@@ -224,18 +249,21 @@ setMaxBackjumps n params =
       depResolverMaxBackjumps = n
     }
 
-dontUpgradeBasePackage :: DepResolverParams -> DepResolverParams
-dontUpgradeBasePackage params =
+-- | Some packages are specific to a given compiler version and should never be
+-- upgraded.
+dontUpgradeNonUpgradeablePackages :: DepResolverParams -> DepResolverParams
+dontUpgradeNonUpgradeablePackages params =
     addConstraints extraConstraints params
   where
     extraConstraints =
       [ PackageConstraintInstalled pkgname
       | all (/=PackageName "base") (depResolverTargets params)
-      , pkgname <-  [ PackageName "base", PackageName "ghc-prim" ]
+      , pkgname <- map PackageName [ "base", "ghc-prim", "integer-gmp"
+                                   , "integer-simple" ]
       , isInstalled pkgname ]
     -- TODO: the top down resolver chokes on the base constraints
     -- below when there are no targets and thus no dep on base.
-    -- Need to refactor contraints separate from needing packages.
+    -- Need to refactor constraints separate from needing packages.
     isInstalled = not . null
                 . InstalledPackageIndex.lookupPackageName
                                  (depResolverInstalledPkgIndex params)
@@ -293,6 +321,88 @@ hideBrokenInstalledPackages params =
            . InstalledPackageIndex.brokenPackages
            $ depResolverInstalledPkgIndex params
 
+-- | Remove upper bounds in dependencies using the policy specified by the
+-- 'AllowNewer' argument (all/some/none).
+removeUpperBounds :: AllowNewer -> DepResolverParams -> DepResolverParams
+removeUpperBounds allowNewer params =
+    params {
+      -- NB: It's important to apply 'removeUpperBounds' after
+      -- 'addSourcePackages'. Otherwise, the packages inserted by
+      -- 'addSourcePackages' won't have upper bounds in dependencies relaxed.
+
+      depResolverSourcePkgIndex = sourcePkgIndex'
+    }
+  where
+    sourcePkgIndex  = depResolverSourcePkgIndex params
+    sourcePkgIndex' = case allowNewer of
+      AllowNewerNone      -> sourcePkgIndex
+      AllowNewerAll       -> fmap relaxAllPackageDeps         sourcePkgIndex
+      AllowNewerSome pkgs -> fmap (relaxSomePackageDeps pkgs) sourcePkgIndex
+
+    relaxAllPackageDeps :: SourcePackage -> SourcePackage
+    relaxAllPackageDeps = onAllBuildDepends doRelax
+      where
+        doRelax (Dependency pkgName verRange) =
+          Dependency pkgName (removeUpperBound verRange)
+
+    relaxSomePackageDeps :: [PackageName] -> SourcePackage -> SourcePackage
+    relaxSomePackageDeps pkgNames = onAllBuildDepends doRelax
+      where
+        doRelax d@(Dependency pkgName verRange)
+          | pkgName `elem` pkgNames = Dependency pkgName
+                                      (removeUpperBound verRange)
+          | otherwise               = d
+
+    -- Walk a 'GenericPackageDescription' and apply 'f' to all 'build-depends'
+    -- fields.
+    onAllBuildDepends :: (Dependency -> Dependency)
+                      -> SourcePackage -> SourcePackage
+    onAllBuildDepends f srcPkg = srcPkg'
+      where
+        gpd        = packageDescription srcPkg
+        pd         = PD.packageDescription gpd
+        condLib    = PD.condLibrary        gpd
+        condExes   = PD.condExecutables    gpd
+        condTests  = PD.condTestSuites     gpd
+        condBenchs = PD.condBenchmarks     gpd
+
+        f' = onBuildInfo f
+        onBuildInfo g bi = bi
+          { targetBuildDepends = map g (targetBuildDepends bi) }
+
+        onLibrary    lib  = lib { PD.libBuildInfo  = f' $ PD.libBuildInfo  lib }
+        onExecutable exe  = exe { PD.buildInfo     = f' $ PD.buildInfo     exe }
+        onTestSuite  tst  = tst { PD.testBuildInfo = f' $ PD.testBuildInfo tst }
+        onBenchmark  bmk  = bmk { PD.benchmarkBuildInfo =
+                                     f' $ PD.benchmarkBuildInfo bmk }
+
+        srcPkg' = srcPkg { packageDescription = gpd' }
+        gpd'    = gpd {
+          PD.packageDescription = pd',
+          PD.condLibrary        = condLib',
+          PD.condExecutables    = condExes',
+          PD.condTestSuites     = condTests',
+          PD.condBenchmarks     = condBenchs'
+          }
+        pd' = pd {
+          PD.buildDepends = map  f            (PD.buildDepends pd),
+          PD.library      = fmap onLibrary    (PD.library pd),
+          PD.executables  = map  onExecutable (PD.executables pd),
+          PD.testSuites   = map  onTestSuite  (PD.testSuites pd),
+          PD.benchmarks   = map  onBenchmark  (PD.benchmarks pd)
+          }
+        condLib'    = fmap (onCondTree onLibrary)             condLib
+        condExes'   = map  (mapSnd $ onCondTree onExecutable) condExes
+        condTests'  = map  (mapSnd $ onCondTree onTestSuite)  condTests
+        condBenchs' = map  (mapSnd $ onCondTree onBenchmark)  condBenchs
+
+        mapSnd :: (a -> b) -> (c,a) -> (c,b)
+        mapSnd = fmap
+
+        onCondTree :: (a -> b) -> PD.CondTree v [Dependency] a
+                   -> PD.CondTree v [Dependency] b
+        onCondTree g = mapCondTree g (map f) id
+
 
 upgradeDependencies :: DepResolverParams -> DepResolverParams
 upgradeDependencies = setPreferenceDefault PreferAllLatest
@@ -303,7 +413,7 @@ reinstallTargets params =
     hideInstalledPackagesAllVersions (depResolverTargets params) params
 
 
-standardInstallPolicy :: InstalledPackageIndex.PackageIndex
+standardInstallPolicy :: InstalledPackageIndex
                       -> SourcePackageDb
                       -> [PackageSpecifier SourcePackage]
                       -> DepResolverParams
@@ -372,11 +482,12 @@ applySandboxInstallPolicy
 -- * Interface to the standard resolver
 -- ------------------------------------------------------------
 
-chooseSolver :: Verbosity -> PreSolver -> CompilerId -> IO Solver
+chooseSolver :: Verbosity -> PreSolver -> CompilerInfo -> IO Solver
 chooseSolver _         AlwaysTopDown _                = return TopDown
 chooseSolver _         AlwaysModular _                = return Modular
-chooseSolver verbosity Choose        (CompilerId f v) = do
-  let chosenSolver | f == GHC && v <= Version [7] [] = TopDown
+chooseSolver verbosity Choose        cinfo            = do
+  let (CompilerId f v) = compilerInfoId cinfo
+      chosenSolver | f == GHC && v <= Version [7] [] = TopDown
                    | otherwise                       = Modular
       msg TopDown = warn verbosity "Falling back to topdown solver for GHC < 7."
       msg Modular = info verbosity "Choosing modular solver."
@@ -384,7 +495,7 @@ chooseSolver verbosity Choose        (CompilerId f v) = do
   return chosenSolver
 
 runSolver :: Solver -> SolverConfig -> DependencyResolver
-runSolver TopDown = const topDownResolver -- TODO: warn about unsuported options
+runSolver TopDown = const topDownResolver -- TODO: warn about unsupported options
 runSolver Modular = modularResolver
 
 -- | Run the dependency solver.
@@ -394,25 +505,27 @@ runSolver Modular = modularResolver
 -- logging messages and the final result or an error.
 --
 resolveDependencies :: Platform
-                    -> CompilerId
+                    -> CompilerInfo
                     -> Solver
                     -> DepResolverParams
                     -> Progress String String InstallPlan
 
-    --TODO: is this needed here? see dontUpgradeBasePackage
+    --TODO: is this needed here? see dontUpgradeNonUpgradeablePackages
 resolveDependencies platform comp _solver params
   | null (depResolverTargets params)
   = return (mkInstallPlan platform comp [])
 
 resolveDependencies platform comp  solver params =
 
-    fmap (mkInstallPlan platform comp)
+    Step (debugDepResolverParams finalparams)
+  $ fmap (mkInstallPlan platform comp)
   $ runSolver solver (SolverConfig reorderGoals indGoals noReinstalls
                       shadowing strFlags maxBkjumps)
                      platform comp installedPkgIndex sourcePkgIndex
                      preferences constraints targets
   where
-    DepResolverParams
+
+    finalparams @ (DepResolverParams
       targets constraints
       prefs defpref
       installedPkgIndex
@@ -422,12 +535,12 @@ resolveDependencies platform comp  solver params =
       noReinstalls
       shadowing
       strFlags
-      maxBkjumps      = dontUpgradeBasePackage
+      maxBkjumps)     = dontUpgradeNonUpgradeablePackages
                       -- TODO:
                       -- The modular solver can properly deal with broken
                       -- packages and won't select them. So the
                       -- 'hideBrokenInstalledPackages' function should be moved
-                      -- into a module that is specific to the Topdown solver.
+                      -- into a module that is specific to the top-down solver.
                       . (if solver /= Modular then hideBrokenInstalledPackages
                                               else id)
                       $ params
@@ -435,20 +548,22 @@ resolveDependencies platform comp  solver params =
     preferences = interpretPackagesPreference
                     (Set.fromList targets) defpref prefs
 
-
 -- | Make an install plan from the output of the dep resolver.
 -- It checks that the plan is valid, or it's an error in the dep resolver.
 --
 mkInstallPlan :: Platform
-              -> CompilerId
+              -> CompilerInfo
               -> [InstallPlan.PlanPackage] -> InstallPlan
 mkInstallPlan platform comp pkgIndex =
-  case InstallPlan.new platform comp (PackageIndex.fromList pkgIndex) of
+  let index = InstalledPackageIndex.fromList pkgIndex in
+  case InstallPlan.new platform comp index of
     Right plan     -> plan
     Left  problems -> error $ unlines $
         "internal error: could not construct a valid install plan."
       : "The proposed (invalid) plan contained the following problems:"
       : map InstallPlan.showPlanProblem problems
+      ++ "Proposed plan:"
+      : [InstallPlan.showPlanIndex index]
 
 
 -- | Give an interpretation to the global 'PackagesPreference' as
@@ -497,7 +612,7 @@ interpretPackagesPreference selected defaultPref prefs =
 -- It is suitable for tasks such as selecting packages to download for user
 -- inspection. It is not suitable for selecting packages to install.
 --
--- Note: if no installed package index is available, it is ok to pass 'mempty'.
+-- Note: if no installed package index is available, it is OK to pass 'mempty'.
 -- It simply means preferences for installed packages will be ignored.
 --
 resolveWithoutDependencies :: DepResolverParams
