@@ -31,6 +31,7 @@ import Distribution.Simple.Program (ProgramConfiguration)
 import Distribution.Simple.Utils
         ( equating, comparing, die, notice )
 import Distribution.Simple.Setup (fromFlag)
+import Distribution.Simple.PackageIndex (InstalledPackageIndex)
 import qualified Distribution.Simple.PackageIndex as InstalledPackageIndex
 import qualified Distribution.Client.PackageIndex as PackageIndex
 import Distribution.Version
@@ -70,7 +71,63 @@ import System.Directory
          ( doesDirectoryExist )
 
 
--- |Show information about packages
+-- | Return a list of packages matching given search strings.
+getPkgList :: Verbosity
+           -> PackageDBStack
+           -> [Repo]
+           -> Compiler
+           -> ProgramConfiguration
+           -> ListFlags
+           -> [String]
+           -> IO [PackageDisplayInfo]
+getPkgList verbosity packageDBs repos comp conf listFlags pats = do
+    installedPkgIndex <- getInstalledPackages verbosity comp packageDBs conf
+    sourcePkgDb       <- getSourcePackages    verbosity repos
+    let sourcePkgIndex = packageIndex sourcePkgDb
+        prefs name = fromMaybe anyVersion
+                       (Map.lookup name (packagePreferences sourcePkgDb))
+
+        pkgsInfo ::
+          [(PackageName, [Installed.InstalledPackageInfo], [SourcePackage])]
+        pkgsInfo
+            -- gather info for all packages
+          | null pats = mergePackages
+                        (InstalledPackageIndex.allPackages installedPkgIndex)
+                        (         PackageIndex.allPackages sourcePkgIndex)
+
+            -- gather info for packages matching search term
+          | otherwise = pkgsInfoMatching
+
+        pkgsInfoMatching ::
+          [(PackageName, [Installed.InstalledPackageInfo], [SourcePackage])]
+        pkgsInfoMatching =
+          let matchingInstalled = matchingPackages
+                                  InstalledPackageIndex.searchByNameSubstring
+                                  installedPkgIndex
+              matchingSource   = matchingPackages
+                                 (\ idx n ->
+                                   concatMap snd
+                                   (PackageIndex.searchByNameSubstring idx n))
+                                 sourcePkgIndex
+          in mergePackages matchingInstalled matchingSource
+
+        matches :: [PackageDisplayInfo]
+        matches = [ mergePackageInfo pref
+                      installedPkgs sourcePkgs selectedPkg False
+                  | (pkgname, installedPkgs, sourcePkgs) <- pkgsInfo
+                  , not onlyInstalled || not (null installedPkgs)
+                  , let pref        = prefs pkgname
+                        selectedPkg = latestWithPref pref sourcePkgs ]
+    return matches
+  where
+    onlyInstalled = fromFlag (listInstalled listFlags)
+    matchingPackages search index =
+      [ pkg
+      | pat <- pats
+      , pkg <- search index pat ]
+
+
+-- | Show information about packages.
 list :: Verbosity
      -> PackageDBStack
      -> [Repo]
@@ -80,30 +137,7 @@ list :: Verbosity
      -> [String]
      -> IO ()
 list verbosity packageDBs repos comp conf listFlags pats = do
-
-    installedPkgIndex <- getInstalledPackages verbosity comp packageDBs conf
-    sourcePkgDb       <- getSourcePackages    verbosity repos
-    let sourcePkgIndex = packageIndex sourcePkgDb
-        prefs name = fromMaybe anyVersion
-                       (Map.lookup name (packagePreferences sourcePkgDb))
-
-        pkgsInfo :: [(PackageName, [Installed.InstalledPackageInfo], [SourcePackage])]
-        pkgsInfo
-            -- gather info for all packages
-          | null pats = mergePackages (InstalledPackageIndex.allPackages installedPkgIndex)
-                                      (         PackageIndex.allPackages sourcePkgIndex)
-
-            -- gather info for packages matching search term
-          | otherwise = mergePackages (matchingPackages InstalledPackageIndex.searchByNameSubstring installedPkgIndex)
-                                      (matchingPackages (\ idx n -> concatMap snd (PackageIndex.searchByNameSubstring idx n)) sourcePkgIndex)
-
-        matches :: [PackageDisplayInfo]
-        matches = [ mergePackageInfo pref
-                      installedPkgs sourcePkgs selectedPkg False
-                  | (pkgname, installedPkgs, sourcePkgs) <- pkgsInfo
-                  , not onlyInstalled || not (null installedPkgs)
-                  , let pref        = prefs pkgname
-                        selectedPkg = latestWithPref pref sourcePkgs ]
+    matches <- getPkgList verbosity packageDBs repos comp conf listFlags pats
 
     if simpleOutput
       then putStr $ unlines
@@ -124,11 +158,6 @@ list verbosity packageDBs repos comp conf listFlags pats = do
     onlyInstalled = fromFlag (listInstalled listFlags)
     simpleOutput  = fromFlag (listSimpleOutput listFlags)
 
-    matchingPackages search index =
-      [ pkg
-      | pat <- pats
-      , pkg <- search index pat ]
-
 info :: Verbosity
      -> PackageDBStack
      -> [Repo]
@@ -154,8 +183,10 @@ info verbosity packageDBs repos comp conf
         -- just available source packages, so we must resolve targets using
         -- the combination of installed and source packages.
     let sourcePkgs' = PackageIndex.fromList
-                    $ map packageId (InstalledPackageIndex.allPackages installedPkgIndex)
-                   ++ map packageId (         PackageIndex.allPackages sourcePkgIndex)
+                    $ map packageId
+                      (InstalledPackageIndex.allPackages installedPkgIndex)
+                   ++ map packageId
+                      (PackageIndex.allPackages sourcePkgIndex)
     pkgSpecifiers <- resolveUserTargets verbosity
                        (fromFlag $ globalWorldFile globalFlags)
                        sourcePkgs' userTargets
@@ -172,11 +203,12 @@ info verbosity packageDBs repos comp conf
 
   where
     gatherPkgInfo :: (PackageName -> VersionRange) ->
-                     InstalledPackageIndex.PackageIndex ->
+                     InstalledPackageIndex ->
                      PackageIndex.PackageIndex SourcePackage ->
                      PackageSpecifier SourcePackage ->
                      Either String PackageDisplayInfo
-    gatherPkgInfo prefs installedPkgIndex sourcePkgIndex (NamedPackage name constraints)
+    gatherPkgInfo prefs installedPkgIndex sourcePkgIndex
+      (NamedPackage name constraints)
       | null (selectedInstalledPkgs) && null (selectedSourcePkgs)
       = Left $ "There is no available version of " ++ display name
             ++ " that satisfies "
@@ -190,10 +222,11 @@ info verbosity packageDBs repos comp conf
         (pref, installedPkgs, sourcePkgs) =
           sourcePkgsInfo prefs name installedPkgIndex sourcePkgIndex
 
-        selectedInstalledPkgs = InstalledPackageIndex.lookupDependency installedPkgIndex
-                                    (Dependency name verConstraint)
-        selectedSourcePkgs    =          PackageIndex.lookupDependency sourcePkgIndex
-                                    (Dependency name verConstraint)
+        selectedInstalledPkgs = InstalledPackageIndex.lookupDependency
+                                installedPkgIndex
+                                (Dependency name verConstraint)
+        selectedSourcePkgs    = PackageIndex.lookupDependency sourcePkgIndex
+                                (Dependency name verConstraint)
         selectedSourcePkg'    = latestWithPref pref selectedSourcePkgs
 
                          -- display a specific package version if the user
@@ -202,7 +235,8 @@ info verbosity packageDBs repos comp conf
         verConstraint  = foldr intersectVersionRanges anyVersion verConstraints
         verConstraints = [ vr | PackageConstraintVersion _ vr <- constraints ]
 
-    gatherPkgInfo prefs installedPkgIndex sourcePkgIndex (SpecificSourcePackage pkg) =
+    gatherPkgInfo prefs installedPkgIndex sourcePkgIndex
+      (SpecificSourcePackage pkg) =
         Right $ mergePackageInfo pref installedPkgs sourcePkgs
                                  selectedPkg True
       where
@@ -214,19 +248,20 @@ info verbosity packageDBs repos comp conf
 sourcePkgsInfo ::
   (PackageName -> VersionRange)
   -> PackageName
-  -> InstalledPackageIndex.PackageIndex
+  -> InstalledPackageIndex
   -> PackageIndex.PackageIndex SourcePackage
   -> (VersionRange, [Installed.InstalledPackageInfo], [SourcePackage])
 sourcePkgsInfo prefs name installedPkgIndex sourcePkgIndex =
   (pref, installedPkgs, sourcePkgs)
   where
     pref          = prefs name
-    installedPkgs = concatMap snd (InstalledPackageIndex.lookupPackageName installedPkgIndex name)
-    sourcePkgs    =                         PackageIndex.lookupPackageName sourcePkgIndex name
+    installedPkgs = concatMap snd (InstalledPackageIndex.lookupPackageName
+                                   installedPkgIndex name)
+    sourcePkgs    = PackageIndex.lookupPackageName sourcePkgIndex name
 
 
 -- | The info that we can display for each package. It is information per
--- package name and covers all installed and avilable versions.
+-- package name and covers all installed and available versions.
 --
 data PackageDisplayInfo = PackageDisplayInfo {
     pkgName           :: PackageName,
@@ -415,11 +450,14 @@ mergePackageInfo versionPref installedPkgs sourcePkgs selectedPkg showVer =
     hasExe       = fromMaybe False
                    (fmap (not . null . Source.condExecutables) sourceGeneric),
     executables  = map fst (maybe [] Source.condExecutables sourceGeneric),
-    modules      = combine Installed.exposedModules installed
-                           (maybe [] Source.exposedModules
-                                   . Source.library) source,
-    dependencies = combine (map (SourceDependency . simplifyDependency) . Source.buildDepends) source
-                           (map InstalledDependency . Installed.depends) installed,
+    modules      = combine (map Installed.exposedName . Installed.exposedModules)
+                           installed
+                           (maybe [] getListOfExposedModules . Source.library)
+                           source,
+    dependencies =
+      combine (map (SourceDependency . simplifyDependency)
+               . Source.buildDepends) source
+      (map InstalledDependency . Installed.depends) installed,
     haddockHtml  = fromMaybe "" . join
                  . fmap (listToMaybe . Installed.haddockHTMLs)
                  $ installed,
@@ -430,6 +468,10 @@ mergePackageInfo versionPref installedPkgs sourcePkgs selectedPkg showVer =
     installed :: Maybe Installed.InstalledPackageInfo
     installed = latestWithPref versionPref installedPkgs
 
+    getListOfExposedModules lib = Source.exposedModules lib
+                               ++ map Source.moduleReexportName
+                                      (Source.reexportedModules lib)
+
     sourceSelected
       | isJust selectedPkg = selectedPkg
       | otherwise          = latestWithPref versionPref sourcePkgs
