@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Distribution.Client.SetupWrapper
@@ -26,7 +27,7 @@ import Distribution.Version
          ( Version(..), VersionRange, anyVersion
          , intersectVersionRanges, orLaterVersion
          , withinRange )
-import Distribution.InstalledPackageInfo (installedPackageId, sourcePackageId)
+import Distribution.InstalledPackageInfo (installedPackageId)
 import Distribution.Package
          ( InstalledPackageId(..), PackageIdentifier(..),
            PackageName(..), Package(..), packageName
@@ -34,23 +35,26 @@ import Distribution.Package
 import Distribution.PackageDescription
          ( GenericPackageDescription(packageDescription)
          , PackageDescription(..), specVersion
-         , BuildType(..), knownBuildTypes )
+         , BuildType(..), knownBuildTypes, defaultRenaming )
 import Distribution.PackageDescription.Parse
          ( readPackageDescription )
 import Distribution.Simple.Configure
          ( configCompilerEx )
-import Distribution.Compiler ( buildCompilerId )
+import Distribution.Compiler
+         ( buildCompilerId, CompilerFlavor(GHC, GHCJS) )
 import Distribution.Simple.Compiler
-         ( CompilerFlavor(GHC), Compiler(compilerId)
-         , compilerVersion
-         , PackageDB(..), PackageDBStack )
+         ( Compiler(compilerId), compilerFlavor, PackageDB(..), PackageDBStack )
+import Distribution.Simple.PreProcess
+         ( runSimplePreProcessor, ppUnlit )
 import Distribution.Simple.Program
          ( ProgramConfiguration, emptyProgramConfiguration
-         , getProgramSearchPath, getDbProgramOutput, runDbProgram, ghcProgram )
+         , getProgramSearchPath, getDbProgramOutput, runDbProgram, ghcProgram
+         , ghcjsProgram )
 import Distribution.Simple.Program.Find
          ( programSearchPathAsPATHVar )
 import Distribution.Simple.Program.Run
          ( getEffectiveEnvironment )
+import qualified Distribution.Simple.Program.Strip as Strip
 import Distribution.Simple.BuildPaths
          ( defaultDistPref, exeExtension )
 import Distribution.Simple.Command
@@ -58,7 +62,7 @@ import Distribution.Simple.Command
 import Distribution.Simple.Program.GHC
          ( GhcMode(..), GhcOptions(..), renderGhcOptions )
 import qualified Distribution.Simple.PackageIndex as PackageIndex
-import Distribution.Simple.PackageIndex (PackageIndex)
+import Distribution.Simple.PackageIndex (InstalledPackageIndex)
 import Distribution.Client.Config
          ( defaultCabalDir )
 import Distribution.Client.IndexUtils
@@ -68,42 +72,75 @@ import Distribution.Client.JobControl
 import Distribution.Simple.Setup
          ( Flag(..) )
 import Distribution.Simple.Utils
-         ( die, debug, info, cabalVersion, findPackageDesc, comparing
+         ( die, debug, info, cabalVersion, tryFindPackageDesc, comparing
          , createDirectoryIfMissingVerbose, installExecutableFile
-         , moreRecentFile, rewriteFile, intercalate )
+         , copyFileVerbose, rewriteFile, intercalate )
 import Distribution.Client.Utils
-         ( inDir, tryCanonicalizePath )
+         ( inDir, tryCanonicalizePath
+         , existsAndIsMoreRecentThan, moreRecentFile
+#if mingw32_HOST_OS
+         , canonicalizePathNoThrow
+#endif
+         )
 import Distribution.System ( Platform(..), buildPlatform )
 import Distribution.Text
          ( display )
+import Distribution.Utils.NubList
+         ( toNubListR )
 import Distribution.Verbosity
          ( Verbosity )
 import Distribution.Compat.Exception
          ( catchIO )
 
-import System.Directory  ( doesFileExist )
-import System.FilePath   ( (</>), (<.>) )
-import System.IO         ( Handle, hPutStr )
-import System.Exit       ( ExitCode(..), exitWith )
-import System.Process    ( runProcess, waitForProcess )
-import Control.Monad     ( when, unless )
-import Data.List         ( foldl1' )
-import Data.Maybe        ( fromMaybe, isJust )
-import Data.Monoid       ( mempty )
-import Data.Char         ( isSpace )
+import System.Directory    ( doesFileExist )
+import System.FilePath     ( (</>), (<.>) )
+import System.IO           ( Handle, hPutStr )
+import System.Exit         ( ExitCode(..), exitWith )
+import System.Process      ( runProcess, waitForProcess )
+#if !MIN_VERSION_base(4,8,0)
+import Control.Applicative ( (<$>), (<*>) )
+#endif
+import Control.Monad       ( when, unless )
+import Data.List           ( foldl1' )
+import Data.Maybe          ( fromMaybe, isJust )
+#if !MIN_VERSION_base(4,8,0)
+import Data.Monoid         ( mempty )
+#endif
+import Data.Char           ( isSpace )
+
+#ifdef mingw32_HOST_OS
+import Distribution.Simple.Utils
+         ( withTempDirectory )
+
+import Control.Exception   ( bracket )
+import System.FilePath     ( equalFilePath, takeDirectory )
+import System.Directory    ( doesDirectoryExist )
+import qualified System.Win32 as Win32
+#endif
 
 data SetupScriptOptions = SetupScriptOptions {
     useCabalVersion          :: VersionRange,
     useCompiler              :: Maybe Compiler,
     usePlatform              :: Maybe Platform,
     usePackageDB             :: PackageDBStack,
-    usePackageIndex          :: Maybe PackageIndex,
+    usePackageIndex          :: Maybe InstalledPackageIndex,
     useProgramConfig         :: ProgramConfiguration,
     useDistPref              :: FilePath,
     useLoggingHandle         :: Maybe Handle,
     useWorkingDir            :: Maybe FilePath,
     forceExternalSetupMethod :: Bool,
 
+    -- Used only by 'cabal clean' on Windows.
+    --
+    -- Note: win32 clean hack
+    -------------------------
+    -- On Windows, running './dist/setup/setup clean' doesn't work because the
+    -- setup script will try to delete itself (which causes it to fail horribly,
+    -- unlike on Linux). So we have to move the setup exe out of the way first
+    -- and then delete it manually. This applies only to the external setup
+    -- method.
+    useWin32CleanHack        :: Bool,
+
     -- Used only when calling setupWrapper from parallel code to serialise
     -- access to the setup cache; should be Nothing otherwise.
     --
@@ -130,6 +167,7 @@ defaultSetupScriptOptions = SetupScriptOptions {
     useDistPref              = defaultDistPref,
     useLoggingHandle         = Nothing,
     useWorkingDir            = Nothing,
+    useWin32CleanHack        = False,
     forceExternalSetupMethod = False,
     setupCacheLock           = Nothing
   }
@@ -156,7 +194,7 @@ setupWrapper verbosity options mpkg cmd flags extraArgs = do
   checkBuildType buildType'
   setupMethod verbosity options' (packageId pkg) buildType' mkArgs
   where
-    getPkg = findPackageDesc (fromMaybe "." (useWorkingDir options))
+    getPkg = tryFindPackageDesc (fromMaybe "." (useWorkingDir options))
          >>= readPackageDescription verbosity
          >>= return . packageDescription
 
@@ -212,15 +250,13 @@ externalSetupMethod :: SetupMethod
 externalSetupMethod verbosity options pkg bt mkargs = do
   debug verbosity $ "Using external setup method with build-type " ++ show bt
   createDirectoryIfMissingVerbose verbosity True setupDir
-  (cabalLibVersion, options') <- cabalLibVersionToUse
+  (cabalLibVersion, mCabalLibInstalledPkgId, options') <- cabalLibVersionToUse
   debug verbosity $ "Using Cabal library version " ++ display cabalLibVersion
-  setupHs <- updateSetupScript cabalLibVersion bt
-  debug verbosity $ "Using " ++ setupHs ++ " as setup script."
-  path <- case bt of
-    -- TODO: Should we also cache the setup exe for the Make and Configure build
-    -- types?
-    Simple -> getCachedSetupExecutable options' cabalLibVersion setupHs
-    _      -> compileSetupExecutable options' cabalLibVersion setupHs False
+  path <- if useCachedSetupExecutable
+          then getCachedSetupExecutable options'
+               cabalLibVersion mCabalLibInstalledPkgId
+          else compileSetupExecutable   options'
+               cabalLibVersion mCabalLibInstalledPkgId False
   invokeSetupScript options' path (mkargs cabalLibVersion)
 
   where
@@ -228,46 +264,113 @@ externalSetupMethod verbosity options pkg bt mkargs = do
                        []  -> "."
                        dir -> dir
   setupDir         = workingDir </> useDistPref options </> "setup"
-  setupVersionFile = setupDir </> "setup" <.> "version"
+  setupVersionFile = setupDir   </> "setup" <.> "version"
+  setupHs          = setupDir   </> "setup" <.> "hs"
+  setupProgFile    = setupDir   </> "setup" <.> exeExtension
+  platform         = fromMaybe buildPlatform (usePlatform options)
+
+  useCachedSetupExecutable = (bt == Simple || bt == Configure || bt == Make)
 
   maybeGetInstalledPackages :: SetupScriptOptions -> Compiler
-                               -> ProgramConfiguration -> IO PackageIndex
+                               -> ProgramConfiguration -> IO InstalledPackageIndex
   maybeGetInstalledPackages options' comp conf =
     case usePackageIndex options' of
       Just index -> return index
       Nothing    -> getInstalledPackages verbosity
                     comp (usePackageDB options') conf
 
-  cabalLibVersionToUse :: IO (Version, SetupScriptOptions)
+  cabalLibVersionToUse :: IO (Version, (Maybe InstalledPackageId)
+                             ,SetupScriptOptions)
   cabalLibVersionToUse = do
-    savedVersion <- savedCabalVersion
-    case savedVersion of
+    savedVer <- savedVersion
+    case savedVer of
       Just version | version `withinRange` useCabalVersion options
-        -> return (version, options)
-      _ -> do (comp, conf, options') <- configureCompiler options
-              version <- installedCabalVersion options' comp conf
-              writeFile setupVersionFile (show version ++ "\n")
-              return (version, options')
+        -> do updateSetupScript version bt
+              -- Does the previously compiled setup executable still exist and
+              -- is it up-to date?
+              useExisting <- canUseExistingSetup version
+              if useExisting
+                then return (version, Nothing, options)
+                else installedVersion
+      _ -> installedVersion
+    where
+      -- This check duplicates the checks in 'getCachedSetupExecutable' /
+      -- 'compileSetupExecutable'. Unfortunately, we have to perform it twice
+      -- because the selected Cabal version may change as a result of this
+      -- check.
+      canUseExistingSetup :: Version -> IO Bool
+      canUseExistingSetup version =
+        if useCachedSetupExecutable
+        then do
+          (_, cachedSetupProgFile) <- cachedSetupDirAndProg options version
+          doesFileExist cachedSetupProgFile
+        else
+          (&&) <$> setupProgFile `existsAndIsMoreRecentThan` setupHs
+               <*> setupProgFile `existsAndIsMoreRecentThan` setupVersionFile
 
-  savedCabalVersion = do
-    versionString <- readFile setupVersionFile `catchIO` \_ -> return ""
-    case reads versionString of
-      [(version,s)] | all isSpace s -> return (Just version)
-      _                             -> return Nothing
+      installedVersion :: IO (Version, Maybe InstalledPackageId
+                             ,SetupScriptOptions)
+      installedVersion = do
+        (comp,    conf,    options')  <- configureCompiler options
+        (version, mipkgid, options'') <- installedCabalVersion options' comp conf
+        updateSetupScript version bt
+        writeFile setupVersionFile (show version ++ "\n")
+        return (version, mipkgid, options'')
 
-  installedCabalVersion :: SetupScriptOptions -> Compiler
-                        -> ProgramConfiguration -> IO Version
-  installedCabalVersion _ _ _ | packageName pkg == PackageName "Cabal" =
-    return (packageVersion pkg)
-  installedCabalVersion options' comp conf = do
-    index <- maybeGetInstalledPackages options' comp conf
-    let cabalDep = Dependency (PackageName "Cabal") (useCabalVersion options')
+      savedVersion :: IO (Maybe Version)
+      savedVersion = do
+        versionString <- readFile setupVersionFile `catchIO` \_ -> return ""
+        case reads versionString of
+          [(version,s)] | all isSpace s -> return (Just version)
+          _                             -> return Nothing
+
+  -- | Update a Setup.hs script, creating it if necessary.
+  updateSetupScript :: Version -> BuildType -> IO ()
+  updateSetupScript _ Custom = do
+    useHs  <- doesFileExist customSetupHs
+    useLhs <- doesFileExist customSetupLhs
+    unless (useHs || useLhs) $ die
+      "Using 'build-type: Custom' but there is no Setup.hs or Setup.lhs script."
+    let src = (if useHs then customSetupHs else customSetupLhs)
+    srcNewer <- src `moreRecentFile` setupHs
+    when srcNewer $ if useHs
+                    then copyFileVerbose verbosity src setupHs
+                    else runSimplePreProcessor ppUnlit src setupHs verbosity
+    where
+      customSetupHs   = workingDir </> "Setup.hs"
+      customSetupLhs  = workingDir </> "Setup.lhs"
+
+  updateSetupScript cabalLibVersion _ =
+    rewriteFile setupHs (buildTypeScript cabalLibVersion)
+
+  buildTypeScript :: Version -> String
+  buildTypeScript cabalLibVersion = case bt of
+    Simple    -> "import Distribution.Simple; main = defaultMain\n"
+    Configure -> "import Distribution.Simple; main = defaultMainWithHooks "
+              ++ if cabalLibVersion >= Version [1,3,10] []
+                   then "autoconfUserHooks\n"
+                   else "defaultUserHooks\n"
+    Make      -> "import Distribution.Make; main = defaultMain\n"
+    Custom             -> error "buildTypeScript Custom"
+    UnknownBuildType _ -> error "buildTypeScript UnknownBuildType"
+
+  installedCabalVersion :: SetupScriptOptions -> Compiler -> ProgramConfiguration
+                        -> IO (Version, Maybe InstalledPackageId
+                              ,SetupScriptOptions)
+  installedCabalVersion options' _ _ | packageName pkg == PackageName "Cabal" =
+    return (packageVersion pkg, Nothing, options')
+  installedCabalVersion options' compiler conf = do
+    index <- maybeGetInstalledPackages options' compiler conf
+    let cabalDep   = Dependency (PackageName "Cabal") (useCabalVersion options')
+        options''  = options' { usePackageIndex = Just index }
     case PackageIndex.lookupDependency index cabalDep of
       []   -> die $ "The package '" ++ display (packageName pkg)
                  ++ "' requires Cabal library version "
                  ++ display (useCabalVersion options)
                  ++ " but no suitable version is installed."
-      pkgs -> return $ bestVersion id (map fst pkgs)
+      pkgs -> let ipkginfo = head . snd . bestVersion fst $ pkgs
+              in return (packageVersion ipkginfo
+                        ,Just . installedPackageId $ ipkginfo, options'')
 
   bestVersion :: (a -> Version) -> [a] -> a
   bestVersion f = firstMaximumBy (comparing (preference . f))
@@ -299,23 +402,6 @@ externalSetupMethod verbosity options pkg bt mkargs = do
                                _       -> False
           latestVersion    = version
 
-  -- TODO: This function looks a lot like @installedCabalVersion@ - can the
-  -- duplication be removed?
-  installedCabalPkgId :: SetupScriptOptions -> Compiler -> ProgramConfiguration
-                         -> Version -> IO (Maybe InstalledPackageId)
-  installedCabalPkgId _ _ _ _ | packageName pkg == PackageName "Cabal" =
-    return Nothing
-  installedCabalPkgId options' compiler conf cabalLibVersion = do
-    index <- maybeGetInstalledPackages options' compiler conf
-    let cabalPkgid = PackageIdentifier (PackageName "Cabal") cabalLibVersion
-    case PackageIndex.lookupSourcePackageId index cabalPkgid of
-      []           -> die $ "The package '" ++ display (packageName pkg)
-                      ++ "' requires Cabal library version "
-                      ++ display (cabalLibVersion)
-                      ++ " but no suitable version is installed."
-      iPkgInfos   -> return . Just . installedPackageId
-                     . bestVersion (pkgVersion . sourcePackageId) $ iPkgInfos
-
   configureCompiler :: SetupScriptOptions
                     -> IO (Compiler, ProgramConfiguration, SetupScriptOptions)
   configureCompiler options' = do
@@ -326,121 +412,111 @@ externalSetupMethod verbosity options pkg bt mkargs = do
                         (useProgramConfig options') verbosity
                       return (comp, conf)
     -- Whenever we need to call configureCompiler, we also need to access the
-    -- package index, so let's cache it here.
+    -- package index, so let's cache it in SetupScriptOptions.
     index <- maybeGetInstalledPackages options' comp conf
     return (comp, conf, options' { useCompiler      = Just comp,
                                    usePackageIndex  = Just index,
                                    useProgramConfig = conf })
 
-  -- | Decide which Setup.hs script to use, creating it if necessary.
-  --
-  updateSetupScript :: Version -> BuildType -> IO FilePath
-  updateSetupScript _ Custom = do
-    useHs  <- doesFileExist setupHs
-    useLhs <- doesFileExist setupLhs
-    unless (useHs || useLhs) $ die
-      "Using 'build-type: Custom' but there is no Setup.hs or Setup.lhs script."
-    return (if useHs then setupHs else setupLhs)
-    where
-      setupHs  = workingDir </> "Setup.hs"
-      setupLhs = workingDir </> "Setup.lhs"
-
-  updateSetupScript cabalLibVersion _ = do
-    rewriteFile setupHs (buildTypeScript cabalLibVersion)
-    return setupHs
-    where
-      setupHs  = setupDir </> "setup.hs"
-
-  buildTypeScript :: Version -> String
-  buildTypeScript cabalLibVersion = case bt of
-    Simple    -> "import Distribution.Simple; main = defaultMain\n"
-    Configure -> "import Distribution.Simple; main = defaultMainWithHooks "
-              ++ if cabalLibVersion >= Version [1,3,10] []
-                   then "autoconfUserHooks\n"
-                   else "defaultUserHooks\n"
-    Make      -> "import Distribution.Make; main = defaultMain\n"
-    Custom             -> error "buildTypeScript Custom"
-    UnknownBuildType _ -> error "buildTypeScript UnknownBuildType"
-
-  -- | Look up the setup executable in the cache; update the cache if the setup
-  -- executable is not found.
-  getCachedSetupExecutable :: SetupScriptOptions -> Version -> FilePath
-                           -> IO FilePath
-  getCachedSetupExecutable options' cabalLibVersion setupHsFile = do
+  -- | Path to the setup exe cache directory and path to the cached setup
+  -- executable.
+  cachedSetupDirAndProg :: SetupScriptOptions -> Version
+                        -> IO (FilePath, FilePath)
+  cachedSetupDirAndProg options' cabalLibVersion = do
     cabalDir <- defaultCabalDir
-    let setupCacheDir = cabalDir </> "setup-exe-cache"
-    let setupProgFile = setupCacheDir
-                        </> ("setup-" ++ cabalVersionString ++ "-"
-                             ++ platformString ++ "-"
-                             ++ compilerVersionString)
-                        <.> exeExtension
-    setupProgFileExists <- doesFileExist setupProgFile
-    if setupProgFileExists
-      then debug verbosity $
-           "Found cached setup executable: " ++ setupProgFile
-      else criticalSection' $ do
-        -- The cache may have been populated while we were waiting.
-        setupProgFileExists' <- doesFileExist setupProgFile
-        if setupProgFileExists'
-          then debug verbosity $
-               "Found cached setup executable: " ++ setupProgFile
-          else do
-          debug verbosity $ "Setup executable not found in the cache."
-          src <- compileSetupExecutable options' cabalLibVersion setupHsFile True
-          createDirectoryIfMissingVerbose verbosity True setupCacheDir
-          installExecutableFile verbosity src setupProgFile
-    return setupProgFile
+    let setupCacheDir       = cabalDir </> "setup-exe-cache"
+        cachedSetupProgFile = setupCacheDir
+                              </> ("setup-" ++ buildTypeString ++ "-"
+                                   ++ cabalVersionString ++ "-"
+                                   ++ platformString ++ "-"
+                                   ++ compilerVersionString)
+                              <.> exeExtension
+    return (setupCacheDir, cachedSetupProgFile)
       where
+        buildTypeString       = show bt
         cabalVersionString    = "Cabal-" ++ (display cabalLibVersion)
         compilerVersionString = display $
                                 fromMaybe buildCompilerId
                                 (fmap compilerId . useCompiler $ options')
-        platformString        = display $
-                                fromMaybe buildPlatform (usePlatform options')
+        platformString        = display platform
+
+  -- | Look up the setup executable in the cache; update the cache if the setup
+  -- executable is not found.
+  getCachedSetupExecutable :: SetupScriptOptions
+                           -> Version -> Maybe InstalledPackageId
+                           -> IO FilePath
+  getCachedSetupExecutable options' cabalLibVersion
+                           maybeCabalLibInstalledPkgId = do
+    (setupCacheDir, cachedSetupProgFile) <-
+      cachedSetupDirAndProg options' cabalLibVersion
+    cachedSetupExists <- doesFileExist cachedSetupProgFile
+    if cachedSetupExists
+      then debug verbosity $
+           "Found cached setup executable: " ++ cachedSetupProgFile
+      else criticalSection' $ do
+        -- The cache may have been populated while we were waiting.
+        cachedSetupExists' <- doesFileExist cachedSetupProgFile
+        if cachedSetupExists'
+          then debug verbosity $
+               "Found cached setup executable: " ++ cachedSetupProgFile
+          else do
+          debug verbosity $ "Setup executable not found in the cache."
+          src <- compileSetupExecutable options'
+                 cabalLibVersion maybeCabalLibInstalledPkgId True
+          createDirectoryIfMissingVerbose verbosity True setupCacheDir
+          installExecutableFile verbosity src cachedSetupProgFile
+          -- Do not strip if we're using GHCJS, since the result may be a script
+          when (maybe True ((/=GHCJS).compilerFlavor) $ useCompiler options') $
+            Strip.stripExe verbosity platform (useProgramConfig options')
+              cachedSetupProgFile
+    return cachedSetupProgFile
+      where
         criticalSection'      = fromMaybe id
                                 (fmap criticalSection $ setupCacheLock options')
 
   -- | If the Setup.hs is out of date wrt the executable then recompile it.
-  -- Currently this is GHC only. It should really be generalised.
+  -- Currently this is GHC/GHCJS only. It should really be generalised.
   --
-  compileSetupExecutable :: SetupScriptOptions -> Version -> FilePath -> Bool
+  compileSetupExecutable :: SetupScriptOptions
+                         -> Version -> Maybe InstalledPackageId -> Bool
                          -> IO FilePath
-  compileSetupExecutable options' cabalLibVersion setupHsFile forceCompile = do
-    setupHsNewer      <- setupHsFile      `moreRecentFile` setupProgFile
+  compileSetupExecutable options' cabalLibVersion maybeCabalLibInstalledPkgId
+                         forceCompile = do
+    setupHsNewer      <- setupHs          `moreRecentFile` setupProgFile
     cabalVersionNewer <- setupVersionFile `moreRecentFile` setupProgFile
     let outOfDate = setupHsNewer || cabalVersionNewer
     when (outOfDate || forceCompile) $ do
       debug verbosity "Setup executable needs to be updated, compiling..."
       (compiler, conf, options'') <- configureCompiler options'
       let cabalPkgid = PackageIdentifier (PackageName "Cabal") cabalLibVersion
-      maybeCabalInstalledPkgId <- installedCabalPkgId options'' compiler conf
-                                  cabalLibVersion
-      let ghcOptions = mempty {
+          (program, extraOpts)
+            = case compilerFlavor compiler of
+                      GHCJS -> (ghcjsProgram, ["-build-runner"])
+                      _     -> (ghcProgram,   ["-threaded"])
+          ghcOptions = mempty {
               ghcOptVerbosity       = Flag verbosity
             , ghcOptMode            = Flag GhcModeMake
-            , ghcOptInputFiles      = [setupHsFile]
+            , ghcOptInputFiles      = toNubListR [setupHs]
             , ghcOptOutputFile      = Flag setupProgFile
             , ghcOptObjDir          = Flag setupDir
             , ghcOptHiDir           = Flag setupDir
             , ghcOptSourcePathClear = Flag True
-            , ghcOptSourcePath      = [workingDir]
+            , ghcOptSourcePath      = toNubListR [workingDir]
             , ghcOptPackageDBs      = usePackageDB options''
-            , ghcOptPackages        =
-              maybe []
-              (\cabalInstalledPkgId -> [(cabalInstalledPkgId, cabalPkgid)])
-              maybeCabalInstalledPkgId
+            , ghcOptPackages        = toNubListR $
+                maybe [] (\ipkgid -> [(ipkgid, cabalPkgid, defaultRenaming)])
+                maybeCabalLibInstalledPkgId
+            , ghcOptExtra           = toNubListR extraOpts
             }
-      let ghcCmdLine = renderGhcOptions (compilerVersion compiler) ghcOptions
+      let ghcCmdLine = renderGhcOptions compiler ghcOptions
       case useLoggingHandle options of
-        Nothing          -> runDbProgram verbosity ghcProgram conf ghcCmdLine
+        Nothing          -> runDbProgram verbosity program conf ghcCmdLine
 
         -- If build logging is enabled, redirect compiler output to the log file.
-        (Just logHandle) -> do output <- getDbProgramOutput verbosity ghcProgram
+        (Just logHandle) -> do output <- getDbProgramOutput verbosity program
                                          conf ghcCmdLine
                                hPutStr logHandle output
     return setupProgFile
-    where
-      setupProgFile = setupDir </> "setup" <.> exeExtension
 
   invokeSetupScript :: SetupScriptOptions -> FilePath -> [String] -> IO ()
   invokeSetupScript options' path args = do
@@ -456,12 +532,48 @@ externalSetupMethod verbosity options pkg bt mkargs = do
     -- working directory.
     path' <- tryCanonicalizePath path
 
-    searchpath <- programSearchPathAsPATHVar
-                    (getProgramSearchPath (useProgramConfig options'))
-    env        <- getEffectiveEnvironment [("PATH", Just searchpath)]
+    -- See 'Note: win32 clean hack' above.
+#if mingw32_HOST_OS
+    -- setupProgFile may not exist if we're using a cached program
+    setupProgFile' <- canonicalizePathNoThrow setupProgFile
+    let win32CleanHackNeeded = (useWin32CleanHack options')
+                               -- Skip when a cached setup script is used.
+                               && setupProgFile' `equalFilePath` path'
+    if win32CleanHackNeeded then doWin32CleanHack path' else doInvoke path'
+#else
+    doInvoke path'
+#endif
 
-    process <- runProcess path' args
-                 (useWorkingDir options') env
-                 Nothing (useLoggingHandle options') (useLoggingHandle options')
-    exitCode <- waitForProcess process
-    unless (exitCode == ExitSuccess) $ exitWith exitCode
+    where
+      doInvoke path' = do
+        searchpath <- programSearchPathAsPATHVar
+                      (getProgramSearchPath (useProgramConfig options'))
+        env        <- getEffectiveEnvironment [("PATH", Just searchpath)]
+
+        process <- runProcess path' args
+                   (useWorkingDir options') env Nothing
+                   (useLoggingHandle options') (useLoggingHandle options')
+        exitCode <- waitForProcess process
+        unless (exitCode == ExitSuccess) $ exitWith exitCode
+
+#if mingw32_HOST_OS
+      doWin32CleanHack path' = do
+        info verbosity $ "Using the Win32 clean hack."
+        -- Recursively removes the temp dir on exit.
+        withTempDirectory verbosity workingDir "cabal-tmp" $ \tmpDir ->
+            bracket (moveOutOfTheWay tmpDir path')
+                    (maybeRestore path')
+                    doInvoke
+
+      moveOutOfTheWay tmpDir path' = do
+        let newPath = tmpDir </> "setup" <.> exeExtension
+        Win32.moveFile path' newPath
+        return newPath
+
+      maybeRestore oldPath path' = do
+        let oldPathDir = takeDirectory oldPath
+        oldPathDirExists <- doesDirectoryExist oldPathDir
+        -- 'setup clean' didn't complete, 'dist/setup' still exists.
+        when oldPathDirExists $
+          Win32.moveFile path' oldPath
+#endif
