@@ -1,4 +1,7 @@
 {-# LANGUAGE BangPatterns, CPP, RankNTypes, MagicHash, UnboxedTuples, MultiParamTypeClasses, FlexibleInstances, FlexibleContexts, DeriveDataTypeable, UnliftedFFITypes #-}
+#if __GLASGOW_HASKELL__ >= 708
+{-# LANGUAGE RoleAnnotations #-}
+#endif
 {-# OPTIONS_HADDOCK hide #-}
 
 -----------------------------------------------------------------------------
@@ -29,8 +32,9 @@ import GHC.Arr          ( STArray )
 import qualified GHC.Arr as Arr
 import qualified GHC.Arr as ArrST
 import GHC.ST           ( ST(..), runST )
-import GHC.Base
-import GHC.Ptr          ( Ptr(..), FunPtr(..), nullPtr, nullFunPtr )
+import GHC.Base         ( IO(..) )
+import GHC.Exts
+import GHC.Ptr          ( nullPtr, nullFunPtr )
 import GHC.Stable       ( StablePtr(..) )
 #if !MIN_VERSION_base(4,6,0)
 import GHC.Exts         ( Word(..) )
@@ -401,6 +405,10 @@ instance IArray Arr.Array e where
 --
 data UArray i e = UArray !i !i !Int ByteArray#
                   deriving Typeable
+#if __GLASGOW_HASKELL__ >= 708
+-- There are class-based invariants on both parameters. See also #9220.
+type role UArray nominal nominal
+#endif
 
 {-# INLINE unsafeArrayUArray #-}
 unsafeArrayUArray :: (MArray (STUArray s) e (ST s), Ix i)
@@ -984,6 +992,11 @@ instance MArray (STArray s) e (Lazy.ST s) where
 -- 'STArray' provides.
 data STUArray s i e = STUArray !i !i !Int (MutableByteArray# s)
                       deriving Typeable
+#if __GLASGOW_HASKELL__ >= 708
+-- The "ST" parameter must be nominal for the safety of the ST trick.
+-- The other parameters have class constraints. See also #9220.
+type role STUArray nominal nominal nominal
+#endif
 
 instance Eq (STUArray s i e) where
     STUArray _ _ _ arr1# == STUArray _ _ _ arr2# =
@@ -1377,7 +1390,11 @@ freeze marr = do
   -- use the safe array creation function here.
   return (listArray (l,u) es)
 
+#if __GLASGOW_HASKELL__ >= 711
+freezeSTUArray :: STUArray s i e -> ST s (UArray i e)
+#else
 freezeSTUArray :: Ix i => STUArray s i e -> ST s (UArray i e)
+#endif
 freezeSTUArray (STUArray l u n marr#) = ST $ \s1# ->
     case sizeofMutableByteArray# marr#  of { n# ->
     case newByteArray# n# s1#           of { (# s2#, marr'# #) ->
@@ -1452,7 +1469,11 @@ thaw arr = case bounds arr of
               | i <- [0 .. n - 1]]
     return marr
 
+#if __GLASGOW_HASKELL__ >= 711
+thawSTUArray :: UArray i e -> ST s (STUArray s i e)
+#else
 thawSTUArray :: Ix i => UArray i e -> ST s (STUArray s i e)
+#endif
 thawSTUArray (UArray l u n arr#) = ST $ \s1# ->
     case sizeofByteArray# arr#          of { n# ->
     case newByteArray# n# s1#           of { (# s2#, marr# #) ->
@@ -1512,7 +1533,11 @@ unsafeThaw :: (Ix i, IArray a e, MArray b e m) => a i e -> m (b i e)
 unsafeThaw = thaw
 
 {-# INLINE unsafeThawSTUArray #-}
+#if __GLASGOW_HASKELL__ >= 711
+unsafeThawSTUArray :: UArray i e -> ST s (STUArray s i e)
+#else
 unsafeThawSTUArray :: Ix i => UArray i e -> ST s (STUArray s i e)
+#endif
 unsafeThawSTUArray (UArray l u n marr#) =
     return (STUArray l u n (unsafeCoerce# marr#))
 
@@ -1522,7 +1547,11 @@ unsafeThawSTUArray (UArray l u n marr#) =
     #-}
 
 {-# INLINE unsafeThawIOArray #-}
+#if __GLASGOW_HASKELL__ >= 711
+unsafeThawIOArray :: Arr.Array ix e -> IO (IOArray ix e)
+#else
 unsafeThawIOArray :: Ix ix => Arr.Array ix e -> IO (IOArray ix e)
+#endif
 unsafeThawIOArray arr = stToIO $ do
     marr <- ArrST.unsafeThawSTArray arr
     return (IOArray marr)
@@ -1531,7 +1560,11 @@ unsafeThawIOArray arr = stToIO $ do
 "unsafeThaw/IOArray"  unsafeThaw = unsafeThawIOArray
     #-}
 
+#if __GLASGOW_HASKELL__ >= 711
+thawIOArray :: Arr.Array ix e -> IO (IOArray ix e)
+#else
 thawIOArray :: Ix ix => Arr.Array ix e -> IO (IOArray ix e)
+#endif
 thawIOArray arr = stToIO $ do
     marr <- ArrST.thawSTArray arr
     return (IOArray marr)
@@ -1540,7 +1573,11 @@ thawIOArray arr = stToIO $ do
 "thaw/IOArray"  thaw = thawIOArray
     #-}
 
+#if __GLASGOW_HASKELL__ >= 711
+freezeIOArray :: IOArray ix e -> IO (Arr.Array ix e)
+#else
 freezeIOArray :: Ix ix => IOArray ix e -> IO (Arr.Array ix e)
+#endif
 freezeIOArray (IOArray marr) = stToIO (ArrST.freezeSTArray marr)
 
 {-# RULES
@@ -1548,7 +1585,11 @@ freezeIOArray (IOArray marr) = stToIO (ArrST.freezeSTArray marr)
     #-}
 
 {-# INLINE unsafeFreezeIOArray #-}
+#if __GLASGOW_HASKELL__ >= 711
+unsafeFreezeIOArray :: IOArray ix e -> IO (Arr.Array ix e)
+#else
 unsafeFreezeIOArray :: Ix ix => IOArray ix e -> IO (Arr.Array ix e)
+#endif
 unsafeFreezeIOArray (IOArray marr) = stToIO (ArrST.unsafeFreezeSTArray marr)
 
 {-# RULES
