@@ -1,4 +1,6 @@
 {-# LANGUAGE Trustworthy #-}
+{-# LANGUAGE NoImplicitPrelude #-}
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Arrow
@@ -41,11 +43,11 @@ module Control.Arrow (
     ArrowLoop(..)
     ) where
 
-import Prelude hiding (id,(.))
-
-import Control.Monad
+import Data.Tuple ( fst, snd, uncurry )
+import Data.Either
 import Control.Monad.Fix
 import Control.Category
+import GHC.Base hiding ( (.), id )
 
 infixr 5 <+>
 infixr 3 ***
@@ -57,7 +59,7 @@ infixr 1 ^<<, <<^
 
 -- | The basic arrow class.
 --
--- Minimal complete definition: 'arr' and 'first', satisfying the laws
+-- Instances should satisfy the following laws:
 --
 --  * @'arr' id = 'id'@
 --
@@ -191,7 +193,7 @@ instance MonadPlus m => ArrowPlus (Kleisli m) where
 -- | Choice, for arrows that support it.  This class underlies the
 -- @if@ and @case@ constructs in arrow notation.
 --
--- Minimal complete definition: 'left', satisfying the laws
+-- Instances should satisfy the following laws:
 --
 --  * @'left' ('arr' f) = 'arr' ('left' f)@
 --
@@ -304,11 +306,19 @@ newtype ArrowMonad a b = ArrowMonad (a () b)
 instance Arrow a => Functor (ArrowMonad a) where
     fmap f (ArrowMonad m) = ArrowMonad $ m >>> arr f
 
+instance Arrow a => Applicative (ArrowMonad a) where
+   pure x = ArrowMonad (arr (const x))
+   ArrowMonad f <*> ArrowMonad x = ArrowMonad (f &&& x >>> arr (uncurry id))
+
 instance ArrowApply a => Monad (ArrowMonad a) where
     return x = ArrowMonad (arr (\_ -> x))
     ArrowMonad m >>= f = ArrowMonad $
         m >>> arr (\x -> let ArrowMonad h = f x in (h, ())) >>> app
 
+instance ArrowPlus a => Alternative (ArrowMonad a) where
+   empty = ArrowMonad zeroArrow
+   ArrowMonad x <|> ArrowMonad y = ArrowMonad (x <+> y)
+
 instance (ArrowApply a, ArrowPlus a) => MonadPlus (ArrowMonad a) where
    mzero = ArrowMonad zeroArrow
    ArrowMonad x `mplus` ArrowMonad y = ArrowMonad (x <+> y)
