@@ -1,9 +1,7 @@
 {-# LANGUAGE Trustworthy #-}
-{-# LANGUAGE CPP #-}
-
-#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 706
-{-# LANGUAGE PolyKinds, GADTs #-}
-#endif
+{-# LANGUAGE GADTs #-}
+{-# LANGUAGE NoImplicitPrelude #-}
+{-# LANGUAGE PolyKinds #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -15,11 +13,11 @@
 -- Stability   :  experimental
 -- Portability :  portable
 
--- http://hackage.haskell.org/trac/ghc/ticket/1773
+-- http://ghc.haskell.org/trac/ghc/ticket/1773
 
 module Control.Category where
 
-import qualified Prelude
+import qualified GHC.Base (id,(.))
 import Data.Type.Coercion
 import Data.Type.Equality
 import GHC.Prim (coerce)
@@ -46,8 +44,8 @@ class Category cat where
  #-}
 
 instance Category (->) where
-    id = Prelude.id
-    (.) = (Prelude..)
+    id = GHC.Base.id
+    (.) = (GHC.Base..)
 
 instance Category (:~:) where
   id          = Refl
