@@ -4,7 +4,6 @@
            , UnboxedTuples
            , ScopedTypeVariables
   #-}
-{-# OPTIONS_GHC -fno-warn-unused-imports #-}
 {-# OPTIONS_GHC -fno-warn-deprecations #-}
 -- kludge for the Control.Concurrent.QSem, Control.Concurrent.QSemN
 -- and Control.Concurrent.SampleVar imports.
@@ -104,27 +103,24 @@ module Control.Concurrent (
 
     ) where
 
-import Prelude
-
 import Control.Exception.Base as Exception
 
-import GHC.Exception
 import GHC.Conc hiding (threadWaitRead, threadWaitWrite,
                         threadWaitReadSTM, threadWaitWriteSTM)
-import qualified GHC.Conc
-import GHC.IO           ( IO(..), unsafeInterleaveIO, unsafeUnmask )
+import GHC.IO           ( unsafeUnmask )
 import GHC.IORef        ( newIORef, readIORef, writeIORef )
 import GHC.Base
 
 import System.Posix.Types ( Fd )
 import Foreign.StablePtr
 import Foreign.C.Types
-import Control.Monad
 
 #ifdef mingw32_HOST_OS
 import Foreign.C
 import System.IO
-import Data.Maybe (Maybe(..))
+import Data.Functor ( void )
+#else
+import qualified GHC.Conc
 #endif
 
 import Control.Concurrent.MVar
@@ -195,7 +191,7 @@ attribute will block all other threads.
 -- This function is useful for informing the parent when a child
 -- terminates, for example.
 --
--- /Since: 4.6.0.0/
+-- @since 4.6.0.0
 forkFinally :: IO a -> (Either SomeException a -> IO ()) -> IO ThreadId
 forkFinally action and_then =
   mask $ \restore ->
@@ -361,8 +357,8 @@ is /bound/, an unbound thread is created temporarily using 'forkIO'.
 
 Use this function /only/ in the rare case that you have actually observed a
 performance loss due to the use of bound threads. A program that
-doesn't need it's main thread to be bound and makes /heavy/ use of concurrency
-(e.g. a web server), might want to wrap it's @main@ action in
+doesn't need its main thread to be bound and makes /heavy/ use of concurrency
+(e.g. a web server), might want to wrap its @main@ action in
 @runInUnboundThread@.
 
 Note that exceptions which are thrown to the current thread are thrown in turn
@@ -435,7 +431,7 @@ threadWaitWrite fd
 -- is an IO action that can be used to deregister interest
 -- in the file descriptor.
 --
--- /Since: 4.7.0.0/
+-- @since 4.7.0.0
 threadWaitReadSTM :: Fd -> IO (STM (), IO ())
 threadWaitReadSTM fd
 #ifdef mingw32_HOST_OS
@@ -459,7 +455,7 @@ threadWaitReadSTM fd
 -- is an IO action that can be used to deregister interest
 -- in the file descriptor.
 --
--- /Since: 4.7.0.0/
+-- @since 4.7.0.0
 threadWaitWriteSTM :: Fd -> IO (STM (), IO ())
 threadWaitWriteSTM fd
 #ifdef mingw32_HOST_OS
