@@ -1,4 +1,5 @@
 {-# LANGUAGE Trustworthy #-}
+{-# LANGUAGE NoImplicitPrelude #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -22,10 +23,13 @@ module Control.Monad.Fix (
         fix
   ) where
 
-import Prelude
-import System.IO
-import Data.Function (fix)
+import Data.Either
+import Data.Function ( fix )
+import Data.Maybe
+import GHC.Base ( Monad, error, (.) )
+import GHC.List ( head, tail )
 import GHC.ST
+import System.IO
 
 -- | Monads having fixed points with a \'knot-tying\' semantics.
 -- Instances of 'MonadFix' should satisfy the following laws:
@@ -65,7 +69,7 @@ instance MonadFix [] where
                (x:_) -> x : mfix (tail . f)
 
 instance MonadFix IO where
-    mfix = fixIO 
+    mfix = fixIO
 
 instance MonadFix ((->) r) where
     mfix f = \ r -> let a = f a r in a
