@@ -1,6 +1,5 @@
 {-# LANGUAGE Unsafe #-}
-{-# LANGUAGE DeriveDataTypeable #-}
-{-# LANGUAGE StandaloneDeriving #-}
+{-# LANGUAGE NoImplicitPrelude #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -17,11 +16,15 @@
 -- More in-depth information can be found on the
 -- <https://ghc.haskell.org/trac/ghc/wiki/Roles Roles wiki page>
 --
--- /Since: 4.7.0.0/
+-- @since 4.7.0.0
 -----------------------------------------------------------------------------
 
 module Data.Coerce
         ( -- * Safe coercions
           coerce, Coercible,
         ) where
-import GHC.Prim (coerce, Coercible)
+import GHC.Prim (coerce)
+import GHC.Types (Coercible)
+
+import GHC.Base () -- for build ordering
+
