@@ -1,26 +1,26 @@
-{-# LANGUAGE Trustworthy, FlexibleInstances #-}
-{-# LANGUAGE RankNTypes, ScopedTypeVariables, PolyKinds #-}
-{-# LANGUAGE StandaloneDeriving, DeriveDataTypeable, TypeOperators,
-             GADTs #-}
+{-# LANGUAGE Trustworthy #-}
+{-# LANGUAGE RankNTypes, ScopedTypeVariables, PolyKinds, StandaloneDeriving,
+             AutoDeriveTypeable, TypeOperators, GADTs, FlexibleInstances #-}
+{-# LANGUAGE NoImplicitPrelude #-}
 
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Data
 -- Copyright   :  (c) The University of Glasgow, CWI 2001--2004
 -- License     :  BSD-style (see the file libraries/base/LICENSE)
--- 
+--
 -- Maintainer  :  libraries@haskell.org
 -- Stability   :  experimental
 -- Portability :  non-portable (local universal quantification)
 --
--- \"Scrap your boilerplate\" --- Generic programming in Haskell.
--- See <http://www.cs.vu.nl/boilerplate/>. This module provides
--- the 'Data' class with its primitives for generic programming, along
--- with instances for many datatypes. It corresponds to a merge between
--- the previous "Data.Generics.Basics" and almost all of 
--- "Data.Generics.Instances". The instances that are not present
--- in this module were moved to the @Data.Generics.Instances@ module
--- in the @syb@ package.
+-- \"Scrap your boilerplate\" --- Generic programming in Haskell.  See
+-- <http://www.haskell.org/haskellwiki/Research_papers/Generics#Scrap_your_boilerplate.21>.
+-- This module provides the 'Data' class with its primitives for
+-- generic programming, along with instances for many datatypes. It
+-- corresponds to a merge between the previous "Data.Generics.Basics"
+-- and almost all of "Data.Generics.Instances". The instances that are
+-- not present in this module were moved to the
+-- @Data.Generics.Instances@ module in the @syb@ package.
 --
 -- For more information, please visit the new
 -- SYB wiki: <http://www.cs.uu.nl/wiki/bin/view/GenericProgramming/SYB>.
@@ -40,7 +40,7 @@ module Data.Data (
                 dataTypeOf,
                 dataCast1,      -- mediate types and unary type constructors
                 dataCast2,      -- mediate types and binary type constructors
-                -- Generic maps defined in terms of gfoldl 
+                -- Generic maps defined in terms of gfoldl
                 gmapT,
                 gmapQ,
                 gmapQl,
@@ -106,20 +106,25 @@ module Data.Data (
 
 ------------------------------------------------------------------------------
 
-import Prelude -- necessary to get dependencies right
-
-import Data.Typeable
+import Data.Either
+import Data.Eq
 import Data.Maybe
+import Data.Ord
+import Data.Typeable
 import Data.Version( Version(..) )
-import Control.Monad
+import GHC.Base
+import GHC.List
+import GHC.Num
+import GHC.Read
+import GHC.Show
+import Text.Read( reads )
 
 -- Imports for the instances
 import Data.Int              -- So we can give Data instance for Int8, ...
 import Data.Type.Coercion
 import Data.Word             -- So we can give Data instance for Word8, ...
-import GHC.Real( Ratio(..) ) -- So we can give Data instance for Ratio
+import GHC.Real              -- So we can give Data instance for Ratio
 --import GHC.IOBase            -- So we can give Data instance for IO, Handle
-import GHC.Prim
 import GHC.Ptr               -- So we can give Data instance for Ptr
 import GHC.ForeignPtr        -- So we can give Data instance for ForeignPtr
 --import GHC.Stable            -- So we can give Data instance for StablePtr
@@ -323,7 +328,7 @@ class Typeable a => Data a where
 
   -- | A generic query that processes the immediate subterms and returns a list
   -- of results.  The list is given in the same order as originally specified
-  -- in the declaratoin of the data constructors.
+  -- in the declaration of the data constructors.
   gmapQ :: (forall d. Data d => d -> u) -> a -> [u]
   gmapQ f = gmapQr (:) [] f
 
@@ -345,10 +350,10 @@ class Typeable a => Data a where
   -- injection and projection using 'return' and '>>='.
   gmapM :: forall m. Monad m => (forall d. Data d => d -> m d) -> a -> m a
 
-  -- Use immediately the monad datatype constructor 
+  -- Use immediately the monad datatype constructor
   -- to instantiate the type constructor c in the type of gfoldl,
   -- so injection and projection is done by return and >>=.
-  --  
+  --
   gmapM f = gfoldl k return
     where
       k :: Data d => m (d -> b) -> d -> m b
@@ -385,8 +390,8 @@ this end, we couple the monadic computation with a Boolean.
 
 {-
 
-We use the same pairing trick as for gmapMp, 
-i.e., we use an extra Bool component to keep track of the 
+We use the same pairing trick as for gmapMp,
+i.e., we use an extra Bool component to keep track of the
 fact whether an immediate subterm was processed successfully.
 However, we cut of mapping over subterms once a first subterm
 was transformed successfully.
@@ -449,7 +454,7 @@ fromConstrB f = unID . gunfold k z
  where
   k :: forall b r. Data b => ID (b -> r) -> ID r
   k c = ID (unID c f)
- 
+
   z :: forall r. r -> ID r
   z = ID
 
@@ -619,7 +624,7 @@ dataTypeConstrs dt = case datarep dt of
 
 
 -- | Gets the field labels of a constructor.  The list of labels
--- is returned in the same order as they were given in the original 
+-- is returned in the same order as they were given in the original
 -- constructor declaration.
 constrFields :: Constr -> [String]
 constrFields = confields
@@ -634,7 +639,7 @@ constrFixity = confixity
 ------------------------------------------------------------------------------
 --
 --      From strings to constr's and vice versa: all data types
---      
+--
 ------------------------------------------------------------------------------
 
 
@@ -777,12 +782,12 @@ mkCharConstr dt c = case datarep dt of
 
 ------------------------------------------------------------------------------
 --
---      Non-representations for non-presentable types
+--      Non-representations for non-representable types
 --
 ------------------------------------------------------------------------------
 
 
--- | Constructs a non-representation for a non-presentable type
+-- | Constructs a non-representation for a non-representable type
 mkNoRepType :: String -> DataType
 mkNoRepType str = DataType
                         { tycon   = str
@@ -1060,9 +1065,9 @@ ratioDataType :: DataType
 ratioDataType = mkDataType "GHC.Real.Ratio" [ratioConstr]
 
 instance (Data a, Integral a) => Data (Ratio a) where
-  gfoldl k z (a :% b) = z (:%) `k` a `k` b
+  gfoldl k z (a :% b) = z (%) `k` a `k` b
   toConstr _ = ratioConstr
-  gunfold k z c | constrIndex c == 1 = k (k (z (:%)))
+  gunfold k z c | constrIndex c == 1 = k (k (z (%)))
   gunfold _ _ _ = error "Data.Data.gunfold(Ratio)"
   dataTypeOf _  = ratioDataType
 
@@ -1313,7 +1318,7 @@ instance (Data a, Typeable a) => Data (ForeignPtr a) where
   dataCast1 x  = gcast1 x
 
 ------------------------------------------------------------------------------
--- The Data instance for Array preserves data abstraction at the cost of 
+-- The Data instance for Array preserves data abstraction at the cost of
 -- inefficiency. We omit reflection services for the sake of data abstraction.
 instance (Typeable a, Data a, Data b, Ix a) => Data (Array a b)
  where
