@@ -1,4 +1,5 @@
-{-# LANGUAGE NoImplicitPrelude, Trustworthy #-}
+{-# LANGUAGE Trustworthy #-}
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE PolyKinds #-}
 
 -----------------------------------------------------------------------------
@@ -12,7 +13,7 @@
 --
 -- Definition of a Proxy type (poly-kinded in GHC)
 --
--- /Since: 4.7.0.0/
+-- @since 4.7.0.0
 -----------------------------------------------------------------------------
 
 module Data.Proxy
@@ -34,6 +35,10 @@ data Proxy t = Proxy
 -- There are no instances for this because it is intended at the kind level only
 data KProxy (t :: *) = KProxy
 
+-- It's common to use (undefined :: Proxy t) and (Proxy :: Proxy t)
+-- interchangeably, so all of these instances are hand-written to be
+-- lazy in Proxy arguments.
+
 instance Eq (Proxy s) where
   _ == _ = True
 
@@ -69,10 +74,21 @@ instance Bounded (Proxy s) where
     minBound = Proxy
     maxBound = Proxy
 
+instance Monoid (Proxy s) where
+    mempty = Proxy
+    mappend _ _ = Proxy
+    mconcat _ = Proxy
+
 instance Functor Proxy where
     fmap _ _ = Proxy
     {-# INLINE fmap #-}
 
+instance Applicative Proxy where
+    pure _ = Proxy
+    {-# INLINE pure #-}
+    _ <*> _ = Proxy
+    {-# INLINE (<*>) #-}
+
 instance Monad Proxy where
     return _ = Proxy
     {-# INLINE return #-}
