@@ -1,12 +1,14 @@
 {-# LANGUAGE Unsafe #-}
-{-# LANGUAGE MagicHash, UnboxedTuples #-}
+{-# LANGUAGE MagicHash #-}
+{-# LANGUAGE NoImplicitPrelude #-}
+{-# LANGUAGE UnboxedTuples #-}
 
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Debug.Trace
 -- Copyright   :  (c) The University of Glasgow 2001
 -- License     :  BSD-style (see the file libraries/base/LICENSE)
--- 
+--
 -- Maintainer  :  libraries@haskell.org
 -- Stability   :  provisional
 -- Portability :  portable
@@ -35,22 +37,21 @@ module Debug.Trace (
         -- $eventlog_tracing
         traceEvent,
         traceEventIO,
-        
+
         -- * Execution phase markers
         -- $markers
         traceMarker,
         traceMarkerIO,
   ) where
 
-import Prelude
 import System.IO.Unsafe
-import Control.Monad
 
 import Foreign.C.String
 import GHC.Base
 import qualified GHC.Foreign
 import GHC.IO.Encoding
 import GHC.Ptr
+import GHC.Show
 import GHC.Stack
 import Data.List
 
@@ -59,7 +60,11 @@ import Data.List
 -- The 'trace', 'traceShow' and 'traceIO' functions print messages to an output
 -- stream. They are intended for \"printf debugging\", that is: tracing the flow
 -- of execution and printing interesting values.
-
+--
+-- All these functions evaluate the message completely before printing
+-- it; so if the message is not fully defined, none of it will be
+-- printed.
+--
 -- The usual output stream is 'System.IO.stderr'. For Windows GUI applications
 -- (that have no stderr) the output is directed to the Windows debug console.
 -- Some implementations of these functions may decorate the string that\'s
@@ -68,7 +73,7 @@ import Data.List
 -- | The 'traceIO' function outputs the trace message from the IO monad.
 -- This sequences the output with respect to other IO actions.
 --
--- /Since: 4.5.0.0/
+-- @since 4.5.0.0
 traceIO :: String -> IO ()
 traceIO msg = do
     withCString "%s\n" $ \cfmt -> do
@@ -114,7 +119,7 @@ trace string expr = unsafePerformIO $ do
 {-|
 Like 'trace' but returns the message instead of a third value.
 
-/Since: 4.7.0.0/
+@since 4.7.0.0
 -}
 traceId :: String -> String
 traceId a = trace a a
@@ -138,7 +143,7 @@ traceShow = trace . show
 {-|
 Like 'traceShow' but returns the shown value instead of a third value.
 
-/Since: 4.7.0.0/
+@since 4.7.0.0
 -}
 traceShowId :: (Show a) => a -> a
 traceShowId a = trace (show a) a
@@ -154,7 +159,7 @@ monad, as 'traceIO' is in the 'IO' monad.
 >   y <- ...
 >   traceM $ "y: " ++ show y
 
-/Since: 4.7.0.0/
+@since 4.7.0.0
 -}
 traceM :: (Monad m) => String -> m ()
 traceM string = trace string $ return ()
@@ -168,7 +173,7 @@ Like 'traceM', but uses 'show' on the argument to convert it to a 'String'.
 >   y <- ...
 >   traceMShow $ x + y
 
-/Since: 4.7.0.0/
+@since 4.7.0.0
 -}
 traceShowM :: (Show a, Monad m) => a -> m ()
 traceShowM = traceM . show
@@ -182,7 +187,7 @@ traceShowM = traceM . show
 -- stack correspond to @SCC@ annotations, so it is a good idea to use
 -- @-fprof-auto@ or @-fprof-auto-calls@ to add SCC annotations automatically.
 --
--- /Since: 4.5.0.0/
+-- @since 4.5.0.0
 traceStack :: String -> a -> a
 traceStack str expr = unsafePerformIO $ do
    traceIO str
@@ -215,7 +220,7 @@ traceStack str expr = unsafePerformIO $ do
 -- duplicate events emitted if two CPUs simultaneously evaluate the same thunk
 -- that uses 'traceEvent'.
 --
--- /Since: 4.5.0.0/
+-- @since 4.5.0.0
 traceEvent :: String -> a -> a
 traceEvent msg expr = unsafeDupablePerformIO $ do
     traceEventIO msg
@@ -227,7 +232,7 @@ traceEvent msg expr = unsafeDupablePerformIO $ do
 -- Compared to 'traceEvent', 'traceEventIO' sequences the event with respect to
 -- other IO actions.
 --
--- /Since: 4.5.0.0/
+-- @since 4.5.0.0
 traceEventIO :: String -> IO ()
 traceEventIO msg =
   GHC.Foreign.withCString utf8 msg $ \(Ptr p) -> IO $ \s ->
@@ -245,7 +250,7 @@ traceEventIO msg =
 --
 -- Markers let us do this: we can annotate the program to emit a marker at
 -- an appropriate point during execution and then see that in a profile.
--- 
+--
 -- Currently this feature is only supported in GHC by the eventlog tracing
 -- system, but in future it may also be supported by the heap profiling or
 -- other profiling tools. These function exists for other Haskell
@@ -265,7 +270,7 @@ traceEventIO msg =
 -- duplicate events emitted if two CPUs simultaneously evaluate the same thunk
 -- that uses 'traceMarker'.
 --
--- /Since: 4.7.0.0/
+-- @since 4.7.0.0
 traceMarker :: String -> a -> a
 traceMarker msg expr = unsafeDupablePerformIO $ do
     traceMarkerIO msg
@@ -277,7 +282,7 @@ traceMarker msg expr = unsafeDupablePerformIO $ do
 -- Compared to 'traceMarker', 'traceMarkerIO' sequences the event with respect to
 -- other IO actions.
 --
--- /Since: 4.7.0.0/
+-- @since 4.7.0.0
 traceMarkerIO :: String -> IO ()
 traceMarkerIO msg =
   GHC.Foreign.withCString utf8 msg $ \(Ptr p) -> IO $ \s ->
