@@ -1,5 +1,6 @@
 {-# LANGUAGE Unsafe #-}
-{-# LANGUAGE MagicHash, UnboxedTuples, DeriveDataTypeable, TypeFamilies, MultiParamTypeClasses, FlexibleInstances #-}
+{-# LANGUAGE MagicHash, UnboxedTuples, AutoDeriveTypeable, TypeFamilies,
+             MultiParamTypeClasses, FlexibleInstances, NoImplicitPrelude #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -49,7 +50,7 @@ module GHC.Exts
         --
         -- | These are available from the /Trustworthy/ module "Data.Coerce" as well
         --
-        -- /Since: 4.7.0.0/
+        -- @since 4.7.0.0
         Data.Coerce.coerce, Data.Coerce.Coercible,
 
         -- * Transform comprehensions
@@ -71,19 +72,19 @@ module GHC.Exts
         IsList(..)
        ) where
 
-import Prelude
-
-import GHC.Prim hiding (coerce, Coercible)
-import GHC.Base hiding (coerce, Coercible) -- implicitly comes from GHC.Prim
+import GHC.Prim hiding (coerce, Constraint)
+import GHC.Base hiding (coerce) -- implicitly comes from GHC.Prim
 import GHC.Word
 import GHC.Int
 import GHC.Ptr
 import GHC.Stack
+
 import qualified Data.Coerce
 import Data.String
-import Data.List
+import Data.OldList
 import Data.Data
 import Data.Ord
+import Data.Version ( Version(..), makeVersion )
 import qualified Debug.Trace
 
 -- XXX This should really be in Data.Tuple, where the definitions are
@@ -126,9 +127,9 @@ traceEvent = Debug.Trace.traceEventIO
 
 
 {- **********************************************************************
-*									*
+*                                                                       *
 *              SpecConstr annotation                                    *
-*									*
+*                                                                       *
 ********************************************************************** -}
 
 -- Annotating a type with NoSpecConstr will make SpecConstr
@@ -143,15 +144,15 @@ data SpecConstrAnnotation = NoSpecConstr | ForceSpecConstr
 
 
 {- **********************************************************************
-*									*
+*                                                                       *
 *              The IsList class                                         *
-*									*
+*                                                                       *
 ********************************************************************** -}
 
 -- | The 'IsList' class and its methods are intended to be used in
 --   conjunction with the OverloadedLists extension.
 --
--- /Since: 4.7.0.0/
+-- @since 4.7.0.0
 class IsList l where
   -- | The 'Item' type function returns the type of items of the structure
   --   @l@.
@@ -177,3 +178,9 @@ instance IsList [a] where
   type (Item [a]) = a
   fromList = id
   toList = id
+
+-- | @since 4.8.0.0
+instance IsList Version where
+  type (Item Version) = Int
+  fromList = makeVersion
+  toList = versionBranch
