@@ -13,7 +13,7 @@
 -- Module      :  GHC.IO
 -- Copyright   :  (c) The University of Glasgow 1994-2002
 -- License     :  see libraries/base/LICENSE
--- 
+--
 -- Maintainer  :  cvs-ghc@haskell.org
 -- Stability   :  internal
 -- Portability :  non-portable (GHC Extensions)
@@ -34,7 +34,7 @@ module GHC.IO (
         FilePath,
 
         catchException, catchAny, throwIO,
-        mask, mask_, uninterruptibleMask, uninterruptibleMask_, 
+        mask, mask_, uninterruptibleMask, uninterruptibleMask_,
         MaskingState(..), getMaskingState,
         unsafeUnmask,
         onException, bracket, finally, evaluate
@@ -44,7 +44,6 @@ import GHC.Base
 import GHC.ST
 import GHC.Exception
 import GHC.Show
-import Data.Maybe
 
 import {-# SOURCE #-} GHC.IO.Exception ( userError )
 
@@ -62,7 +61,7 @@ system.  The following list may or may not be exhaustive:
 Compiler  - types of various primitives in PrimOp.lhs
 
 RTS       - forceIO (StgMiscClosures.hc)
-          - catchzh_fast, (un)?blockAsyncExceptionszh_fast, raisezh_fast 
+          - catchzh_fast, (un)?blockAsyncExceptionszh_fast, raisezh_fast
             (Exceptions.hc)
           - raiseAsync (Schedule.c)
 
@@ -130,8 +129,8 @@ different precautions:
         two side effects that were meant to be separate.  A good example
         is using multiple global variables (like @test@ in the example below).
 
-  * Make sure that the either you switch off let-floating (@-fno-full-laziness@), or that the 
-        call to 'unsafePerformIO' cannot float outside a lambda.  For example, 
+  * Make sure that the either you switch off let-floating (@-fno-full-laziness@), or that the
+        call to 'unsafePerformIO' cannot float outside a lambda.  For example,
         if you say:
         @
            f x = unsafePerformIO (newIORef [])
@@ -148,7 +147,7 @@ It is less well known that
 
 >     test :: IORef [a]
 >     test = unsafePerformIO $ newIORef []
->     
+>
 >     main = do
 >             writeIORef test [42]
 >             bang <- readIORef test
@@ -164,7 +163,7 @@ help of 'unsafePerformIO'.  So be careful!
 unsafePerformIO :: IO a -> a
 unsafePerformIO m = unsafeDupablePerformIO (noDuplicate >> m)
 
-{-| 
+{-|
 This version of 'unsafePerformIO' is more efficient
 because it omits the check that the IO is only being performed by a
 single thread.  Hence, when you use 'unsafeDupablePerformIO',
@@ -175,32 +174,40 @@ of the duplicated IO actions is only run partially, and then interrupted
 in the middle without an exception being raised. Therefore, functions
 like 'bracket' cannot be used safely within 'unsafeDupablePerformIO'.
 
-/Since: 4.4.0.0/
+@since 4.4.0.0
 -}
 {-# NOINLINE unsafeDupablePerformIO #-}
+    -- See Note [unsafeDupablePerformIO is NOINLINE]
 unsafeDupablePerformIO  :: IO a -> a
 unsafeDupablePerformIO (IO m) = lazy (case m realWorld# of (# _, r #) -> r)
+     -- See Note [unsafeDupablePerformIO has a lazy RHS]
 
+-- Note [unsafeDupablePerformIO is NOINLINE]
+-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 -- Why do we NOINLINE unsafeDupablePerformIO?  See the comment with
 -- GHC.ST.runST.  Essentially the issue is that the IO computation
 -- inside unsafePerformIO must be atomic: it must either all run, or
 -- not at all.  If we let the compiler see the application of the IO
 -- to realWorld#, it might float out part of the IO.
 
+-- Note [unsafeDupablePerformIO has a lazy RHS]
+-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 -- Why is there a call to 'lazy' in unsafeDupablePerformIO?
 -- If we don't have it, the demand analyser discovers the following strictness
 -- for unsafeDupablePerformIO:  C(U(AV))
 -- But then consider
---      unsafeDupablePerformIO (\s -> let r = f x in 
+--      unsafeDupablePerformIO (\s -> let r = f x in
 --                             case writeIORef v r s of (# s1, _ #) ->
---                             (# s1, r #)
+--                             (# s1, r #) )
 -- The strictness analyser will find that the binding for r is strict,
--- (because of uPIO's strictness sig), and so it'll evaluate it before 
--- doing the writeIORef.  This actually makes tests/lib/should_run/memo002
--- get a deadlock!  
+-- (because of uPIO's strictness sig), and so it'll evaluate it before
+-- doing the writeIORef.  This actually makes libraries/base/tests/memo002
+-- get a deadlock, where we specifically wanted to write a lazy thunk
+-- into the ref cell.
 --
 -- Solution: don't expose the strictness of unsafeDupablePerformIO,
 --           by hiding it with 'lazy'
+-- But see discussion in Trac #9390 (comment:33)
 
 {-|
 'unsafeInterleaveIO' allows 'IO' computation to be deferred lazily.
@@ -238,7 +245,7 @@ unsafeDupableInterleaveIO (IO m)
                 in
                 (# s, r #))
 
-{-| 
+{-|
 Ensures that the suspensions under evaluation by the current thread
 are unique; that is, the current thread is not evaluating anything
 that is also under evaluation by another thread that has also executed
@@ -341,7 +348,7 @@ blockUninterruptible (IO io) = IO $ maskUninterruptible# io
 -- exception is received.
 data MaskingState
   = Unmasked -- ^ asynchronous exceptions are unmasked (the normal state)
-  | MaskedInterruptible 
+  | MaskedInterruptible
       -- ^ the state during 'mask': asynchronous exceptions are masked, but blocking operations may still be interrupted
   | MaskedUninterruptible
       -- ^ the state during 'uninterruptibleMask': asynchronous exceptions are masked, and blocking operations may not be interrupted
@@ -349,7 +356,7 @@ data MaskingState
 
 -- | Returns the 'MaskingState' for the current thread.
 getMaskingState :: IO MaskingState
-getMaskingState  = IO $ \s -> 
+getMaskingState  = IO $ \s ->
   case getMaskingState# s of
      (# s', i #) -> (# s', case i of
                              0# -> Unmasked
@@ -395,13 +402,14 @@ onException io what = io `catchException` \e -> do _ <- what
 -- state if the masked thread /blocks/ in certain ways; see
 -- "Control.Exception#interruptible".
 --
--- Threads created by 'Control.Concurrent.forkIO' inherit the masked
--- state from the parent; that is, to start a thread in blocked mode,
+-- Threads created by 'Control.Concurrent.forkIO' inherit the
+-- 'MaskingState' from the parent; that is, to start a thread in the
+-- 'MaskedInterruptible' state,
 -- use @mask_ $ forkIO ...@.  This is particularly useful if you need
 -- to establish an exception handler in the forked thread before any
 -- asynchronous exceptions are received.  To create a a new thread in
 -- an unmasked state use 'Control.Concurrent.forkIOUnmasked'.
--- 
+--
 mask  :: ((forall a. IO a -> IO a) -> IO b) -> IO b
 
 -- | Like 'mask', but does not pass a @restore@ action to the argument.
@@ -428,8 +436,9 @@ mask_ io = mask $ \_ -> io
 mask io = do
   b <- getMaskingState
   case b of
-    Unmasked -> block $ io unblock
-    _        -> io id
+    Unmasked              -> block $ io unblock
+    MaskedInterruptible   -> io block
+    MaskedUninterruptible -> io blockUninterruptible
 
 uninterruptibleMask_ io = uninterruptibleMask $ \_ -> io
 
@@ -438,7 +447,7 @@ uninterruptibleMask io = do
   case b of
     Unmasked              -> blockUninterruptible $ io unblock
     MaskedInterruptible   -> blockUninterruptible $ io block
-    MaskedUninterruptible -> io id
+    MaskedUninterruptible -> io blockUninterruptible
 
 bracket
         :: IO a         -- ^ computation to run first (\"acquire resource\")
