@@ -35,7 +35,6 @@ import Data.Maybe
 import GHC.IntWord64
 #endif
 
--- import {-# SOURCE #-} GHC.Exception
 import GHC.Base
 import GHC.Enum
 import GHC.Num
@@ -43,7 +42,6 @@ import GHC.Real
 import GHC.Read
 import GHC.Arr
 import GHC.Show
-import GHC.Float ()     -- for RealFrac methods
 
 ------------------------------------------------------------------------
 -- type Word8
@@ -154,6 +152,8 @@ instance Bits Word8 where
 
 instance FiniteBits Word8 where
     finiteBitSize _ = 8
+    countLeadingZeros  (W8# x#) = I# (word2Int# (clz8# x#))
+    countTrailingZeros (W8# x#) = I# (word2Int# (ctz8# x#))
 
 {-# RULES
 "fromIntegral/Word8->Word8"   fromIntegral = id :: Word8 -> Word8
@@ -301,10 +301,12 @@ instance Bits Word16 where
 
 instance FiniteBits Word16 where
     finiteBitSize _ = 16
+    countLeadingZeros  (W16# x#) = I# (word2Int# (clz16# x#))
+    countTrailingZeros (W16# x#) = I# (word2Int# (ctz16# x#))
 
 -- | Swap bytes in 'Word16'.
 --
--- /Since: 4.7.0.0/
+-- @since 4.7.0.0
 byteSwap16 :: Word16 -> Word16
 byteSwap16 (W16# w#) = W16# (narrow16Word# (byteSwap16# w#))
 
@@ -495,6 +497,8 @@ instance Bits Word32 where
 
 instance FiniteBits Word32 where
     finiteBitSize _ = 32
+    countLeadingZeros  (W32# x#) = I# (word2Int# (clz32# x#))
+    countTrailingZeros (W32# x#) = I# (word2Int# (ctz32# x#))
 
 {-# RULES
 "fromIntegral/Word8->Word32"   fromIntegral = \(W8# x#) -> W32# x#
@@ -534,7 +538,7 @@ instance Read Word32 where
 
 -- | Reverse order of bytes in 'Word32'.
 --
--- /Since: 4.7.0.0/
+-- @since 4.7.0.0
 byteSwap32 :: Word32 -> Word32
 byteSwap32 (W32# w#) = W32# (narrow32Word# (byteSwap32# w#))
 
@@ -767,6 +771,8 @@ uncheckedShiftRL64# = uncheckedShiftRL#
 
 instance FiniteBits Word64 where
     finiteBitSize _ = 64
+    countLeadingZeros  (W64# x#) = I# (word2Int# (clz64# x#))
+    countTrailingZeros (W64# x#) = I# (word2Int# (ctz64# x#))
 
 instance Show Word64 where
     showsPrec p x = showsPrec p (toInteger x)
@@ -788,7 +794,7 @@ instance Read Word64 where
 
 -- | Reverse order of bytes in 'Word64'.
 --
--- /Since: 4.7.0.0/
+-- @since 4.7.0.0
 #if WORD_SIZE_IN_BITS < 64
 byteSwap64 :: Word64 -> Word64
 byteSwap64 (W64# w#) = W64# (byteSwap64# w#)
