@@ -1,12 +1,12 @@
 {-# LANGUAGE Trustworthy #-}
-{-# LANGUAGE CPP, NoImplicitPrelude #-}
+{-# LANGUAGE CPP, NoImplicitPrelude, CApiFFI #-}
 
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  System.IO
 -- Copyright   :  (c) The University of Glasgow 2001
 -- License     :  BSD-style (see the file libraries/base/LICENSE)
--- 
+--
 -- Maintainer  :  libraries@haskell.org
 -- Stability   :  stable
 -- Portability :  portable
@@ -181,21 +181,21 @@ module System.IO (
     -- 'readFile'), where a decoding error merely results in
     -- termination of the character stream, as with other I/O errors.
 
-    hSetEncoding, 
+    hSetEncoding,
     hGetEncoding,
 
     -- ** Unicode encodings
-    TextEncoding, 
+    TextEncoding,
     latin1,
     utf8, utf8_bom,
     utf16, utf16le, utf16be,
-    utf32, utf32le, utf32be, 
+    utf32, utf32le, utf32be,
     localeEncoding,
     char8,
     mkTextEncoding,
 
     -- * Newline conversion
-    
+
     -- | In Haskell, a newline is always represented by the character
     -- '\n'.  However, in files and external character streams, a
     -- newline may be represented by another character sequence, such
@@ -213,16 +213,15 @@ module System.IO (
     --
     -- Binary-mode 'Handle's do no newline translation at all.
     --
-    hSetNewlineMode, 
-    Newline(..), nativeNewline, 
-    NewlineMode(..), 
+    hSetNewlineMode,
+    Newline(..), nativeNewline,
+    NewlineMode(..),
     noNewlineTranslation, universalNewlineMode, nativeNewlineMode,
   ) where
 
 import Control.Exception.Base
 
 import Data.Bits
-import Data.List
 import Data.Maybe
 import Foreign.C.Error
 #ifdef mingw32_HOST_OS
@@ -233,6 +232,7 @@ import System.Posix.Internals
 import System.Posix.Types
 
 import GHC.Base
+import GHC.List
 import GHC.IO hiding ( bracket, onException )
 import GHC.IO.IOMode
 import GHC.IO.Handle.FD
@@ -241,7 +241,6 @@ import GHC.IO.Handle
 import GHC.IO.Handle.Text ( hGetBufSome, hPutStrLn )
 import GHC.IO.Exception ( userError )
 import GHC.IO.Encoding
-import GHC.Num
 import Text.Read
 import GHC.Show
 import GHC.MVar
@@ -359,7 +358,7 @@ localeEncoding = initLocaleEncoding
 
 -- | Computation 'hReady' @hdl@ indicates whether at least one item is
 -- available for input from handle @hdl@.
--- 
+--
 -- This operation may fail with:
 --
 --  * 'System.IO.Error.isEOFError' if the end of file has been reached.
@@ -454,19 +453,17 @@ openBinaryTempFile tmp_dir template
 openTempFileWithDefaultPermissions :: FilePath -> String
                                    -> IO (FilePath, Handle)
 openTempFileWithDefaultPermissions tmp_dir template
-    = openTempFile' "openBinaryTempFile" tmp_dir template False 0o666
+    = openTempFile' "openTempFileWithDefaultPermissions" tmp_dir template False 0o666
 
 -- | Like 'openBinaryTempFile', but uses the default file permissions
 openBinaryTempFileWithDefaultPermissions :: FilePath -> String
                                          -> IO (FilePath, Handle)
 openBinaryTempFileWithDefaultPermissions tmp_dir template
-    = openTempFile' "openBinaryTempFile" tmp_dir template True 0o666
+    = openTempFile' "openBinaryTempFileWithDefaultPermissions" tmp_dir template True 0o666
 
 openTempFile' :: String -> FilePath -> String -> Bool -> CMode
               -> IO (FilePath, Handle)
-openTempFile' loc tmp_dir template binary mode = do
-  pid <- c_getpid
-  findTempName pid
+openTempFile' loc tmp_dir template binary mode = findTempName
   where
     -- We split off the last extension, so we can use .foo.ext files
     -- for temporary files (hidden on Unix OSes). Unfortunately we're
@@ -485,10 +482,13 @@ openTempFile' loc tmp_dir template binary mode = do
          -- beginning with '.' as the second component.
          _                      -> error "bug in System.IO.openTempFile"
 
-    findTempName x = do
+    findTempName = do
+      rs <- rand_string
+      let filename = prefix ++ rs ++ suffix
+          filepath = tmp_dir `combine` filename
       r <- openNewFile filepath binary mode
       case r of
-        FileExists -> findTempName (x + 1)
+        FileExists -> findTempName
         OpenNewError errno -> ioError (errnoToIOError loc errno Nothing (Just tmp_dir))
         NewFileCreated fd -> do
           (fD,fd_type) <- FD.mkFD fd ReadWriteMode Nothing{-no stat-}
@@ -501,9 +501,6 @@ openTempFile' loc tmp_dir template binary mode = do
           return (filepath, h)
 
       where
-        filename        = prefix ++ show x ++ suffix
-        filepath        = tmp_dir `combine` filename
-
         -- XXX bits copied from System.FilePath, since that's not available here
         combine a b
                   | null b = a
@@ -511,6 +508,16 @@ openTempFile' loc tmp_dir template binary mode = do
                   | last a == pathSeparator = a ++ b
                   | otherwise = a ++ [pathSeparator] ++ b
 
+-- int rand(void) from <stdlib.h>, limited by RAND_MAX (small value, 32768)
+foreign import capi "stdlib.h rand" c_rand :: IO CInt
+
+-- build large digit-alike number
+rand_string :: IO String
+rand_string = do
+  r1 <- c_rand
+  r2 <- c_rand
+  return $ show r1 ++ show r2
+
 data OpenNewFileResult
   = NewFileCreated CInt
   | FileExists
