@@ -19,7 +19,6 @@ module System.Mem
        , performMajorGC
        , performMinorGC
        ) where
-import Prelude
 
 -- | Triggers an immediate garbage collection.
 performGC :: IO ()
@@ -27,10 +26,10 @@ performGC = performMajorGC
 
 -- | Triggers an immediate garbage collection.
 --
--- /Since: 4.7.0.0/
+-- @since 4.7.0.0
 foreign import ccall "performMajorGC" performMajorGC :: IO ()
 
 -- | Triggers an immediate minor garbage collection.
 --
--- /Since: 4.7.0.0/
+-- @since 4.7.0.0
 foreign import ccall "performGC" performMinorGC :: IO ()
