@@ -12,7 +12,7 @@
 -- Module      : Data.Binary.Get
 -- Copyright   : Lennart Kolmodin
 -- License     : BSD3-style (see LICENSE)
--- 
+--
 -- Maintainer  : Lennart Kolmodin <kolmodin@gmail.com>
 -- Stability   : experimental
 -- Portability : portable to Hugs and GHC.
@@ -33,11 +33,11 @@
 -- A corresponding Haskell value looks like this:
 --
 -- @
--- data Trade = Trade
---   { timestamp :: !'Word32'
---   , price     :: !'Word32'
---   , qty       :: !'Word16'
---   } deriving ('Show')
+--data Trade = Trade
+--  { timestamp :: !'Word32'
+--  , price     :: !'Word32'
+--  , qty       :: !'Word16'
+--  } deriving ('Show')
 -- @
 --
 -- The fields in @Trade@ are marked as strict (using @!@) since we don't need
@@ -46,21 +46,21 @@
 -- <http://www.haskell.org/ghc/docs/latest/html/users_guide/pragmas.html#unpack-pragma>
 --
 -- Now, let's have a look at a decoder for this format.
--- 
+--
 -- @
--- getTrade :: 'Get' Trade
--- getTrade = do
---   timestamp <- 'getWord32le'
---   price     <- 'getWord32le'
---   quantity  <- 'getWord16le'
---   return '$!' Trade timestamp price quantity
+--getTrade :: 'Get' Trade
+--getTrade = do
+--  timestamp <- 'getWord32le'
+--  price     <- 'getWord32le'
+--  quantity  <- 'getWord16le'
+--  return '$!' Trade timestamp price quantity
 -- @
--- 
+--
 -- Or even simpler using applicative style:
 --
 -- @
--- getTrade' :: 'Get' Trade
--- getTrade' = Trade '<$>' 'getWord32le' '<*>' 'getWord32le' '<*>' 'getWord16le'
+--getTrade' :: 'Get' Trade
+--getTrade' = Trade '<$>' 'getWord32le' '<*>' 'getWord32le' '<*>' 'getWord16le'
 -- @
 --
 -- The applicative style can sometimes result in faster code, as @binary@
@@ -73,24 +73,23 @@
 -- Let's first define a function that decodes many @Trade@s.
 --
 -- @
--- getTrades :: Get ['Trade']
--- getTrades = do
---   empty <- 'isEmpty'
---   if empty
---     then return []
---     else do trade <- getTrade
---             trades <- getTrades
---             return (trade:trades)
+--getTrades :: Get [Trade]
+--getTrades = do
+--  empty <- 'isEmpty'
+--  if empty
+--    then return []
+--    else do trade <- getTrade
+--            trades <- getTrades
+--            return (trade:trades)
 -- @
 --
 -- Finally, we run the decoder:
 --
 -- @
--- example :: IO ()
--- example = do
+--lazyIOExample :: IO [Trade]
+--lazyIOExample = do
 --  input <- BL.readFile \"trades.bin\"
---  let trades = runGet getTrades input 
---  print trades
+--  return ('runGet' getTrades input)
 -- @
 --
 -- This decoder has the downside that it will need to read all the input before
@@ -98,24 +97,41 @@
 -- it knows it could decode without any decoder errors.
 --
 -- You could also refactor to a left-fold, to decode in a more streaming fashion,
--- and get the following decoder. It will start to return data without knowning
+-- and get the following decoder. It will start to return data without knowing
 -- that it can decode all input.
 --
 -- @
--- example2 :: BL.ByteString -> [Trade]
--- example2 input
---   | BL.null input = []
---   | otherwise =
---      let (trade, rest, _) = 'runGetState' getTrade input 0
---      in trade : example2 rest
+--incrementalExample :: BL.ByteString -> [Trade]
+--incrementalExample input0 = go decoder input0
+--  where
+--    decoder = 'runGetIncremental' getTrade
+--    go :: 'Decoder' Trade -> BL.ByteString -> [Trade]
+--    go ('Done' leftover _consumed trade) input =
+--      trade : go decoder (BL.chunk leftover input)
+--    go ('Partial' k) input                     =
+--      go (k . takeHeadChunk $ input) (dropHeadChunk input)
+--    go ('Fail' _leftover _consumed msg) _input =
+--      error msg
+--
+--takeHeadChunk :: BL.ByteString -> Maybe BS.ByteString
+--takeHeadChunk lbs =
+--  case lbs of
+--    (BL.Chunk bs _) -> Just bs
+--    _ -> Nothing
+--
+--dropHeadChunk :: BL.ByteString -> BL.ByteString
+--dropHeadChunk lbs =
+--  case lbs of
+--    (BL.Chunk _ lbs') -> lbs'
+--    _ -> BL.Empty
 -- @
 --
--- Both these examples use lazy I/O to read the file from the disk, which is
+-- The @lazyIOExample@ uses lazy I/O to read the file from the disk, which is
 -- not suitable in all applications, and certainly not if you need to read
 -- from a socket which has higher likelihood to fail. To address these needs,
--- use the incremental input method.
--- For an example of this, see the implementation of 'decodeFileOrFail' in
--- "Data.Binary".
+-- use the incremental input method like in @incrementalExample@.
+-- For an example of how to read incrementally from a Handle,
+-- see the implementation of 'decodeFileOrFail' in "Data.Binary".
 -----------------------------------------------------------------------------
 
 
@@ -126,7 +142,7 @@ module Data.Binary.Get (
 
     -- * The lazy input interface
     -- $lazyinterface
-    , runGet 
+    , runGet
     , runGetOrFail
     , ByteOffset
 
@@ -144,9 +160,11 @@ module Data.Binary.Get (
     , skip
     , isEmpty
     , bytesRead
+    , isolate
     , lookAhead
     , lookAheadM
     , lookAheadE
+    , label
 
     -- ** ByteStrings
     , getByteString
@@ -200,7 +218,7 @@ import GHC.Word
 -- The lazy interface consumes a single lazy 'L.ByteString'. It's the easiest
 -- interface to get started with, but it doesn't support interleaving I\/O and
 -- parsing, unless lazy I/O is used.
--- 
+--
 -- There is no way to provide more input other than the initial data. To be
 -- able to incrementally give more data, see the incremental input interface.
 
@@ -255,7 +273,7 @@ calculateOffset r0 = go r0 0
                     go (k $! (acc - unused)) acc
 
 -- | DEPRECATED. Provides compatibility with previous versions of this library.
--- Run a 'Get' monad and return a tuple with thee values.
+-- Run a 'Get' monad and return a tuple with three values.
 -- The first value is the result of the decoder. The second and third are the
 -- unused input, and the number of consumed bytes.
 {-# DEPRECATED runGetState "Use runGetIncremental instead. This function will be removed." #-}
@@ -341,60 +359,44 @@ pushEndOfInput r =
     Done _ _ _ -> r
     Partial k -> k Nothing
     Fail _ _ _ -> r
- 
+
+-- | Skip ahead @n@ bytes. Fails if fewer than @n@ bytes are available.
+skip :: Int -> Get ()
+skip n = withInputChunks (fromIntegral n) consumeBytes (const ()) failOnEOF
+
 -- | An efficient get method for lazy ByteStrings. Fails if fewer than @n@
 -- bytes are left in the input.
 getLazyByteString :: Int64 -> Get L.ByteString
-getLazyByteString n0 = L.fromChunks <$> go n0
-  where
-  consume n str
-    | fromIntegral (B.length str) >= n = Right (B.splitAt (fromIntegral n) str)
-    | otherwise = Left (fromIntegral (B.length str))
-  go n = do
-    str <- get
-    case consume n str of
-      Left used -> do
-        put B.empty
-        demandInput
-        fmap (str:) (go (n - used))
-      Right (want,rest) -> do
-        put rest
-        return [want]
+getLazyByteString n0 = withInputChunks n0 consumeBytes L.fromChunks failOnEOF
+
+consumeBytes :: Consume Int64
+consumeBytes n str
+  | fromIntegral (B.length str) >= n = Right (B.splitAt (fromIntegral n) str)
+  | otherwise = Left (n - fromIntegral (B.length str))
+
+consumeUntilNul :: Consume ()
+consumeUntilNul _ str =
+  case B.break (==0) str of
+    (want, rest) | B.null rest -> Left ()
+                 | otherwise -> Right (want, B.drop 1 rest)
+
+consumeAll :: Consume ()
+consumeAll _ _ = Left ()
+
+resumeOnEOF :: [B.ByteString] -> Get L.ByteString
+resumeOnEOF = return . L.fromChunks
 
 -- | Get a lazy ByteString that is terminated with a NUL byte.
 -- The returned string does not contain the NUL byte. Fails
 -- if it reaches the end of input without finding a NUL.
 getLazyByteStringNul :: Get L.ByteString
-getLazyByteStringNul = L.fromChunks <$> go
-  where
-  findNull str =
-    case B.break (==0) str of
-      (want,rest) | B.null rest -> Nothing
-                  | otherwise -> Just (want, B.drop 1 rest)
-  go = do
-    str <- get
-    case findNull str of
-      Nothing -> do
-        put B.empty
-        demandInput
-        fmap (str:) go
-      Just (want,rest) -> do
-        put rest
-        return [want]
- 
+getLazyByteStringNul = withInputChunks () consumeUntilNul L.fromChunks failOnEOF
+
 -- | Get the remaining bytes as a lazy ByteString.
 -- Note that this can be an expensive function to use as it forces reading
 -- all input and keeping the string in-memory.
 getRemainingLazyByteString :: Get L.ByteString
-getRemainingLazyByteString = L.fromChunks <$> go
-  where
-  go = do
-    str <- get
-    put B.empty
-    done <- isEmpty
-    if done
-      then return [str]
-      else fmap (str:) go
+getRemainingLazyByteString = withInputChunks () consumeAll L.fromChunks resumeOnEOF
 
 ------------------------------------------------------------------------
 -- Primtives
@@ -419,8 +421,7 @@ getWord8 = readN 1 B.unsafeHead
 "getWord32be/readN" getWord32be = readN 4 word32be
 "getWord32le/readN" getWord32le = readN 4 word32le
 "getWord64be/readN" getWord64be = readN 8 word64be
-"getWord64le/readN" getWord64le = readN 8 word64le
- #-}
+"getWord64le/readN" getWord64le = readN 8 word64le #-}
 
 -- | Read a Word16 in big endian format
 getWord16be :: Get Word16
