@@ -1,46 +1,34 @@
-{-# LANGUAGE ScopedTypeVariables #-}
-module Main where
+{-# LANGUAGE CPP, ScopedTypeVariables #-}
+module Main ( main ) where
 
-import Data.Binary
-import Data.Binary.Put
-import Data.Binary.Get
+#if MIN_VERSION_base(4,8,0)
+#define HAS_NATURAL
+#endif
 
-import Control.Applicative
-import Control.Monad (unless)
+import           Control.Applicative
+import           Control.Exception                    as C (SomeException,
+                                                            catch, evaluate)
+import           Control.Monad                        (unless)
+import qualified Data.ByteString                      as B
+import qualified Data.ByteString.Lazy                 as L
+import qualified Data.ByteString.Lazy.Internal        as L
+import           Data.Int
+import           Data.Ratio
+import           System.IO.Unsafe
 
-import qualified Data.ByteString as B
--- import qualified Data.ByteString.Internal as B
--- import qualified Data.ByteString.Unsafe as B
-import qualified Data.ByteString.Lazy as L
-import qualified Data.ByteString.Lazy.Internal as L
--- import qualified Data.Map as Map
--- import qualified Data.Set as Set
--- import qualified Data.IntMap as IntMap
--- import qualified Data.IntSet as IntSet
+#ifdef HAS_NATURAL
+import           Numeric.Natural
+#endif
 
--- import Data.Array (Array)
--- import Data.Array.IArray
--- import Data.Array.Unboxed (UArray)
+import           Test.Framework
+import           Test.Framework.Providers.QuickCheck2
+import           Test.QuickCheck
 
--- import Data.Word
-import Data.Int
-import Data.Ratio
-
-import Control.Exception as C (catch,evaluate,SomeException)
--- import Control.Monad
--- import System.Environment
--- import System.IO
-import System.IO.Unsafe
-
-import Test.QuickCheck
--- import Text.Printf
-
-import Test.Framework
-import Test.Framework.Providers.QuickCheck2
--- import Data.Monoid
-
-import Action (prop_action)
-import Arbitrary()
+import qualified Action                               (tests)
+import           Arbitrary                            ()
+import           Data.Binary
+import           Data.Binary.Get
+import           Data.Binary.Put
 
 ------------------------------------------------------------------------
 
@@ -199,6 +187,93 @@ prop_readTooMuch x = mustThrowError $ x == a && x /= b
     (a,b) = decode (encode x)
     _types = [a,b]
 
+-- In binary-0.5 the Get monad looked like
+--
+-- > data S = S {-# UNPACK #-} !B.ByteString
+-- >            L.ByteString
+-- >            {-# UNPACK #-} !Int64
+-- >
+-- > newtype Get a = Get { unGet :: S -> (# a, S #) }
+--
+-- with a helper function
+--
+-- > mkState :: L.ByteString -> Int64 -> S
+-- > mkState l = case l of
+-- >     L.Empty      -> S B.empty L.empty
+-- >     L.Chunk x xs -> S x xs
+--
+-- Note that mkState is strict in its first argument. This goes wrong in this
+-- function:
+--
+-- > getBytes :: Int -> Get B.ByteString
+-- > getBytes n = do
+-- >     S s ss bytes <- traceNumBytes n $ get
+-- >     if n <= B.length s
+-- >         then do let (consume,rest) = B.splitAt n s
+-- >                 put $! S rest ss (bytes + fromIntegral n)
+-- >                 return $! consume
+-- >         else
+-- >               case L.splitAt (fromIntegral n) (s `join` ss) of
+-- >                 (consuming, rest) ->
+-- >                     do let now = B.concat . L.toChunks $ consuming
+-- >                        put $ mkState rest (bytes + fromIntegral n)
+-- >                        -- forces the next chunk before this one is returned
+-- >                        if (B.length now < n)
+-- >                          then
+-- >                             fail "too few bytes"
+-- >                          else
+-- >                             return now
+--
+-- Consider the else-branch of this function; suppose we ask for n bytes;
+-- the call to L.splitAt gives us a lazy bytestring 'consuming' of precisely @n@
+-- bytes (unless we don't have enough data, in which case we fail); but then
+-- the strict evaluation of mkState on 'rest' means we look ahead too far.
+--
+-- Although this is all done completely differently in binary-0.7 it is
+-- important that the same bug does not get introduced in some other way. The
+-- test is basically the same test that already exists in this test suite,
+-- verifying that
+--
+-- > decode . refragment . encode == id
+--
+-- However, we use a different 'refragment', one that introduces an exception
+-- as the tail of the bytestring after rechunking. If we don't look ahead too
+-- far then this should make no difference, but if we do then this will throw
+-- an exception (for instance, in binary-0.5, this will throw an exception for
+-- certain rechunkings, but not for others).
+--
+-- To make sure that the property holds no matter what refragmentation we use,
+-- we test exhaustively for a single chunk, and all ways to break the string
+-- into 2, 3 and 4 chunks.
+prop_lookAheadIndepOfChunking :: (Eq a, Binary a) => a -> Property
+prop_lookAheadIndepOfChunking testInput =
+   forAll (testCuts (L.length (encode testInput))) $
+     roundTrip testInput . rechunk
+  where
+    testCuts :: forall a. (Num a, Enum a) => a -> Gen [a]
+    testCuts len = elements $ [ [] ]
+                           ++ [ [i]
+                              | i <- [0 .. len] ]
+                           ++ [ [i, j]
+                              | i <- [0 .. len]
+                              , j <- [0 .. len - i] ]
+                           ++ [ [i, j, k]
+                              | i <- [0 .. len]
+                              , j <- [0 .. len - i]
+                              , k <- [0 .. len - i - j] ]
+
+    -- Rechunk a bytestring, leaving the tail as an exception rather than Empty
+    rechunk :: forall a. Integral a => [a] -> L.ByteString -> L.ByteString
+    rechunk cuts = fromChunks . cut cuts . B.concat . L.toChunks
+      where
+        cut :: [a] -> B.ByteString -> [B.ByteString]
+        cut []     bs = [bs]
+        cut (i:is) bs = let (bs0, bs1) = B.splitAt (fromIntegral i) bs
+                        in bs0 : cut is bs1
+
+        fromChunks :: [B.ByteString] ->  L.ByteString
+        fromChunks []       = error "Binary should not have to ask for this chunk!"
+        fromChunks (bs:bss) = L.Chunk bs (fromChunks bss)
 
 -- String utilities
 
@@ -282,6 +357,25 @@ main = defaultMain tests
 
 ------------------------------------------------------------------------
 
+#ifdef HAS_NATURAL
+prop_test_Natural :: Property
+prop_test_Natural = forAll (gen :: Gen Natural) test
+  where
+    gen :: Gen Natural
+    gen = do
+      b <- arbitrary
+      if b
+        then do
+          x <- arbitrarySizedNatural :: Gen Natural
+          -- arbitrarySizedNatural generates numbers smaller than
+          -- (maxBound :: Word64), so let's make them bigger to better test
+          -- the Binary instance.
+          return (x + fromIntegral (maxBound :: Word64))
+        else arbitrarySizedNatural
+#endif
+
+------------------------------------------------------------------------
+
 type T a = a -> Property
 type B a = a -> Bool
 
@@ -304,6 +398,9 @@ tests =
         , testGroup "Boundaries"
             [ testProperty "read to much"         (p (prop_readTooMuch :: B Word8))
             , testProperty "read negative length" (p (prop_getByteString_negative :: T Int))
+            , -- Arbitrary test input
+              let testInput :: [Int] ; testInput = [0 .. 10]
+              in testProperty "look-ahead independent of chunking" (p (prop_lookAheadIndepOfChunking testInput))
             ]
 
         , testGroup "Partial"
@@ -314,8 +411,7 @@ tests =
             ]
 
         , testGroup "Model"
-            [ testProperty "action" Action.prop_action
-            ]
+            Action.tests
 
         , testGroup "Primitives"
             [ testProperty "Word16be"   (p prop_Word16be)
@@ -332,9 +428,9 @@ tests =
 
         , testGroup "String utils"
             [ testProperty "getLazyByteString"          prop_getLazyByteString
-            , testProperty "getLazyByteStringNul"       prop_getLazyByteStringNul 
+            , testProperty "getLazyByteStringNul"       prop_getLazyByteStringNul
             , testProperty "getLazyByteStringNul No Null" prop_getLazyByteStringNul_noNul
-            , testProperty "getRemainingLazyByteString" prop_getRemainingLazyByteString 
+            , testProperty "getRemainingLazyByteString" prop_getRemainingLazyByteString
             ]
 
         , testGroup "Using Binary class, refragmented ByteString" $ map (uncurry testProperty)
@@ -357,6 +453,9 @@ tests =
             , ("Word",       p (test :: T Word                   ))
             , ("Int",        p (test :: T Int                    ))
             , ("Integer",    p (test :: T Integer                ))
+#ifdef HAS_NATURAL
+            , ("Natural",      (prop_test_Natural :: Property    ))
+#endif
 
             , ("Float",      p (test :: T Float                  ))
             , ("Double",     p (test :: T Double                 ))
@@ -401,10 +500,6 @@ tests =
                       p (test :: T (Int,Int,Int,Int,Int,Int,Int,Int,Int)))
             , ("(Int,Int,Int,Int,Int,Int,Int,Int,Int,Int)",
                       p (test :: T (Int,Int,Int,Int,Int,Int,Int,Int,Int,Int)))
-    {-
-            , ("IntSet",            p (test      :: T IntSet.IntSet          ))
-            , ("IntMap ByteString", p (test      :: T (IntMap.IntMap B.ByteString) ))
-    -}
 
             , ("B.ByteString",  p (test :: T B.ByteString        ))
             , ("L.ByteString",  p (test :: T L.ByteString        ))
@@ -417,6 +512,3 @@ tests =
             , ("[L.ByteString] invariant", p (prop_invariant :: B [L.ByteString]               ))
             ]
         ]
-
--- GHC only:
---      ,("Sequence", p (roundTrip :: Seq.Seq Int64 -> Bool))
