@@ -237,15 +237,13 @@ import Control.Monad            (when)
 
 import Foreign.C.String         (CString, CStringLen)
 import Foreign.C.Types          (CSize)
+import Foreign.ForeignPtr       (ForeignPtr, withForeignPtr, touchForeignPtr)
 #if MIN_VERSION_base(4,5,0)
-import Foreign.ForeignPtr       (ForeignPtr, newForeignPtr, withForeignPtr
-                                ,touchForeignPtr)
 import Foreign.ForeignPtr.Unsafe(unsafeForeignPtrToPtr)
 #else
-import Foreign.ForeignPtr       (ForeignPtr, newForeignPtr, withForeignPtr
-                                ,touchForeignPtr, unsafeForeignPtrToPtr)
+import Foreign.ForeignPtr       (unsafeForeignPtrToPtr)
 #endif
-import Foreign.Marshal.Alloc    (allocaBytes, mallocBytes, reallocBytes, finalizerFree)
+import Foreign.Marshal.Alloc    (allocaBytes)
 import Foreign.Marshal.Array    (allocaArray)
 import Foreign.Ptr
 import Foreign.Storable         (Storable(..))
@@ -256,7 +254,9 @@ import System.IO                (stdin,stdout,hClose,hFileSize
                                 ,IOMode(..))
 import System.IO.Error          (mkIOError, illegalOperationErrorType)
 
+#if !(MIN_VERSION_base(4,8,0))
 import Data.Monoid              (Monoid(..))
+#endif
 
 #if !defined(__GLASGOW_HASKELL__)
 import System.IO.Unsafe
@@ -315,16 +315,6 @@ hWaitForInput _ _ = return ()
 unsafeDupablePerformIO = unsafePerformIO
 #endif
 
--- -----------------------------------------------------------------------------
---
--- Useful macros, until we have bang patterns
---
-
-#define STRICT1(f) f a | a `seq` False = undefined
-#define STRICT2(f) f a b | a `seq` b `seq` False = undefined
-#define STRICT3(f) f a b c | a `seq` b `seq` c `seq` False = undefined
-#define STRICT4(f) f a b c d | a `seq` b `seq` c `seq` d `seq` False = undefined
-#define STRICT5(f) f a b c d e | a `seq` b `seq` c `seq` d `seq` e `seq` False = undefined
 
 -- -----------------------------------------------------------------------------
 -- Introducing and eliminating 'ByteString's
@@ -430,7 +420,7 @@ snoc (PS x s l) c = unsafeCreate (l+1) $ \p -> withForeignPtr x $ \f -> do
 head :: ByteString -> Word8
 head (PS x s l)
     | l <= 0    = errorEmptyList "head"
-    | otherwise = inlinePerformIO $ withForeignPtr x $ \p -> peekByteOff p s
+    | otherwise = accursedUnutterablePerformIO $ withForeignPtr x $ \p -> peekByteOff p s
 {-# INLINE head #-}
 
 -- | /O(1)/ Extract the elements after the head of a ByteString, which must be non-empty.
@@ -446,8 +436,8 @@ tail (PS p s l)
 uncons :: ByteString -> Maybe (Word8, ByteString)
 uncons (PS x s l)
     | l <= 0    = Nothing
-    | otherwise = Just (inlinePerformIO $ withForeignPtr x
-                                        $ \p -> peekByteOff p s,
+    | otherwise = Just (accursedUnutterablePerformIO $ withForeignPtr x
+                                                     $ \p -> peekByteOff p s,
                         PS x (s+1) (l-1))
 {-# INLINE uncons #-}
 
@@ -456,7 +446,8 @@ uncons (PS x s l)
 last :: ByteString -> Word8
 last ps@(PS x s l)
     | null ps   = errorEmptyList "last"
-    | otherwise = inlinePerformIO $ withForeignPtr x $ \p -> peekByteOff p (s+l-1)
+    | otherwise = accursedUnutterablePerformIO $
+                    withForeignPtr x $ \p -> peekByteOff p (s+l-1)
 {-# INLINE last #-}
 
 -- | /O(1)/ Return all the elements of a 'ByteString' except the last one.
@@ -473,8 +464,8 @@ unsnoc :: ByteString -> Maybe (ByteString, Word8)
 unsnoc (PS x s l)
     | l <= 0    = Nothing
     | otherwise = Just (PS x s (l-1),
-                        inlinePerformIO $ withForeignPtr x
-                                        $ \p -> peekByteOff p (s+l-1))
+                        accursedUnutterablePerformIO $
+                          withForeignPtr x $ \p -> peekByteOff p (s+l-1))
 {-# INLINE unsnoc #-}
 
 -- | /O(n)/ Append two ByteStrings
@@ -492,8 +483,7 @@ map f (PS fp s len) = unsafeDupablePerformIO $ withForeignPtr fp $ \a ->
     create len $ map_ 0 (a `plusPtr` s)
   where
     map_ :: Int -> Ptr Word8 -> Ptr Word8 -> IO ()
-    STRICT3(map_)
-    map_ n p1 p2
+    map_ !n !p1 !p2
        | n >= len = return ()
        | otherwise = do
             x <- peekByteOff p1 n
@@ -535,7 +525,7 @@ foldl f z (PS fp off len) =
     where
       -- not tail recursive; traverses array right to left
       go !p !q | p == q    = z
-               | otherwise = let !x = inlinePerformIO $ do
+               | otherwise = let !x = accursedUnutterablePerformIO $ do
                                         x' <- peek p
                                         touchForeignPtr fp
                                         return x'
@@ -546,7 +536,7 @@ foldl f z (PS fp off len) =
 --
 foldl' :: (a -> Word8 -> a) -> a -> ByteString -> a
 foldl' f v (PS fp off len) =
-      inlinePerformIO $ withForeignPtr fp $ \p ->
+      accursedUnutterablePerformIO $ withForeignPtr fp $ \p ->
         go v (p `plusPtr` off) (p `plusPtr` (off+len))
     where
       -- tail recursive; traverses array left to right
@@ -565,7 +555,7 @@ foldr k z (PS fp off len) =
     where
       -- not tail recursive; traverses array left to right
       go !p !q | p == q    = z
-               | otherwise = let !x = inlinePerformIO $ do
+               | otherwise = let !x = accursedUnutterablePerformIO $ do
                                         x' <- peek p
                                         touchForeignPtr fp
                                         return x'
@@ -575,7 +565,7 @@ foldr k z (PS fp off len) =
 -- | 'foldr'' is like 'foldr', but strict in the accumulator.
 foldr' :: (Word8 -> a -> a) -> a -> ByteString -> a
 foldr' k v (PS fp off len) =
-      inlinePerformIO $ withForeignPtr fp $ \p ->
+      accursedUnutterablePerformIO $ withForeignPtr fp $ \p ->
         go v (p `plusPtr` (off+len-1)) (p `plusPtr` (off-1))
     where
       -- tail recursive; traverses array right to left
@@ -635,14 +625,13 @@ concatMap f = concat . foldr ((:) . f) []
 -- any element of the 'ByteString' satisfies the predicate.
 any :: (Word8 -> Bool) -> ByteString -> Bool
 any _ (PS _ _ 0) = False
-any f (PS x s l) = inlinePerformIO $ withForeignPtr x $ \ptr ->
+any f (PS x s l) = accursedUnutterablePerformIO $ withForeignPtr x $ \ptr ->
         go (ptr `plusPtr` s) (ptr `plusPtr` (s+l))
     where
-        STRICT2(go)
-        go p q | p == q    = return False
-               | otherwise = do c <- peek p
-                                if f c then return True
-                                       else go (p `plusPtr` 1) q
+        go !p !q | p == q    = return False
+                 | otherwise = do c <- peek p
+                                  if f c then return True
+                                         else go (p `plusPtr` 1) q
 {-# INLINE any #-}
 
 -- todo fuse
@@ -651,15 +640,14 @@ any f (PS x s l) = inlinePerformIO $ withForeignPtr x $ \ptr ->
 -- if all elements of the 'ByteString' satisfy the predicate.
 all :: (Word8 -> Bool) -> ByteString -> Bool
 all _ (PS _ _ 0) = True
-all f (PS x s l) = inlinePerformIO $ withForeignPtr x $ \ptr ->
+all f (PS x s l) = accursedUnutterablePerformIO $ withForeignPtr x $ \ptr ->
         go (ptr `plusPtr` s) (ptr `plusPtr` (s+l))
     where
-        STRICT2(go)
-        go p q | p == q     = return True  -- end of list
-               | otherwise  = do c <- peek p
-                                 if f c
-                                    then go (p `plusPtr` 1) q
-                                    else return False
+        go !p !q | p == q     = return True  -- end of list
+                 | otherwise  = do c <- peek p
+                                   if f c
+                                      then go (p `plusPtr` 1) q
+                                      else return False
 {-# INLINE all #-}
 
 ------------------------------------------------------------------------
@@ -670,7 +658,7 @@ all f (PS x s l) = inlinePerformIO $ withForeignPtr x $ \ptr ->
 maximum :: ByteString -> Word8
 maximum xs@(PS x s l)
     | null xs   = errorEmptyList "maximum"
-    | otherwise = inlinePerformIO $ withForeignPtr x $ \p ->
+    | otherwise = accursedUnutterablePerformIO $ withForeignPtr x $ \p ->
                       c_maximum (p `plusPtr` s) (fromIntegral l)
 {-# INLINE maximum #-}
 
@@ -680,7 +668,7 @@ maximum xs@(PS x s l)
 minimum :: ByteString -> Word8
 minimum xs@(PS x s l)
     | null xs   = errorEmptyList "minimum"
-    | otherwise = inlinePerformIO $ withForeignPtr x $ \p ->
+    | otherwise = accursedUnutterablePerformIO $ withForeignPtr x $ \p ->
                       c_minimum (p `plusPtr` s) (fromIntegral l)
 {-# INLINE minimum #-}
 
@@ -696,8 +684,7 @@ mapAccumL f acc (PS fp o len) = unsafeDupablePerformIO $ withForeignPtr fp $ \a
     acc' <- withForeignPtr gp $ \p -> mapAccumL_ acc 0 (a `plusPtr` o) p
     return $! (acc', PS gp 0 len)
   where
-    STRICT4(mapAccumL_)
-    mapAccumL_ s n p1 p2
+    mapAccumL_ !s !n !p1 !p2
        | n >= len = return s
        | otherwise = do
             x <- peekByteOff p1 n
@@ -716,8 +703,7 @@ mapAccumR f acc (PS fp o len) = unsafeDupablePerformIO $ withForeignPtr fp $ \a
     acc' <- withForeignPtr gp $ \p -> mapAccumR_ acc (len-1) (a `plusPtr` o) p
     return $! (acc', PS gp 0 len)
   where
-    STRICT4(mapAccumR_)
-    mapAccumR_ s n p q
+    mapAccumR_ !s !n !p !q
        | n <  0    = return s
        | otherwise = do
             x  <- peekByteOff p n
@@ -745,8 +731,7 @@ scanl f v (PS fp s len) = unsafeDupablePerformIO $ withForeignPtr fp $ \a ->
         poke q v
         scanl_ v 0 (a `plusPtr` s) (q `plusPtr` 1)
   where
-    STRICT4(scanl_)
-    scanl_ z n p q
+    scanl_ !z !n !p !q
         | n >= len  = return ()
         | otherwise = do
             x <- peekByteOff p n
@@ -776,8 +761,7 @@ scanr f v (PS fp s len) = unsafeDupablePerformIO $ withForeignPtr fp $ \a ->
         poke (q `plusPtr` len) v
         scanr_ v (len-1) (a `plusPtr` s) q
   where
-    STRICT4(scanr_)
-    scanr_ z n p q
+    scanr_ !z !n !p !q
         | n <  0    = return ()
         | otherwise = do
             x <- peekByteOff p n
@@ -841,14 +825,13 @@ unfoldrN :: Int -> (a -> Maybe (Word8, a)) -> a -> (ByteString, Maybe a)
 unfoldrN i f x0
     | i < 0     = (empty, Just x0)
     | otherwise = unsafePerformIO $ createAndTrim' i $ \p -> go p x0 0
-  where STRICT3(go)
-        go p x n =
-          case f x of
-            Nothing      -> return (0, n, Nothing)
-            Just (w,x')
-             | n == i    -> return (0, n, Just x)
-             | otherwise -> do poke p w
-                               go (p `plusPtr` 1) x' (n+1)
+  where
+    go !p !x !n
+      | n == i    = return (0, n, Just x)
+      | otherwise = case f x of
+                      Nothing     -> return (0, n, Nothing)
+                      Just (w,x') -> do poke p w
+                                        go (p `plusPtr` 1) x' (n+1)
 {-# INLINE unfoldrN #-}
 
 -- ---------------------------------------------------------------------
@@ -928,6 +911,7 @@ breakByte c p = case elemIndex c p of
     Nothing -> (p,empty)
     Just n  -> (unsafeTake n p, unsafeDrop n p)
 {-# INLINE breakByte #-}
+{-# DEPRECATED breakByte "It is an internal function and should never have been exported. Use 'break (== x)' instead. (There are rewrite rules that handle this special case of 'break'.)" #-}
 
 -- | 'breakEnd' behaves like 'break' but from the end of the 'ByteString'
 -- 
@@ -950,15 +934,16 @@ span p ps = break (not . p) ps
 -- > span  (=='c') "abcd" == spanByte 'c' "abcd"
 --
 spanByte :: Word8 -> ByteString -> (ByteString, ByteString)
-spanByte c ps@(PS x s l) = inlinePerformIO $ withForeignPtr x $ \p ->
-    go (p `plusPtr` s) 0
+spanByte c ps@(PS x s l) =
+    accursedUnutterablePerformIO $
+      withForeignPtr x $ \p ->
+        go (p `plusPtr` s) 0
   where
-    STRICT2(go)
-    go p i | i >= l    = return (ps, empty)
-           | otherwise = do c' <- peekByteOff p i
-                            if c /= c'
-                                then return (unsafeTake i ps, unsafeDrop i ps)
-                                else go p (i+1)
+    go !p !i | i >= l    = return (ps, empty)
+             | otherwise = do c' <- peekByteOff p i
+                              if c /= c'
+                                  then return (unsafeTake i ps, unsafeDrop i ps)
+                                  else go p (i+1)
 {-# INLINE spanByte #-}
 
 {-# RULES
@@ -997,9 +982,10 @@ splitWith _pred (PS _  _   0) = []
 splitWith pred_ (PS fp off len) = splitWith0 pred# off len fp
   where pred# c# = pred_ (W8# c#)
 
-        STRICT4(splitWith0)
-        splitWith0 pred' off' len' fp' = withPtr fp $ \p ->
-            splitLoop pred' p 0 off' len' fp'
+        splitWith0 !pred' !off' !len' !fp' =
+          accursedUnutterablePerformIO $
+            withForeignPtr fp $ \p ->
+              splitLoop pred' p 0 off' len' fp'
 
         splitLoop :: (Word# -> Bool)
                   -> Ptr Word8
@@ -1021,9 +1007,8 @@ splitWith pred_ (PS fp off len) = splitWith0 pred# off len fp
 splitWith _ (PS _ _ 0) = []
 splitWith p ps = loop p ps
     where
-        STRICT2(loop)
-        loop q qs = if null rest then [chunk]
-                                 else chunk : loop q (unsafeTail rest)
+        loop !q !qs = if null rest then [chunk]
+                                   else chunk : loop q (unsafeTail rest)
             where (chunk,rest) = break q qs
 #endif
 
@@ -1047,53 +1032,18 @@ split :: Word8 -> ByteString -> [ByteString]
 split _ (PS _ _ 0) = []
 split w (PS x s l) = loop 0
     where
-        STRICT1(loop)
-        loop n =
-            let q = inlinePerformIO $ withForeignPtr x $ \p ->
+        loop !n =
+            let q = accursedUnutterablePerformIO $ withForeignPtr x $ \p ->
                       memchr (p `plusPtr` (s+n))
                              w (fromIntegral (l-n))
             in if q == nullPtr
                 then [PS x (s+n) (l-n)]
-                else let i = inlinePerformIO $ withForeignPtr x $ \p ->
+                else let i = accursedUnutterablePerformIO $ withForeignPtr x $ \p ->
                                return (q `minusPtr` (p `plusPtr` s))
                       in PS x (s+n) (i-n) : loop (i+1)
 
 {-# INLINE split #-}
 
-{-
--- slower. but stays inside Haskell.
-split _ (PS _  _   0) = []
-split (W8# w#) (PS fp off len) = splitWith' off len fp
-    where
-        splitWith' off' len' fp' = withPtr fp $ \p ->
-            splitLoop p 0 off' len' fp'
-
-        splitLoop :: Ptr Word8
-                  -> Int -> Int -> Int
-                  -> ForeignPtr Word8
-                  -> IO [ByteString]
-
-        STRICT5(splitLoop)
-        splitLoop p idx' off' len' fp'
-            | idx' >= len'  = return [PS fp' off' idx']
-            | otherwise = do
-                (W8# x#) <- peekElemOff p (off'+idx')
-                if word2Int# w# ==# word2Int# x#
-                   then return (PS fp' off' idx' :
-                              splitWith' (off'+idx'+1) (len'-idx'-1) fp')
-                   else splitLoop p (idx'+1) off' len' fp'
--}
-
-{-
--- | Like 'splitWith', except that sequences of adjacent separators are
--- treated as a single separator. eg.
--- 
--- > tokens (=='a') "aabbaca" == ["bb","c"]
---
-tokens :: (Word8 -> Bool) -> ByteString -> [ByteString]
-tokens f = P.filter (not.null) . splitWith f
-{-# INLINE tokens #-}
--}
 
 -- | The 'group' function takes a ByteString and returns a list of
 -- ByteStrings such that the concatenation of the result is equal to the
@@ -1163,7 +1113,7 @@ index ps n
 -- element, or 'Nothing' if there is no such element. 
 -- This implementation uses memchr(3).
 elemIndex :: Word8 -> ByteString -> Maybe Int
-elemIndex c (PS x s l) = inlinePerformIO $ withForeignPtr x $ \p -> do
+elemIndex c (PS x s l) = accursedUnutterablePerformIO $ withForeignPtr x $ \p -> do
     let p' = p `plusPtr` s
     q <- memchr p' c (fromIntegral l)
     return $! if q == nullPtr then Nothing else Just $! q `minusPtr` p'
@@ -1178,15 +1128,14 @@ elemIndex c (PS x s l) = inlinePerformIO $ withForeignPtr x $ \p -> do
 -- > (-) (length xs - 1) `fmap` elemIndex c (reverse xs)
 --
 elemIndexEnd :: Word8 -> ByteString -> Maybe Int
-elemIndexEnd ch (PS x s l) = inlinePerformIO $ withForeignPtr x $ \p ->
+elemIndexEnd ch (PS x s l) = accursedUnutterablePerformIO $ withForeignPtr x $ \p ->
     go (p `plusPtr` s) (l-1)
   where
-    STRICT2(go)
-    go p i | i < 0     = return Nothing
-           | otherwise = do ch' <- peekByteOff p i
-                            if ch == ch'
-                                then return $ Just i
-                                else go p (i-1)
+    go !p !i | i < 0     = return Nothing
+             | otherwise = do ch' <- peekByteOff p i
+                              if ch == ch'
+                                  then return $ Just i
+                                  else go p (i-1)
 {-# INLINE elemIndexEnd #-}
 
 -- | /O(n)/ The 'elemIndices' function extends 'elemIndex', by returning
@@ -1195,66 +1144,37 @@ elemIndexEnd ch (PS x s l) = inlinePerformIO $ withForeignPtr x $ \p ->
 elemIndices :: Word8 -> ByteString -> [Int]
 elemIndices w (PS x s l) = loop 0
     where
-        STRICT1(loop)
-        loop n = let q = inlinePerformIO $ withForeignPtr x $ \p ->
+        loop !n = let q = accursedUnutterablePerformIO $ withForeignPtr x $ \p ->
                            memchr (p `plusPtr` (n+s))
                                                 w (fromIntegral (l - n))
-                 in if q == nullPtr
+                  in if q == nullPtr
                         then []
-                        else let i = inlinePerformIO $ withForeignPtr x $ \p ->
+                        else let i = accursedUnutterablePerformIO $ withForeignPtr x $ \p ->
                                        return (q `minusPtr` (p `plusPtr` s))
                              in i : loop (i+1)
 {-# INLINE elemIndices #-}
 
-{-
--- much slower
-elemIndices :: Word8 -> ByteString -> [Int]
-elemIndices c ps = loop 0 ps
-   where STRICT2(loop)
-         loop _ ps' | null ps'            = []
-         loop n ps' | c == unsafeHead ps' = n : loop (n+1) (unsafeTail ps')
-                    | otherwise           = loop (n+1) (unsafeTail ps')
--}
-
 -- | count returns the number of times its argument appears in the ByteString
 --
 -- > count = length . elemIndices
 --
 -- But more efficiently than using length on the intermediate list.
 count :: Word8 -> ByteString -> Int
-count w (PS x s m) = inlinePerformIO $ withForeignPtr x $ \p ->
+count w (PS x s m) = accursedUnutterablePerformIO $ withForeignPtr x $ \p ->
     fmap fromIntegral $ c_count (p `plusPtr` s) (fromIntegral m) w
 {-# INLINE count #-}
 
-{-
---
--- around 30% slower
---
-count w (PS x s m) = inlinePerformIO $ withForeignPtr x $ \p ->
-     go (p `plusPtr` s) (fromIntegral m) 0
-    where
-        go :: Ptr Word8 -> CSize -> Int -> IO Int
-        STRICT3(go)
-        go p l i = do
-            q <- memchr p w l
-            if q == nullPtr
-                then return i
-                else do let k = fromIntegral $ q `minusPtr` p
-                        go (q `plusPtr` 1) (l-k-1) (i+1)
--}
-
 -- | The 'findIndex' function takes a predicate and a 'ByteString' and
 -- returns the index of the first element in the ByteString
 -- satisfying the predicate.
 findIndex :: (Word8 -> Bool) -> ByteString -> Maybe Int
-findIndex k (PS x s l) = inlinePerformIO $ withForeignPtr x $ \f -> go (f `plusPtr` s) 0
+findIndex k (PS x s l) = accursedUnutterablePerformIO $ withForeignPtr x $ \f -> go (f `plusPtr` s) 0
   where
-    STRICT2(go)
-    go ptr n | n >= l    = return Nothing
-             | otherwise = do w <- peek ptr
-                              if k w
-                                then return (Just n)
-                                else go (ptr `plusPtr` 1) (n+1)
+    go !ptr !n | n >= l    = return Nothing
+               | otherwise = do w <- peek ptr
+                                if k w
+                                  then return (Just n)
+                                  else go (ptr `plusPtr` 1) (n+1)
 {-# INLINE findIndex #-}
 
 -- | The 'findIndices' function extends 'findIndex', by returning the
@@ -1262,10 +1182,9 @@ findIndex k (PS x s l) = inlinePerformIO $ withForeignPtr x $ \f -> go (f `plusP
 findIndices :: (Word8 -> Bool) -> ByteString -> [Int]
 findIndices p ps = loop 0 ps
    where
-     STRICT2(loop)
-     loop n qs | null qs           = []
-               | p (unsafeHead qs) = n : loop (n+1) (unsafeTail qs)
-               | otherwise         =     loop (n+1) (unsafeTail qs)
+     loop !n !qs | null qs           = []
+                 | p (unsafeHead qs) = n : loop (n+1) (unsafeTail qs)
+                 | otherwise         =     loop (n+1) (unsafeTail qs)
 
 -- ---------------------------------------------------------------------
 -- Searching ByteStrings
@@ -1290,11 +1209,10 @@ filter k ps@(PS x s l)
         t <- go (f `plusPtr` s) p (f `plusPtr` (s + l))
         return $! t `minusPtr` p -- actual length
     where
-        STRICT3(go)
-        go f t end | f == end  = return t
-                   | otherwise = do
-                        w <- peek f
-                        if k w
+        go !f !t !end | f == end  = return t
+                      | otherwise = do
+                          w <- peek f
+                          if k w
                             then poke t w >> go (f `plusPtr` 1) (t `plusPtr` 1) end
                             else             go (f `plusPtr` 1) t               end
 {-# INLINE filter #-}
@@ -1334,16 +1252,6 @@ find f p = case findIndex f p of
                     _      -> Nothing
 {-# INLINE find #-}
 
-{-
---
--- fuseable, but we don't want to walk the whole array.
--- 
-find k = foldl findEFL Nothing
-    where findEFL a@(Just _) _ = a
-          findEFL _          c | k c       = Just c
-                               | otherwise = Nothing
--}
-
 -- | /O(n)/ The 'partition' function takes a predicate a ByteString and returns
 -- the pair of ByteStrings with elements which do and do not satisfy the
 -- predicate, respectively; i.e.,
@@ -1363,7 +1271,7 @@ isPrefixOf :: ByteString -> ByteString -> Bool
 isPrefixOf (PS x1 s1 l1) (PS x2 s2 l2)
     | l1 == 0   = True
     | l2 < l1   = False
-    | otherwise = inlinePerformIO $ withForeignPtr x1 $ \p1 ->
+    | otherwise = accursedUnutterablePerformIO $ withForeignPtr x1 $ \p1 ->
         withForeignPtr x2 $ \p2 -> do
             i <- memcmp (p1 `plusPtr` s1) (p2 `plusPtr` s2) (fromIntegral l1)
             return $! i == 0
@@ -1381,7 +1289,7 @@ isSuffixOf :: ByteString -> ByteString -> Bool
 isSuffixOf (PS x1 s1 l1) (PS x2 s2 l2)
     | l1 == 0   = True
     | l2 < l1   = False
-    | otherwise = inlinePerformIO $ withForeignPtr x1 $ \p1 ->
+    | otherwise = accursedUnutterablePerformIO $ withForeignPtr x1 $ \p1 ->
         withForeignPtr x2 $ \p2 -> do
             i <- memcmp (p1 `plusPtr` s1) (p2 `plusPtr` s2 `plusPtr` (l2 - l1)) (fromIntegral l1)
             return $! i == 0
@@ -1425,8 +1333,7 @@ breakSubstring :: ByteString -- ^ String to search for
 
 breakSubstring pat src = search 0 src
   where
-    STRICT2(search)
-    search n s
+    search !n !s
         | null s             = (src,empty)      -- not found
         | pat `isPrefixOf` s = (take n src,s)
         | otherwise          = search (n+1) (unsafeTail s)
@@ -1441,19 +1348,6 @@ findSubstring f i = listToMaybe (findSubstrings f i)
 
 {-# DEPRECATED findSubstring "findSubstring is deprecated in favour of breakSubstring." #-}
 
-{-
-findSubstring pat str = search 0 str
-    where
-        STRICT2(search)
-        search n s
-            = let x = pat `isPrefixOf` s
-              in
-                if null s
-                    then if x then Just n else Nothing
-                    else if x then Just n
-                              else     search (n+1) (unsafeTail s)
--}
-
 -- | Find the indexes of all (possibly overlapping) occurances of a
 -- substring in a string.
 --
@@ -1464,37 +1358,13 @@ findSubstrings pat str
     | null pat         = [0 .. length str]
     | otherwise        = search 0 str
   where
-    STRICT2(search)
-    search n s
+    search !n !s
         | null s             = []
         | pat `isPrefixOf` s = n : search (n+1) (unsafeTail s)
         | otherwise          =     search (n+1) (unsafeTail s)
 
 {-# DEPRECATED findSubstrings "findSubstrings is deprecated in favour of breakSubstring." #-}
 
-{-
-{- This function uses the Knuth-Morris-Pratt string matching algorithm.  -}
-
-findSubstrings pat@(PS _ _ m) str@(PS _ _ n) = search 0 0
-  where
-      patc x = pat `unsafeIndex` x
-      strc x = str `unsafeIndex` x
-
-      -- maybe we should make kmpNext a UArray before using it in search?
-      kmpNext = listArray (0,m) (-1:kmpNextL pat (-1))
-      kmpNextL p _ | null p = []
-      kmpNextL p j = let j' = next (unsafeHead p) j + 1
-                         ps = unsafeTail p
-                         x = if not (null ps) && unsafeHead ps == patc j'
-                                then kmpNext Array.! j' else j'
-                        in x:kmpNextL ps j'
-      search i j = match ++ rest -- i: position in string, j: position in pattern
-        where match = if j == m then [(i - j)] else []
-              rest = if i == n then [] else search (i+1) (next (strc i) j + 1)
-      next c j | j >= 0 && (j == m || c /= patc j) = next c (kmpNext Array.! j)
-               | otherwise = j
--}
-
 -- ---------------------------------------------------------------------
 -- Zipping
 
@@ -1530,8 +1400,7 @@ zipWith' f (PS fp s l) (PS fq t m) = unsafeDupablePerformIO $
     create len $ zipWith_ 0 (a `plusPtr` s) (b `plusPtr` t)
   where
     zipWith_ :: Int -> Ptr Word8 -> Ptr Word8 -> Ptr Word8 -> IO ()
-    STRICT4(zipWith_)
-    zipWith_ n p1 p2 r
+    zipWith_ !n !p1 !p2 !r
        | n >= len = return ()
        | otherwise = do
             x <- peekByteOff p1 n
@@ -1577,41 +1446,24 @@ sort (PS input s l) = unsafeCreate l $ \p -> allocaArray 256 $ \arr -> do
     _ <- memset (castPtr arr) 0 (256 * fromIntegral (sizeOf (undefined :: CSize)))
     withForeignPtr input (\x -> countOccurrences arr (x `plusPtr` s) l)
 
-    let STRICT2(go)
-        go 256 _   = return ()
-        go i   ptr = do n <- peekElemOff arr i
-                        when (n /= 0) $ memset ptr (fromIntegral i) n >> return ()
-                        go (i + 1) (ptr `plusPtr` (fromIntegral n))
+    let go 256 !_   = return ()
+        go i   !ptr = do n <- peekElemOff arr i
+                         when (n /= 0) $ memset ptr (fromIntegral i) n >> return ()
+                         go (i + 1) (ptr `plusPtr` (fromIntegral n))
     go 0 p
   where
     -- | Count the number of occurrences of each byte.
     -- Used by 'sort'
     --
     countOccurrences :: Ptr CSize -> Ptr Word8 -> Int -> IO ()
-    STRICT3(countOccurrences)
-    countOccurrences counts str len = go 0
+    countOccurrences !counts !str !len = go 0
      where
-        STRICT1(go)
-        go i | i == len    = return ()
-             | otherwise = do k <- fromIntegral `fmap` peekElemOff str i
-                              x <- peekElemOff counts k
-                              pokeElemOff counts k (x + 1)
-                              go (i + 1)
+        go !i | i == len    = return ()
+              | otherwise = do k <- fromIntegral `fmap` peekElemOff str i
+                               x <- peekElemOff counts k
+                               pokeElemOff counts k (x + 1)
+                               go (i + 1)
 
-{-
-sort :: ByteString -> ByteString
-sort (PS x s l) = unsafeCreate l $ \p -> withForeignPtr x $ \f -> do
-        memcpy p (f `plusPtr` s) l
-        c_qsort p l -- inplace
--}
-
--- The 'sortBy' function is the non-overloaded version of 'sort'.
---
--- Try some linear sorts: radix, counting
--- Or mergesort.
---
--- sortBy :: (Word8 -> Word8 -> Ordering) -> ByteString -> ByteString
--- sortBy f ps = undefined
 
 -- ---------------------------------------------------------------------
 -- Low level constructors
@@ -1692,8 +1544,7 @@ hGetLine h =
          else haveBuf h_ buf 0 []
  where
 
-  fill h_@Handle__{haByteBuffer,haDevice} buf len xss =
-    len `seq` do
+  fill h_@Handle__{haByteBuffer,haDevice} buf !len xss = do
     (r,buf') <- Buffered.fillReadBuffer haDevice buf
     if r == 0
        then do writeIORef haByteBuffer buf{ bufR=0, bufL=0 }
@@ -1752,8 +1603,7 @@ hGetLine h = wantReadableHandle "Data.ByteString.hGetLine" h $ \ handle_ -> do
         hGetLineBufferedLoop handle_ ref buf 0 []
 
     hGetLineBufferedLoop handle_ ref
-            buf@Buffer{ bufRPtr=r, bufWPtr=w, bufBuf=raw } len xss =
-        len `seq` do
+            buf@Buffer{ bufRPtr=r, bufWPtr=w, bufBuf=raw } !len xss = do
         off <- findEOL r w raw
         let new_len = len + off - r
         xs <- mkPS raw r off
@@ -1865,7 +1715,7 @@ putStrLn = hPutStrLn stdout
 -- is far more efficient than reading the characters into a 'String'
 -- and then using 'pack'. First argument is the Handle to read from, 
 -- and the second is the number of bytes to read. It returns the bytes
--- read, up to n, or 'null' if EOF has been reached.
+-- read, up to n, or 'empty' if EOF has been reached.
 --
 -- 'hGet' is implemented in terms of 'hGetBuf'.
 --
@@ -1930,41 +1780,43 @@ illegalBufferSize handle fn sz =
       msg = fn ++ ": illegal ByteString size " ++ showsPrec 9 sz []
 
 
--- | Read entire handle contents strictly into a 'ByteString'.
+-- | Read a handle's entire contents strictly into a 'ByteString'.
 --
--- This function reads chunks at a time, doubling the chunksize on each
--- read. The final buffer is then realloced to the appropriate size. For
+-- This function reads chunks at a time, increasing the chunk size on each
+-- read. The final string is then realloced to the appropriate size. For
 -- files > half of available memory, this may lead to memory exhaustion.
 -- Consider using 'readFile' in this case.
 --
--- As with 'hGet', the string representation in the file is assumed to
--- be ISO-8859-1.
---
 -- The Handle is closed once the contents have been read,
 -- or if an exception is thrown.
 --
 hGetContents :: Handle -> IO ByteString
-hGetContents h = always (hClose h) $ do -- strict, so hClose
-    let start_size = 1024
-    p <- mallocBytes start_size
-    i <- hGetBuf h p start_size
-    if i < start_size
-        then do p' <- reallocBytes p i
-                fp <- newForeignPtr finalizerFree p'
-                return $! PS fp 0 i
-        else f p start_size
-    where
-        always = flip finally
-        f p s = do
-            let s' = 2 * s
-            p' <- reallocBytes p s'
-            i  <- hGetBuf h (p' `plusPtr` s) s
-            if i < s
-                then do let i' = s + i
-                        p'' <- reallocBytes p' i'
-                        fp  <- newForeignPtr finalizerFree p''
-                        return $! PS fp 0 i'
-                else f p' s'
+hGetContents hnd = do
+    bs <- hGetContentsSizeHint hnd 1024 2048
+            `finally` hClose hnd
+    -- don't waste too much space for small files:
+    if length bs < 900
+      then return $! copy bs
+      else return bs
+
+hGetContentsSizeHint :: Handle
+                     -> Int -- ^ first read size
+                     -> Int -- ^ initial buffer size increment
+                     -> IO ByteString
+hGetContentsSizeHint hnd =
+    readChunks []
+  where
+    readChunks chunks sz sz' = do
+      fp        <- mallocByteString sz
+      readcount <- withForeignPtr fp $ \buf -> hGetBuf hnd buf sz
+      let chunk = PS fp 0 readcount
+      -- We rely on the hGetBuf behaviour (not hGetBufSome) where it reads up
+      -- to the size we ask for, or EOF. So short reads indicate EOF.
+      if readcount < sz && sz > 0
+        then return $! concat (P.reverse (chunk : chunks))
+        else readChunks (chunk : chunks) sz' ((sz+sz') `min` 32752)
+             -- we grow the buffer sizes, but not too huge
+             -- we concatenate in the end anyway
 
 -- | getContents. Read stdin strictly. Equivalent to hGetContents stdin
 -- The 'Handle' is closed after the contents have been read.
@@ -1980,14 +1832,18 @@ getContents = hGetContents stdin
 interact :: (ByteString -> ByteString) -> IO ()
 interact transformer = putStr . transformer =<< getContents
 
--- | Read an entire file strictly into a 'ByteString'.  This is far more
--- efficient than reading the characters into a 'String' and then using
--- 'pack'.  It also may be more efficient than opening the file and
--- reading it using 'hGet'.
+-- | Read an entire file strictly into a 'ByteString'.
 --
 readFile :: FilePath -> IO ByteString
-readFile f = bracket (openBinaryFile f ReadMode) hClose
-    (\h -> hFileSize h >>= hGet h . fromIntegral)
+readFile f =
+    bracket (openBinaryFile f ReadMode) hClose $ \h -> do
+      filesz <- hFileSize h
+      let readsz = (fromIntegral filesz `max` 0) + 1
+      hGetContentsSizeHint h readsz (readsz `max` 255)
+      -- Our initial size is one bigger than the file size so that in the
+      -- typical case we will read the whole file in one go and not have
+      -- to allocate any more chunks. We'll still do the right thing if the
+      -- file size is 0 or is changed before we do the read.
 
 -- | Write a 'ByteString' to a file.
 writeFile :: FilePath -> ByteString -> IO ()
@@ -2005,21 +1861,18 @@ appendFile f txt = bracket (openBinaryFile f AppendMode) hClose
 -- | 'findIndexOrEnd' is a variant of findIndex, that returns the length
 -- of the string if no element is found, rather than Nothing.
 findIndexOrEnd :: (Word8 -> Bool) -> ByteString -> Int
-findIndexOrEnd k (PS x s l) = inlinePerformIO $ withForeignPtr x $ \f -> go (f `plusPtr` s) 0
+findIndexOrEnd k (PS x s l) =
+    accursedUnutterablePerformIO $
+      withForeignPtr x $ \f ->
+        go (f `plusPtr` s) 0
   where
-    STRICT2(go)
-    go ptr n | n >= l    = return l
-             | otherwise = do w <- peek ptr
-                              if k w
-                                then return n
-                                else go (ptr `plusPtr` 1) (n+1)
+    go !ptr !n | n >= l    = return l
+               | otherwise = do w <- peek ptr
+                                if k w
+                                  then return n
+                                  else go (ptr `plusPtr` 1) (n+1)
 {-# INLINE findIndexOrEnd #-}
 
--- | Perform an operation with a temporary ByteString
-withPtr :: ForeignPtr a -> (Ptr a -> IO b) -> b
-withPtr fp io = inlinePerformIO (withForeignPtr fp io)
-{-# INLINE withPtr #-}
-
 -- Common up near identical calls to `error' to reduce the number
 -- constant strings created when compiled:
 errorEmptyList :: String -> a
@@ -2044,7 +1897,6 @@ moduleErrorMsg fun msg = "Data.ByteString." ++ fun ++ ':':' ':msg
 
 -- Find from the end of the string using predicate
 findFromEndUntil :: (Word8 -> Bool) -> ByteString -> Int
-STRICT2(findFromEndUntil)
 findFromEndUntil f ps@(PS x s l) =
     if null ps then 0
     else if f (unsafeLast ps) then l
