@@ -44,8 +44,10 @@ module System.Directory
     , copyFile
 
     , canonicalizePath
+    , makeAbsolute
     , makeRelativeToCurrentDirectory
     , findExecutable
+    , findExecutables
     , findFile
     , findFiles
     , findFilesWith
@@ -95,8 +97,8 @@ import Foreign.C
 
 import Data.Maybe
 
-import Data.Time
-import Data.Time.Clock.POSIX
+import Data.Time ( UTCTime )
+import Data.Time.Clock.POSIX ( posixSecondsToUTCTime )
 
 #ifdef __GLASGOW_HASKELL__
 
@@ -196,7 +198,8 @@ The operation may fail with:
 getPermissions :: FilePath -> IO Permissions
 getPermissions name = do
 #ifdef mingw32_HOST_OS
-  withFilePath name $ \s -> do
+  -- issue #9: Windows doesn't like trailing path separators
+  withFilePath (dropTrailingPathSeparator name) $ \s -> do
   -- stat() does a better job of guessing the permissions on Windows
   -- than access() does.  e.g. for execute permission, it looks at the
   -- filename extension :-)
@@ -366,11 +369,11 @@ createDirectoryIfMissing create_parents path0
     parents = reverse . scanl1 (</>) . splitDirectories . normalise
 
     createDirs []         = return ()
-    createDirs (dir:[])   = createDir dir throw
+    createDirs (dir:[])   = createDir dir throwIO
     createDirs (dir:dirs) =
       createDir dir $ \_ -> do
         createDirs dirs
-        createDir dir throw
+        createDir dir throwIO
 
     createDir :: FilePath -> (IOException -> IO ()) -> IO ()
     createDir dir notExistHandler = do
@@ -393,22 +396,45 @@ createDirectoryIfMissing create_parents path0
           -- This caused GHCi to crash when loading a module in the root
           -- directory.
           | isAlreadyExistsError e
-         || isPermissionError e -> (do
+         || isPermissionError e -> do
 #ifdef mingw32_HOST_OS
-              withFileStatus "createDirectoryIfMissing" dir $ \st -> do
-                 isDir <- isDirectory st
-                 if isDir then return ()
-                          else throw e
+              canIgnore <- (withFileStatus "createDirectoryIfMissing" dir isDirectory)
 #else
-              stat <- Posix.getFileStatus dir
-              if Posix.isDirectory stat
-                 then return ()
-                 else throw e
+              canIgnore <- (Posix.isDirectory `fmap` Posix.getFileStatus dir)
 #endif
-              ) `E.catch` ((\_ -> return ()) :: IOException -> IO ())
-          | otherwise              -> throw e
+                           `E.catch` ((\ _ -> return (isAlreadyExistsError e))
+                                    :: IOException -> IO Bool)
+              unless canIgnore (throwIO e)
+          | otherwise              -> throwIO e
 
 #if __GLASGOW_HASKELL__
+
+-- | * @'NotDirectory'@:   not a directory.
+--   * @'Directory'@:      a true directory (not a symbolic link).
+--   * @'DirectoryLink'@:  a directory symbolic link (only exists on Windows).
+data DirectoryType = NotDirectory
+                   | Directory
+                   | DirectoryLink
+                   deriving (Enum, Eq, Ord, Read, Show)
+
+-- | Obtain the type of a directory.
+getDirectoryType :: FilePath -> IO DirectoryType
+getDirectoryType path =
+  (`ioeSetLocation` "getDirectoryType") `modifyIOError` do
+#ifdef mingw32_HOST_OS
+    fmap classify (Win32.getFileAttributes path)
+    where fILE_ATTRIBUTE_REPARSE_POINT = 0x400
+          classify attr
+            | attr .&. Win32.fILE_ATTRIBUTE_DIRECTORY == 0 = NotDirectory
+            | attr .&. fILE_ATTRIBUTE_REPARSE_POINT   == 0 = Directory
+            | otherwise                                    = DirectoryLink
+#else
+    stat <- Posix.getSymbolicLinkStatus path
+    return $ if Posix.isDirectory stat
+             then Directory
+             else NotDirectory
+#endif
+
 {- | @'removeDirectory' dir@ removes an existing directory /dir/.  The
 implementation may specify additional constraints which must be
 satisfied before a directory can be removed (e.g. the directory has to
@@ -460,23 +486,39 @@ removeDirectory path =
 
 #endif
 
--- | @'removeDirectoryRecursive' dir@  removes an existing directory /dir/
--- together with its content and all subdirectories. Be careful,
--- if the directory contains symlinks, the function will follow them.
+-- | @'removeDirectoryRecursive' dir@ removes an existing directory /dir/
+-- together with its contents and subdirectories. Symbolic links are removed
+-- without affecting their the targets.
 removeDirectoryRecursive :: FilePath -> IO ()
-removeDirectoryRecursive startLoc = do
-  cont <- getDirectoryContents startLoc
-  sequence_ [rm (startLoc </> x) | x <- cont, x /= "." && x /= ".."]
-  removeDirectory startLoc
-  where
-    rm :: FilePath -> IO ()
-    rm f = do temp <- E.try (removeFile f)
-              case temp of
-                Left e  -> do isDir <- doesDirectoryExist f
-                              -- If f is not a directory, re-throw the error
-                              unless isDir $ throw (e :: SomeException)
-                              removeDirectoryRecursive f
-                Right _ -> return ()
+removeDirectoryRecursive path =
+  (`ioeSetLocation` "removeDirectoryRecursive") `modifyIOError` do
+    dirType <- getDirectoryType path
+    case dirType of
+      Directory -> removeContentsRecursive path
+      _         -> ioError . (`ioeSetErrorString` "not a directory") $
+                   mkIOError InappropriateType "" Nothing (Just path)
+
+-- | @'removePathRecursive' path@ removes an existing file or directory at
+-- /path/ together with its contents and subdirectories. Symbolic links are
+-- removed without affecting their the targets.
+removePathRecursive :: FilePath -> IO ()
+removePathRecursive path =
+  (`ioeSetLocation` "removePathRecursive") `modifyIOError` do
+    dirType <- getDirectoryType path
+    case dirType of
+      NotDirectory  -> removeFile path
+      Directory     -> removeContentsRecursive path
+      DirectoryLink -> removeDirectory path
+
+-- | @'removeContentsRecursive' dir@ removes the contents of the directory
+-- /dir/ recursively. Symbolic links are removed without affecting their the
+-- targets.
+removeContentsRecursive :: FilePath -> IO ()
+removeContentsRecursive path =
+  (`ioeSetLocation` "removeContentsRecursive") `modifyIOError` do
+    cont <- getDirectoryContents path
+    mapM_ removePathRecursive [path </> x | x <- cont, x /= "." && x /= ".."]
+    removeDirectory path
 
 #if __GLASGOW_HASKELL__
 {- |'removeFile' /file/ removes the directory entry for an existing file
@@ -637,26 +679,32 @@ Either path refers to an existing directory.
 -}
 
 renameFile :: FilePath -> FilePath -> IO ()
-renameFile opath npath = do
-   -- XXX this test isn't performed atomically with the following rename
-#ifdef mingw32_HOST_OS
-   -- ToDo: use Win32 API
-   withFileOrSymlinkStatus "renameFile" opath $ \st -> do
-   is_dir <- isDirectory st
-#else
-   stat <- Posix.getSymbolicLinkStatus opath
-   let is_dir = Posix.isDirectory stat
-#endif
-   if is_dir
-        then ioError (ioeSetErrorString
-                          (mkIOError InappropriateType "renameFile" Nothing (Just opath))
-                          "is a directory")
-        else do
+renameFile opath npath = (`ioeSetLocation` "renameFile") `modifyIOError` do
+   -- XXX the tests are not performed atomically with the rename
+   checkNotDir opath
 #ifdef mingw32_HOST_OS
    Win32.moveFileEx opath npath Win32.mOVEFILE_REPLACE_EXISTING
 #else
    Posix.rename opath npath
 #endif
+     -- The underlying rename implementation can throw odd exceptions when the
+     -- destination is a directory.  For example, Windows typically throws a
+     -- permission error, while POSIX systems may throw a resource busy error
+     -- if one of the paths refers to the current directory.  In these cases,
+     -- we check if the destination is a directory and, if so, throw an
+     -- InappropriateType error.
+     `catchIOError` \ err -> do
+       checkNotDir npath
+       ioError err
+   where checkNotDir path = do
+           dirType <- getDirectoryType path
+                      `catchIOError` \ _ -> return NotDirectory
+           case dirType of
+             Directory     -> errIsDir path
+             DirectoryLink -> errIsDir path
+             NotDirectory  -> return ()
+         errIsDir path = ioError . (`ioeSetErrorString` "is a directory") $
+                         mkIOError InappropriateType "" Nothing (Just path)
 
 #endif /* __GLASGOW_HASKELL__ */
 
@@ -668,7 +716,7 @@ copied to /new/, if possible.
 
 copyFile :: FilePath -> FilePath -> IO ()
 copyFile fromFPath toFPath =
-    copy `catchIOError` (\exc -> throw $ ioeSetLocation exc "copyFile")
+    copy `catchIOError` (\exc -> throwIO $ ioeSetLocation exc "copyFile")
     where copy = bracket (openBinaryFile fromFPath ReadMode) hClose $ \hFrom ->
                  bracketOnError openTmp cleanTmp $ \(tmpFPath, hTmp) ->
                  do allocaBytes bufferSize $ copyContents hFrom hTmp
@@ -689,22 +737,29 @@ copyFile fromFPath toFPath =
 
           ignoreIOExceptions io = io `catchIOError` (\_ -> return ())
 
--- | Given a path referring to a file or directory, returns a
--- canonicalized path. The intent is that two paths referring
--- to the same file\/directory will map to the same canonicalized
--- path.
+-- | Canonicalize the path of an existing file or directory.  The intent is
+-- that two paths referring to the same file\/directory will map to the same
+-- canonicalized path.
 --
--- Note that it is impossible to guarantee that the
--- implication (same file\/dir \<=\> same canonicalizedPath) holds
--- in either direction: this function can make only a best-effort
--- attempt.
+-- __Note__: if you only require an absolute path, consider using
+-- @'makeAbsolute'@ instead, which is more reliable and does not have
+-- unspecified behavior on nonexistent paths.
+--
+-- It is impossible to guarantee that the implication (same file\/dir \<=\>
+-- same canonicalized path) holds in either direction: this function can make
+-- only a best-effort attempt.
+--
+-- The precise behaviour is that of the POSIX @realpath@ function (or
+-- @GetFullPathNameW@ on Windows).  In particular, the behaviour on paths that
+-- don't exist can vary from platform to platform.  Some platforms do not
+-- alter the input, some do, and some throw an exception.
+--
+-- An empty path is considered to be equivalent to the current directory.
+--
+-- /Known bug(s)/: on Windows, this function does not resolve symbolic links.
 --
--- The precise behaviour is that of the C realpath function
--- GetFullPathNameW on Windows). In particular, the behaviour
--- on paths that do not exist is known to vary from platform
--- to platform. Some platforms do not alter the input, some
--- do, and on some an exception will be thrown.
 canonicalizePath :: FilePath -> IO FilePath
+canonicalizePath ""    = canonicalizePath "."
 canonicalizePath fpath =
 #if defined(mingw32_HOST_OS)
          do path <- Win32.getFullPathName fpath
@@ -713,6 +768,7 @@ canonicalizePath fpath =
      GHC.withCString enc fpath $ \pInPath ->
        allocaBytes long_path_size $ \pOutPath ->
          do _ <- throwErrnoPathIfNull "canonicalizePath" fpath $ c_realpath pInPath pOutPath
+
             -- NB: pOutPath will be passed thru as result pointer by c_realpath
             path <- GHC.peekCString enc pOutPath
 #endif
@@ -726,6 +782,18 @@ foreign import ccall unsafe "realpath"
                               -> IO CString
 #endif
 
+-- | Make a path absolute by prepending the current directory (if it isn't
+-- already absolute) and applying @'normalise'@ to the result.
+--
+-- The operation may fail with the same exceptions as @'getCurrentDirectory'@.
+--
+-- /Since: 1.2.2.0/
+makeAbsolute :: FilePath -> IO FilePath
+makeAbsolute = fmap normalise . absolutize
+  where absolutize path -- avoid the call to `getCurrentDirectory` if we can
+          | isRelative path = fmap (</> path) getCurrentDirectory
+          | otherwise       = return path
+
 -- | 'makeRelative' the current directory.
 makeRelativeToCurrentDirectory :: FilePath -> IO FilePath
 makeRelativeToCurrentDirectory x = do
@@ -758,7 +826,7 @@ findExecutable fileName = do
 -- | Given a file name, searches for the file and returns a list of all
 -- occurences that are executable.
 --
--- /Since: 1.2.1.0/
+-- /Since: 1.2.2.0/
 findExecutables :: String -> IO [FilePath]
 findExecutables binary = do
 #if defined(mingw32_HOST_OS)
@@ -842,13 +910,16 @@ getDirectoryContents path =
     bracket
       (Posix.openDirStream path)
       Posix.closeDirStream
-      loop
+      start
  where
-  loop dirp = do
-     e <- Posix.readDirStream dirp
-     if null e then return [] else do
-       es <- loop dirp
-       return (e:es)
+  start dirp =
+      loop id
+    where
+      loop acc = do
+        e <- Posix.readDirStream dirp
+        if null e
+          then return (acc [])
+          else loop (acc . (e:))
 #else
   bracket
      (Win32.findFirstFile (path </> "*"))
@@ -996,28 +1067,35 @@ The operation may fail with:
 
 * 'isDoesNotExistError' if the file or directory does not exist.
 
-Note: When linked against @unix-2.6.0.0@ or later the reported time
-supports sub-second precision if provided by the underlying system
-call.
-
+Note: This function returns a timestamp with sub-second resolution
+only if this package is compiled against @unix-2.6.0.0@ or later
+for unix systems, and @Win32-2.3.1.0@ or later for windows systems.
+Of course this also requires that the underlying file system supports
+such high resolution timestamps.
 -}
 
 getModificationTime :: FilePath -> IO UTCTime
 getModificationTime name = do
 #ifdef mingw32_HOST_OS
- -- ToDo: use Win32 API so we can get sub-second resolution
- withFileStatus "getModificationTime" name $ \ st -> do
- modificationTime st
+#if MIN_VERSION_Win32(2,3,1)
+  fad <- Win32.getFileAttributesExStandard name
+  let win32_epoch_adjust = 116444736000000000
+      Win32.FILETIME ft = Win32.fadLastWriteTime fad
+      mod_time = fromIntegral (ft - win32_epoch_adjust) / 10000000
+#else
+  mod_time <- withFileStatus "getModificationTime" name $ \stat -> do
+    mtime <- st_mtime stat
+    return $ realToFrac (mtime :: CTime)
+#endif
 #else
   stat <- Posix.getFileStatus name
-  let mod_time :: POSIXTime
 #if MIN_VERSION_unix(2,6,0)
-      mod_time = Posix.modificationTimeHiRes stat
+  let mod_time = Posix.modificationTimeHiRes stat
 #else
-      mod_time = realToFrac $ Posix.modificationTime stat
+  let mod_time = realToFrac $ Posix.modificationTime stat
+#endif
 #endif
   return $ posixSecondsToUTCTime mod_time
-#endif
 
 #endif /* __GLASGOW_HASKELL__ */
 
@@ -1030,19 +1108,6 @@ withFileStatus loc name f = do
         throwErrnoIfMinus1Retry_ loc (c_stat s p)
         f p
 
-withFileOrSymlinkStatus :: String -> FilePath -> (Ptr CStat -> IO a) -> IO a
-withFileOrSymlinkStatus loc name f = do
-  modifyIOError (`ioeSetFileName` name) $
-    allocaBytes sizeof_stat $ \p ->
-      withFilePath name $ \s -> do
-        throwErrnoIfMinus1Retry_ loc (lstat s p)
-        f p
-
-modificationTime :: Ptr CStat -> IO UTCTime
-modificationTime stat = do
-    mtime <- st_mtime stat
-    return $ posixSecondsToUTCTime $ realToFrac (mtime :: CTime)
-
 isDirectory :: Ptr CStat -> IO Bool
 isDirectory stat = do
   mode <- st_mode stat
@@ -1121,7 +1186,7 @@ writable.
 On Unix, this function returns @$HOME\/.appName@.  On Windows, a
 typical path might be
 
-> C:/Documents And Settings/user/Application Data/appName
+> C:/Users/user/AppData/Roaming/appName
 
 The operation may fail with:
 
@@ -1206,7 +1271,7 @@ getTemporaryDirectory = do
 #else
   getEnv "TMPDIR"
     `catchIOError` \e -> if isDoesNotExistError e then return "/tmp"
-                          else throw e
+                          else throwIO e
 #endif
 
 -- ToDo: This should be determined via autoconf (AC_EXEEXT)
