@@ -1,4 +1,5 @@
-{-# LANGUAGE MagicHash, NoImplicitPrelude, TypeFamilies, UnboxedTuples #-}
+{-# LANGUAGE MagicHash, NoImplicitPrelude, TypeFamilies, UnboxedTuples,
+             RoleAnnotations #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  GHC.Types
@@ -20,7 +21,8 @@ module GHC.Types (
         Float(..), Double(..),
         Ordering(..), IO(..),
         isTrue#,
-        SPEC(..)
+        SPEC(..),
+        Coercible,
     ) where
 
 import GHC.Prim
@@ -79,6 +81,29 @@ at some point, directly or indirectly, from @Main.main@.
 or the '>>' and '>>=' operations from the 'Monad' class.
 -}
 newtype IO a = IO (State# RealWorld -> (# State# RealWorld, a #))
+type role IO representational
+{-
+The above role annotation is redundant but is included because this role
+is significant in the normalisation of FFI types. Specifically, if this
+role were to become nominal (which would be very strange, indeed!), changes
+elsewhere in GHC would be necessary. See [FFI type roles] in TcForeign.
+-}
+
+{-
+Note [Kind-changing of (~) and Coercible]
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+(~) and Coercible are tricky to define. To the user, they must appear as
+constraints, but we cannot define them as such in Haskell. But we also cannot
+just define them only in GHC.Prim (like (->)), because we need a real module
+for them, e.g. to compile the constructor's info table.
+
+Furthermore the type of MkCoercible cannot be written in Haskell
+(no syntax for ~#R).
+
+So we define them as regular data types in GHC.Types, and do magic in TysWiredIn,
+inside GHC, to change the kind and type.
+-}
 
 
 -- | A data constructor used to box up all unlifted equalities
@@ -88,9 +113,6 @@ newtype IO a = IO (State# RealWorld -> (# State# RealWorld, a #))
 data (~) a b = Eq# ((~#) a b)
 
 
--- Despite this not being exported here, the documentation will
--- be used by haddock, hence the user-facing blurb here, and not in primops.txt.pp:
-
 -- | This two-parameter class has instances for types @a@ and @b@ if
 --      the compiler can infer that they have the same representation. This class
 --      does not have regular instances; instead they are created on-the-fly during
@@ -137,10 +159,15 @@ data (~) a b = Eq# ((~#) a b)
 --      <http://www.cis.upenn.edu/~eir/papers/2014/coercible/coercible.pdf Safe Coercions>
 --      by Joachim Breitner, Richard A. Eisenberg, Simon Peyton Jones and Stephanie Weirich.
 --
---      /Since: 4.7.0.0/
+--      @since 4.7.0.0
 data Coercible a b = MkCoercible ((~#) a b)
+-- It's really ~R# (representational equality), not ~#,
+-- but  * we don't yet have syntax for ~R#,
+--      * the compiled code is the same either way
+--      * TysWiredIn has the truthful types
+-- Also see Note [Kind-changing of (~) and Coercible]
 
--- | Alias for tagToEnum#. Returns True of its parameter is 1# and False
+-- | Alias for 'tagToEnum#'. Returns True if its parameter is 1# and False
 --   if it is 0#.
 
 {-# INLINE isTrue# #-}
@@ -202,7 +229,7 @@ isTrue# x = tagToEnum# x
 -- you're reading this in 2023 then things went wrong). See #8326.
 --
 
--- | SPEC is used by GHC in the @SpecConstr@ pass in order to inform
+-- | 'SPEC' is used by GHC in the @SpecConstr@ pass in order to inform
 -- the compiler when to be particularly aggressive. In particular, it
 -- tells GHC to specialize regardless of size or the number of
 -- specializations. However, not all loops fall into this category.
