@@ -2,6 +2,7 @@
 {-# LANGUAGE BangPatterns #-}
 #if __GLASGOW_HASKELL__ >= 701
 {-# LANGUAGE Safe #-}
+{-# LANGUAGE DeriveGeneric #-}
 #endif
 
 -----------------------------------------------------------------------------
@@ -10,7 +11,7 @@
 -- Copyright   :  (c) The University of Glasgow 2001
 -- License     :  BSD-style (see the file LICENSE)
 --
--- Maintainer  :  David Terei <dave.terei@gmail.com>
+-- Maintainer  :  David Terei <code@davidterei.com>
 -- Stability   :  stable
 -- Portability :  portable
 --
@@ -41,6 +42,7 @@ module Text.PrettyPrint.HughesPJ (
 
         -- ** Wrapping documents in delimiters
         parens, brackets, braces, quotes, doubleQuotes,
+        maybeParens, maybeBrackets, maybeBraces, maybeQuotes, maybeDoubleQuotes,
 
         -- ** Combining documents
         empty,
@@ -75,8 +77,14 @@ module Text.PrettyPrint.HughesPJ (
     ) where
 #endif
 
-import Data.Monoid ( Monoid(mempty, mappend) )
-import Data.String ( IsString(fromString) )
+import Control.DeepSeq ( NFData(rnf) )
+import Data.Function   ( on )
+#if __GLASGOW_HASKELL__ < 709
+import Data.Monoid     ( Monoid(mempty, mappend)  )
+#endif
+import Data.String     ( IsString(fromString) )
+
+import GHC.Generics
 
 -- ---------------------------------------------------------------------------
 -- The Doc calculus
@@ -175,6 +183,9 @@ data Doc
   | NoDoc                                            -- The empty set of documents
   | Beside Doc Bool Doc                              -- True <=> space between
   | Above Doc Bool Doc                               -- True <=> never overlap
+#if __GLASGOW_HASKELL__ >= 701
+  deriving (Generic)
+#endif
 
 {-
 Here are the invariants:
@@ -222,6 +233,9 @@ data TextDetails = Chr  {-# UNPACK #-} !Char -- ^ A single Char fragment
                  | PStr String -- ^ Used to represent a Fast String fragment
                                --   but now deprecated and identical to the
                                --   Str constructor.
+#if __GLASGOW_HASKELL__ >= 701
+                 deriving (Show, Eq, Generic)
+#endif
 
 -- Combining @Doc@ values
 instance Monoid Doc where
@@ -236,6 +250,24 @@ instance Show Doc where
                                     (ribbonsPerLine style)
                                     txtPrinter cont doc
 
+instance Eq Doc where
+  (==) = (==) `on` render
+
+instance NFData Doc where
+  rnf Empty               = ()
+  rnf (NilAbove d)        = rnf d
+  rnf (TextBeside td i d) = rnf td `seq` rnf i `seq` rnf d
+  rnf (Nest k d)          = rnf k  `seq` rnf d
+  rnf (Union ur ul)       = rnf ur `seq` rnf ul
+  rnf NoDoc               = ()
+  rnf (Beside ld s rd)    = rnf ld `seq` rnf s `seq` rnf rd
+  rnf (Above ud s ld)     = rnf ud `seq` rnf s `seq` rnf ld
+
+instance NFData TextDetails where
+  rnf (Chr c)    = rnf c
+  rnf (Str str)  = rnf str
+  rnf (PStr str) = rnf str
+
 -- ---------------------------------------------------------------------------
 -- Values and Predicates on GDocs and TextDetails
 
@@ -339,9 +371,9 @@ rbrack = char ']'
 lbrace = char '{'
 rbrace = char '}'
 
-space_text, nl_text :: TextDetails
-space_text = Chr ' '
-nl_text    = Chr '\n'
+spaceText, nlText :: TextDetails
+spaceText = Chr ' '
+nlText    = Chr '\n'
 
 int      :: Int      -> Doc -- ^ @int n = text (show n)@
 integer  :: Integer  -> Doc -- ^ @integer n = text (show n)@
@@ -365,6 +397,30 @@ parens p       = char '(' <> p <> char ')'
 brackets p     = char '[' <> p <> char ']'
 braces p       = char '{' <> p <> char '}'
 
+-- | Apply 'parens' to 'Doc' if boolean is true.
+maybeParens :: Bool -> Doc -> Doc
+maybeParens False = id
+maybeParens True = parens
+
+-- | Apply 'brackets' to 'Doc' if boolean is true.
+maybeBrackets :: Bool -> Doc -> Doc
+maybeBrackets False = id
+maybeBrackets True = brackets
+
+-- | Apply 'braces' to 'Doc' if boolean is true.
+maybeBraces :: Bool -> Doc -> Doc
+maybeBraces False = id
+maybeBraces True = braces
+
+-- | Apply 'quotes' to 'Doc' if boolean is true.
+maybeQuotes :: Bool -> Doc -> Doc
+maybeQuotes False = id
+maybeQuotes True = quotes
+
+-- | Apply 'doubleQuotes' to 'Doc' if boolean is true.
+maybeDoubleQuotes :: Bool -> Doc -> Doc
+maybeDoubleQuotes False = id
+maybeDoubleQuotes True = doubleQuotes
 
 -- ---------------------------------------------------------------------------
 -- Structural operations on GDocs
@@ -446,17 +502,17 @@ reduceAB (Beside Empty _ q) = q
 reduceAB doc                = doc
 
 nilAbove_ :: RDoc -> RDoc
-nilAbove_ p = NilAbove p
+nilAbove_ = NilAbove
 
 -- Arg of a TextBeside is always an RDoc
 textBeside_ :: TextDetails -> Int -> RDoc -> RDoc
-textBeside_ s sl p = TextBeside s sl p
+textBeside_ = TextBeside
 
 nest_ :: Int -> RDoc -> RDoc
-nest_ k p = Nest k p
+nest_ = Nest
 
 union_ :: RDoc -> RDoc -> RDoc
-union_ p q = Union p q
+union_ = Union
 
 
 -- ---------------------------------------------------------------------------
@@ -496,7 +552,7 @@ above_ p g q     = Above p g q
 
 above :: Doc -> Bool -> RDoc -> RDoc
 above (Above p g1 q1)  g2 q2 = above p g1 (above q1 g2 q2)
-above p@(Beside _ _ _) g  q  = aboveNest (reduceDoc p) g 0 (reduceDoc q)
+above p@(Beside{})     g  q  = aboveNest (reduceDoc p) g 0 (reduceDoc q)
 above p g q                  = aboveNest p             g 0 (reduceDoc q)
 
 -- Specfication: aboveNest p g k q = p $g$ (nest k q)
@@ -564,7 +620,7 @@ beside (Nest k p)          g q   = nest_ k $! beside p g q
 beside p@(Beside p1 g1 q1) g2 q2
          | g1 == g2              = beside p1 g1 $! beside q1 g2 q2
          | otherwise             = beside (reduceDoc p) g2 q2
-beside p@(Above _ _ _)     g q   = let !d = reduceDoc p in beside d g q
+beside p@(Above{})         g q   = let !d = reduceDoc p in beside d g q
 beside (NilAbove p)        g q   = nilAbove_ $! beside p g q
 beside (TextBeside s sl p) g q   = textBeside_ s sl $! rest
                                where
@@ -577,7 +633,7 @@ beside (TextBeside s sl p) g q   = textBeside_ s sl $! rest
 nilBeside :: Bool -> RDoc -> RDoc
 nilBeside _ Empty         = Empty -- Hence the text "" in the spec
 nilBeside g (Nest _ p)    = nilBeside g p
-nilBeside g p | g         = textBeside_ space_text 1 p
+nilBeside g p | g         = textBeside_ spaceText 1 p
               | otherwise = p
 
 
@@ -708,8 +764,7 @@ best :: Int   -- Line length
      -> Int   -- Ribbon length
      -> RDoc
      -> RDoc  -- No unions in here!
-best w0 r p0
-  = get w0 p0
+best w0 r = get w0
   where
     get w _ | w == 0 && False = undefined
     get _ Empty               = Empty
@@ -733,7 +788,7 @@ best w0 r p0
     get1 _ _  (Beside {})         = error "best get1 Beside"
 
 nicest :: Int -> Int -> Doc -> Doc -> Doc
-nicest !w !r p q = nicest1 w r 0 p q
+nicest !w !r = nicest1 w r 0
 
 nicest1 :: Int -> Int -> Int -> Doc -> Doc -> Doc
 nicest1 !w !r !sl p q | fits ((w `min` r) - sl) p = p
@@ -786,8 +841,11 @@ oneLiner (Beside {})         = error "oneLiner Beside"
 data Style
   = Style { mode           :: Mode  -- ^ The rendering mode
           , lineLength     :: Int   -- ^ Length of line, in chars
-          , ribbonsPerLine :: Float -- ^ Ratio of ribbon length to line length
+          , ribbonsPerLine :: Float -- ^ Ratio of line length to ribbon length
           }
+#if __GLASGOW_HASKELL__ >= 701
+  deriving (Show, Eq, Generic)
+#endif
 
 -- | The default style (@mode=PageMode, lineLength=100, ribbonsPerLine=1.5@).
 style :: Style
@@ -798,16 +856,19 @@ data Mode = PageMode     -- ^ Normal
           | ZigZagMode   -- ^ With zig-zag cuts
           | LeftMode     -- ^ No indentation, infinitely long lines
           | OneLineMode  -- ^ All on one line
+#if __GLASGOW_HASKELL__ >= 701
+          deriving (Show, Eq, Generic)
+#endif
 
 -- | Render the @Doc@ to a String using the default @Style@.
 render :: Doc -> String
-render doc = fullRender (mode style) (lineLength style) (ribbonsPerLine style)
-                        txtPrinter "" doc
+render = fullRender (mode style) (lineLength style) (ribbonsPerLine style)
+                    txtPrinter ""
 
 -- | Render the @Doc@ to a String using the given @Style@.
 renderStyle :: Style -> Doc -> String
-renderStyle s doc = fullRender (mode s) (lineLength s) (ribbonsPerLine s)
-                    txtPrinter "" doc
+renderStyle s = fullRender (mode s) (lineLength s) (ribbonsPerLine s)
+                txtPrinter ""
 
 -- | Default TextDetails printer
 txtPrinter :: TextDetails -> String -> String
@@ -824,9 +885,9 @@ fullRender :: Mode                     -- ^ Rendering mode
            -> Doc                      -- ^ The document
            -> a                        -- ^ Result
 fullRender OneLineMode _ _ txt end doc
-  = easy_display space_text (\_ y -> y) txt end (reduceDoc doc)
+  = easyDisplay spaceText (\_ y -> y) txt end (reduceDoc doc)
 fullRender LeftMode    _ _ txt end doc
-  = easy_display nl_text first txt end (reduceDoc doc)
+  = easyDisplay nlText first txt end (reduceDoc doc)
 
 fullRender m lineLen ribbons txt rest doc
   = display m lineLen ribbonLen txt rest doc'
@@ -839,23 +900,23 @@ fullRender m lineLen ribbons txt rest doc
                       ZigZagMode -> maxBound
                       _          -> lineLen
 
-easy_display :: TextDetails
+easyDisplay :: TextDetails
              -> (Doc -> Doc -> Doc)
              -> (TextDetails -> a -> a)
              -> a
              -> Doc
              -> a
-easy_display nl_space_text choose txt end doc
-  = lay doc
+easyDisplay nlSpaceText choose txt end
+  = lay
   where
-    lay NoDoc              = error "easy_display: NoDoc"
+    lay NoDoc              = error "easyDisplay: NoDoc"
     lay (Union p q)        = lay (choose p q)
     lay (Nest _ p)         = lay p
     lay Empty              = end
-    lay (NilAbove p)       = nl_space_text `txt` lay p
+    lay (NilAbove p)       = nlSpaceText `txt` lay p
     lay (TextBeside s _ p) = s `txt` lay p
-    lay (Above {})         = error "easy_display Above"
-    lay (Beside {})        = error "easy_display Beside"
+    lay (Above {})         = error "easyDisplay Above"
+    lay (Beside {})        = error "easyDisplay Beside"
 
 display :: Mode -> Int -> Int -> (TextDetails -> a -> a) -> a -> Doc -> a
 display m !page_width !ribbon_width txt end doc
@@ -865,19 +926,19 @@ display m !page_width !ribbon_width txt end doc
         lay k _            | k `seq` False = undefined
         lay k (Nest k1 p)  = lay (k + k1) p
         lay _ Empty        = end
-        lay k (NilAbove p) = nl_text `txt` lay k p
+        lay k (NilAbove p) = nlText `txt` lay k p
         lay k (TextBeside s sl p)
             = case m of
                     ZigZagMode |  k >= gap_width
-                               -> nl_text `txt` (
+                               -> nlText `txt` (
                                   Str (replicate shift '/') `txt` (
-                                  nl_text `txt`
+                                  nlText `txt`
                                   lay1 (k - shift) s sl p ))
 
                                |  k < 0
-                               -> nl_text `txt` (
+                               -> nlText `txt` (
                                   Str (replicate shift '\\') `txt` (
-                                  nl_text `txt`
+                                  nlText `txt`
                                   lay1 (k + shift) s sl p ))
 
                     _ -> lay1 k s sl p
@@ -890,7 +951,7 @@ display m !page_width !ribbon_width txt end doc
                              in Str (indent k) `txt` (s `txt` lay2 r p)
 
         lay2 k _ | k `seq` False   = undefined
-        lay2 k (NilAbove p)        = nl_text `txt` lay k p
+        lay2 k (NilAbove p)        = nlText `txt` lay k p
         lay2 k (TextBeside s sl p) = s `txt` lay2 (k + sl) p
         lay2 k (Nest _ p)          = lay2 k p
         lay2 _ Empty               = end
