@@ -1,5 +1,5 @@
 {-# LANGUAGE CPP, RecordWildCards, BangPatterns #-}
-{-# OPTIONS_HADDOCK hide #-}
+{-# OPTIONS_HADDOCK not-home #-}
 #ifdef __GLASGOW_HASKELL__
 {-# LANGUAGE Trustworthy #-}
 {-# LANGUAGE InterruptibleFFI #-}
@@ -15,48 +15,52 @@
 -- Stability   :  experimental
 -- Portability :  portable
 --
--- Operations for creating and interacting with sub-processes.
+-- __Note:__ This module exports internal implementation details that may
+-- change anytime.  If you want a more stable API, use "System.Process"
+-- instead.
 --
 -----------------------------------------------------------------------------
 
 module System.Process.Internals (
-        ProcessHandle(..), ProcessHandle__(..),
-        PHANDLE, closePHANDLE, mkProcessHandle,
-        modifyProcessHandle, withProcessHandle,
+    ProcessHandle(..), ProcessHandle__(..),
+    PHANDLE, closePHANDLE, mkProcessHandle,
+    modifyProcessHandle, withProcessHandle,
 #ifdef __GLASGOW_HASKELL__
-        CreateProcess(..),
-        CmdSpec(..), StdStream(..),
-        createProcess_,
-        runGenProcess_, --deprecated
+    CreateProcess(..),
+    CmdSpec(..), StdStream(..),
+    createProcess_,
+    runGenProcess_, --deprecated
 #endif
-        startDelegateControlC,
-        endDelegateControlC,
+    startDelegateControlC,
+    endDelegateControlC,
+    stopDelegateControlC,
 #if !defined(mingw32_HOST_OS) && !defined(__MINGW32__)
-        pPrPr_disableITimers, c_execvpe,
-        ignoreSignal, defaultSignal,
-#endif
-        withFilePathException, withCEnvironment,
-        translate,
-        fdToHandle,
-  ) where
-
-#if !defined(mingw32_HOST_OS) && !defined(__MINGW32__)
-import Control.Monad
-import Data.Char
-import System.Posix.Types
-import System.Posix.Process.Internals ( pPrPr_disableITimers, c_execvpe )
-import System.IO
+    pPrPr_disableITimers, c_execvpe,
+    ignoreSignal, defaultSignal,
 #endif
+    withFilePathException, withCEnvironment,
+    translate,
+    fdToHandle,
+    ) where
 
 import Control.Concurrent
 import Control.Exception
 import Data.Bits
+import Data.String
 import Foreign.C
 import Foreign.Marshal
 import Foreign.Ptr
 import Foreign.Storable
 import System.IO.Unsafe
 
+#if !defined(mingw32_HOST_OS) && !defined(__MINGW32__)
+import Control.Monad
+import Data.Char
+import System.IO
+import System.Posix.Process.Internals ( pPrPr_disableITimers, c_execvpe )
+import System.Posix.Types
+#endif
+
 #ifdef __GLASGOW_HASKELL__
 import System.Posix.Internals
 import GHC.IO.Exception
@@ -68,12 +72,12 @@ import GHC.IO.Handle.Internals
 import GHC.IO.Handle.Types hiding (ClosedHandle)
 import System.IO.Error
 import Data.Typeable
-#if defined(mingw32_HOST_OS)
+# if defined(mingw32_HOST_OS)
 import GHC.IO.IOMode
 import System.Win32.DebugApi (PHANDLE)
-#else
+# else
 import System.Posix.Signals as Sig
-#endif
+# endif
 #endif
 
 #if defined(mingw32_HOST_OS)
@@ -99,7 +103,7 @@ import System.FilePath
 -- ProcessHandle type
 
 {- | A handle to a process, which can be used to wait for termination
-     of the process using 'waitForProcess'.
+     of the process using 'System.Process.waitForProcess'.
 
      None of the process-creation functions in this library wait for
      termination: they all return a 'ProcessHandle' which may be used
@@ -182,11 +186,36 @@ data CreateProcess = CreateProcess{
 
 data CmdSpec
   = ShellCommand String
-      -- ^ a command line to execute using the shell
+      -- ^ A command line to execute using the shell
   | RawCommand FilePath [String]
-      -- ^ the filename of an executable with a list of arguments.
-      -- see 'System.Process.proc' for the precise interpretation of
-      -- the @FilePath@ field.
+      -- ^ The name of an executable with a list of arguments
+      --
+      -- The 'FilePath' argument names the executable, and is interpreted
+      -- according to the platform's standard policy for searching for
+      -- executables. Specifically:
+      --
+      -- * on Unix systems the
+      --   <http://pubs.opengroup.org/onlinepubs/9699919799/functions/execvp.html execvp(3)>
+      --   semantics is used, where if the executable filename does not
+      --   contain a slash (@/@) then the @PATH@ environment variable is
+      --   searched for the executable.
+      --
+      -- * on Windows systems the Win32 @CreateProcess@ semantics is used.
+      --   Briefly: if the filename does not contain a path, then the
+      --   directory containing the parent executable is searched, followed
+      --   by the current directory, then some standard locations, and
+      --   finally the current @PATH@.  An @.exe@ extension is added if the
+      --   filename does not already have an extension.  For full details
+      --   see the
+      --   <http://msdn.microsoft.com/en-us/library/windows/desktop/aa365527%28v=vs.85%29.aspx documentation>
+      --   for the Windows @SearchPath@ API.
+
+
+-- | construct a `ShellCommand` from a string literal
+--
+-- /Since: 1.2.1.0/
+instance IsString CmdSpec where
+  fromString = ShellCommand
 
 data StdStream
   = Inherit                  -- ^ Inherit Handle from parent
@@ -196,6 +225,19 @@ data StdStream
                              -- and newline translation mode (just
                              -- like @Handle@s created by @openFile@).
 
+-- | This function is almost identical to
+-- 'System.Process.createProcess'. The only differences are:
+--
+-- * 'Handle's provided via 'UseHandle' are not closed automatically.
+--
+-- * This function takes an extra @String@ argument to be used in creating
+--   error messages.
+--
+-- This function has been available from the "System.Process.Internals" module
+-- for some time, and is part of the "System.Process" module since version
+-- 1.2.1.0.
+--
+-- /Since: 1.2.1.0/
 createProcess_
   :: String                     -- ^ function name (for error messages)
   -> CreateProcess
@@ -252,6 +294,8 @@ createProcess_ fun CreateProcess{ cmdspec = cmdsp,
      when (proc_handle == -1) $ do
          cFailedDoing <- peek pFailedDoing
          failedDoing <- peekCString cFailedDoing
+         when mb_delegate_ctlc
+           stopDelegateControlC
          throwErrno (fun ++ ": " ++ failedDoing)
 
      hndStdInput  <- mbPipe mb_stdin  pfdStdInput  WriteMode
@@ -291,7 +335,6 @@ startDelegateControlC =
     modifyMVar_ runInteractiveProcess_delegate_ctlc $ \delegating -> do
       case delegating of
         Nothing -> do
---          print ("startDelegateControlC", "Nothing")
           -- We're going to ignore ^C in the parent while there are any
           -- processes using ^C delegation.
           --
@@ -303,30 +346,31 @@ startDelegateControlC =
           return (Just (1, old_int, old_quit))
 
         Just (count, old_int, old_quit) -> do
---          print ("startDelegateControlC", count)
           -- If we're already doing it, just increment the count
           let !count' = count + 1
           return (Just (count', old_int, old_quit))
 
-endDelegateControlC :: ExitCode -> IO ()
-endDelegateControlC exitCode = do
+stopDelegateControlC :: IO ()
+stopDelegateControlC =
     modifyMVar_ runInteractiveProcess_delegate_ctlc $ \delegating -> do
       case delegating of
         Just (1, old_int, old_quit) -> do
---          print ("endDelegateControlC", exitCode, 1 :: Int)
           -- Last process, so restore the old signal handlers
           _ <- installHandler sigINT  old_int  Nothing
           _ <- installHandler sigQUIT old_quit Nothing
           return Nothing
 
         Just (count, old_int, old_quit) -> do
---          print ("endDelegateControlC", exitCode, count)
           -- Not the last, just decrement the count
           let !count' = count - 1
           return (Just (count', old_int, old_quit))
 
         Nothing -> return Nothing -- should be impossible
 
+endDelegateControlC :: ExitCode -> IO ()
+endDelegateControlC exitCode = do
+    stopDelegateControlC
+
     -- And if the process did die due to SIGINT or SIGQUIT then
     -- we throw our equivalent exception here (synchronously).
     --
@@ -419,12 +463,21 @@ createProcess_ fun CreateProcess{ cmdspec = cmdsp,
 runInteractiveProcess_lock :: MVar ()
 runInteractiveProcess_lock = unsafePerformIO $ newMVar ()
 
+-- The following functions are always present in the export list. For
+-- compatibility with the non-Windows code, we provide the same functions with
+-- matching type signatures, but implemented as no-ops. For details, see:
+-- <https://github.com/haskell/process/pull/21>
 startDelegateControlC :: IO ()
 startDelegateControlC = return ()
 
 endDelegateControlC :: ExitCode -> IO ()
 endDelegateControlC _ = return ()
 
+stopDelegateControlC :: IO ()
+stopDelegateControlC = return ()
+
+-- End no-op functions
+
 foreign import ccall unsafe "runInteractiveProcess"
   c_runInteractiveProcess
         :: CWString
