@@ -27,6 +27,10 @@
 #include "STM.h"
 #include "RtsUtils.h"
 
+#if !defined(mingw32_HOST_OS)
+#include "rts/IOManager.h" // for setIOManagerControlFd()
+#endif
+
 #include <string.h>
 
 // one global capability, this is the Capability for non-threaded
@@ -39,7 +43,7 @@ nat enabled_capabilities = 0;
 // The array of Capabilities.  It's important that when we need
 // to allocate more Capabilities we don't have to move the existing
 // Capabilities, because there may be pointers to them in use
-// (e.g. threads in waitForReturnCapability(), see #8209), so this is
+// (e.g. threads in waitForCapability(), see #8209), so this is
 // an array of Capability* rather than an array of Capability.
 Capability **capabilities = NULL;
 
@@ -126,7 +130,7 @@ findSpark (Capability *cap)
       if (n_capabilities == 1) { return NULL; } // makes no sense...
 
       debugTrace(DEBUG_sched,
-                 "cap %d: Trying to steal work from other capabilities", 
+                 "cap %d: Trying to steal work from other capabilities",
                  cap->no);
 
       /* visit cap.s 0..n-1 in sequence until a theft succeeds. We could
@@ -154,7 +158,7 @@ findSpark (Capability *cap)
           if (spark != NULL) {
               cap->spark_stats.converted++;
               traceEventSparkSteal(cap, robbed->no);
-              
+
               return spark;
           }
           // otherwise: no success, try next one
@@ -196,10 +200,10 @@ newReturningTask (Capability *cap, Task *task)
     ASSERT_LOCK_HELD(&cap->lock);
     ASSERT(task->next == NULL);
     if (cap->returning_tasks_hd) {
-	ASSERT(cap->returning_tasks_tl->next == NULL);
-	cap->returning_tasks_tl->next = task;
+        ASSERT(cap->returning_tasks_tl->next == NULL);
+        cap->returning_tasks_tl->next = task;
     } else {
-	cap->returning_tasks_hd = task;
+        cap->returning_tasks_hd = task;
     }
     cap->returning_tasks_tl = task;
 }
@@ -213,7 +217,7 @@ popReturningTask (Capability *cap)
     ASSERT(task);
     cap->returning_tasks_hd = task->next;
     if (!cap->returning_tasks_hd) {
-	cap->returning_tasks_tl = NULL;
+        cap->returning_tasks_tl = NULL;
     }
     task->next = NULL;
     return task;
@@ -255,6 +259,9 @@ initCapability( Capability *cap, nat i )
     cap->spark_stats.converted  = 0;
     cap->spark_stats.gcd        = 0;
     cap->spark_stats.fizzled    = 0;
+#if !defined(mingw32_HOST_OS)
+    cap->io_manager_control_wr_fd = -1;
+#endif
 #endif
     cap->total_allocated        = 0;
 
@@ -263,16 +270,18 @@ initCapability( Capability *cap, nat i )
     cap->f.stgGCFun        = (StgFunPtr)__stg_gc_fun;
 
     cap->mut_lists  = stgMallocBytes(sizeof(bdescr *) *
-				     RtsFlags.GcFlags.generations,
-				     "initCapability");
+                                     RtsFlags.GcFlags.generations,
+                                     "initCapability");
     cap->saved_mut_lists = stgMallocBytes(sizeof(bdescr *) *
                                           RtsFlags.GcFlags.generations,
                                           "initCapability");
 
     for (g = 0; g < RtsFlags.GcFlags.generations; g++) {
-	cap->mut_lists[g] = NULL;
+        cap->mut_lists[g] = NULL;
     }
 
+    cap->weak_ptr_list_hd = NULL;
+    cap->weak_ptr_list_tl = NULL;
     cap->free_tvar_watch_queues = END_STM_WATCH_QUEUE;
     cap->free_invariant_check_queues = END_INVARIANT_CHECK_QUEUE;
     cap->free_trec_chunks = END_STM_CHUNK_LIST;
@@ -288,6 +297,10 @@ initCapability( Capability *cap, nat i )
     cap->r.rCCCS = NULL;
 #endif
 
+    // cap->r.rCurrentTSO is charged for calls to allocate(), so we
+    // don't want it set when not running a Haskell thread.
+    cap->r.rCurrentTSO = NULL;
+
     traceCapCreate(cap);
     traceCapsetAssignCap(CAPSET_OSPROCESS_DEFAULT, i);
     traceCapsetAssignCap(CAPSET_CLOCKDOMAIN_DEFAULT, i);
@@ -317,8 +330,8 @@ initCapabilities( void )
 #ifndef REG_Base
     // We can't support multiple CPUs if BaseReg is not a register
     if (RtsFlags.ParFlags.nNodes > 1) {
-	errorBelch("warning: multiple CPUs not supported in this build, reverting to 1");
-	RtsFlags.ParFlags.nNodes = 1;
+        errorBelch("warning: multiple CPUs not supported in this build, reverting to 1");
+        RtsFlags.ParFlags.nNodes = 1;
     }
 #endif
 
@@ -355,7 +368,7 @@ moreCapabilities (nat from USED_IF_THREADS, nat to USED_IF_THREADS)
     if (to == 1) {
         // THREADED_RTS must work on builds that don't have a mutable
         // BaseReg (eg. unregisterised), so in this case
-	// capabilities[0] must coincide with &MainCapability.
+        // capabilities[0] must coincide with &MainCapability.
         capabilities[0] = &MainCapability;
         initCapability(&MainCapability, 0);
     }
@@ -437,16 +450,15 @@ giveCapabilityToTask (Capability *cap USED_IF_DEBUG, Task *task)
 #endif
 
 /* ----------------------------------------------------------------------------
- * Function:  releaseCapability(Capability*)
+ * releaseCapability
  *
- * Purpose:   Letting go of a capability. Causes a
- *            'returning worker' thread or a 'waiting worker'
- *            to wake up, in that order.
+ * The current Task (cap->task) releases the Capability.  The Capability is
+ * marked free, and if there is any work to do, an appropriate Task is woken up.
  * ------------------------------------------------------------------------- */
 
 #if defined(THREADED_RTS)
 void
-releaseCapability_ (Capability* cap, 
+releaseCapability_ (Capability* cap,
                     rtsBool always_wakeup)
 {
     Task *task;
@@ -460,56 +472,56 @@ releaseCapability_ (Capability* cap,
     // Check to see whether a worker thread can be given
     // the go-ahead to return the result of an external call..
     if (cap->returning_tasks_hd != NULL) {
-	giveCapabilityToTask(cap,cap->returning_tasks_hd);
-	// The Task pops itself from the queue (see waitForReturnCapability())
-	return;
+        giveCapabilityToTask(cap,cap->returning_tasks_hd);
+        // The Task pops itself from the queue (see waitForCapability())
+        return;
     }
 
     // If there is a pending sync, then we should just leave the
     // Capability free.  The thread trying to sync will be about to
-    // call waitForReturnCapability().
+    // call waitForCapability().
     if (pending_sync != 0 && pending_sync != SYNC_GC_PAR) {
       last_free_capability = cap; // needed?
       debugTrace(DEBUG_sched, "sync pending, set capability %d free", cap->no);
       return;
-    } 
+    }
 
     // If the next thread on the run queue is a bound thread,
     // give this Capability to the appropriate Task.
     if (!emptyRunQueue(cap) && peekRunQueue(cap)->bound) {
-	// Make sure we're not about to try to wake ourselves up
-	// ASSERT(task != cap->run_queue_hd->bound);
+        // Make sure we're not about to try to wake ourselves up
+        // ASSERT(task != cap->run_queue_hd->bound);
         // assertion is false: in schedule() we force a yield after
-	// ThreadBlocked, but the thread may be back on the run queue
-	// by now.
-	task = peekRunQueue(cap)->bound->task;
-	giveCapabilityToTask(cap, task);
-	return;
+        // ThreadBlocked, but the thread may be back on the run queue
+        // by now.
+        task = peekRunQueue(cap)->bound->task;
+        giveCapabilityToTask(cap, task);
+        return;
     }
 
     if (!cap->spare_workers) {
-	// Create a worker thread if we don't have one.  If the system
-	// is interrupted, we only create a worker task if there
-	// are threads that need to be completed.  If the system is
-	// shutting down, we never create a new worker.
-	if (sched_state < SCHED_SHUTTING_DOWN || !emptyRunQueue(cap)) {
-	    debugTrace(DEBUG_sched,
-		       "starting new worker on capability %d", cap->no);
-	    startWorkerTask(cap);
-	    return;
-	}
+        // Create a worker thread if we don't have one.  If the system
+        // is interrupted, we only create a worker task if there
+        // are threads that need to be completed.  If the system is
+        // shutting down, we never create a new worker.
+        if (sched_state < SCHED_SHUTTING_DOWN || !emptyRunQueue(cap)) {
+            debugTrace(DEBUG_sched,
+                       "starting new worker on capability %d", cap->no);
+            startWorkerTask(cap);
+            return;
+        }
     }
 
     // If we have an unbound thread on the run queue, or if there's
     // anything else to do, give the Capability to a worker thread.
-    if (always_wakeup || 
+    if (always_wakeup ||
         !emptyRunQueue(cap) || !emptyInbox(cap) ||
         (!cap->disabled && !emptySparkPoolCap(cap)) || globalWorkToDo()) {
-	if (cap->spare_workers) {
-	    giveCapabilityToTask(cap, cap->spare_workers);
-	    // The worker Task pops itself from the queue;
-	    return;
-	}
+        if (cap->spare_workers) {
+            giveCapabilityToTask(cap, cap->spare_workers);
+            // The worker Task pops itself from the queue;
+            return;
+        }
     }
 
 #ifdef PROFILING
@@ -536,62 +548,156 @@ releaseAndWakeupCapability (Capability* cap USED_IF_THREADS)
 }
 
 static void
-releaseCapabilityAndQueueWorker (Capability* cap USED_IF_THREADS)
+enqueueWorker (Capability* cap USED_IF_THREADS)
 {
     Task *task;
 
-    ACQUIRE_LOCK(&cap->lock);
-
     task = cap->running_task;
 
     // If the Task is stopped, we shouldn't be yielding, we should
     // be just exiting.
     ASSERT(!task->stopped);
+    ASSERT(task->worker);
 
-    // If the current task is a worker, save it on the spare_workers
-    // list of this Capability.  A worker can mark itself as stopped,
-    // in which case it is not replaced on the spare_worker queue.
-    // This happens when the system is shutting down (see
-    // Schedule.c:workerStart()).
-    if (!isBoundTask(task))
+    if (cap->n_spare_workers < MAX_SPARE_WORKERS)
     {
-        if (cap->n_spare_workers < MAX_SPARE_WORKERS)
-        {
-            task->next = cap->spare_workers;
-            cap->spare_workers = task;
-            cap->n_spare_workers++;
-        }
-        else
-        {
-            debugTrace(DEBUG_sched, "%d spare workers already, exiting",
-                       cap->n_spare_workers);
-            releaseCapability_(cap,rtsFalse);
-            // hold the lock until after workerTaskStop; c.f. scheduleWorker()
-            workerTaskStop(task);
-            RELEASE_LOCK(&cap->lock);
-            shutdownThread();
-        }
+        task->next = cap->spare_workers;
+        cap->spare_workers = task;
+        cap->n_spare_workers++;
+    }
+    else
+    {
+        debugTrace(DEBUG_sched, "%d spare workers already, exiting",
+                   cap->n_spare_workers);
+        releaseCapability_(cap,rtsFalse);
+        // hold the lock until after workerTaskStop; c.f. scheduleWorker()
+        workerTaskStop(task);
+        RELEASE_LOCK(&cap->lock);
+        shutdownThread();
     }
-    // Bound tasks just float around attached to their TSOs.
-
-    releaseCapability_(cap,rtsFalse);
-
-    RELEASE_LOCK(&cap->lock);
 }
+
 #endif
 
 /* ----------------------------------------------------------------------------
- * waitForReturnCapability (Capability **pCap, Task *task)
+ * waitForWorkerCapability(task)
+ *
+ * waits to be given a Capability, and then returns the Capability.  The task
+ * must be either a worker (and on a cap->spare_workers queue), or a bound Task.
+ * ------------------------------------------------------------------------- */
+
+#if defined(THREADED_RTS)
+
+static Capability * waitForWorkerCapability (Task *task)
+{
+    Capability *cap;
+
+    for (;;) {
+        ACQUIRE_LOCK(&task->lock);
+        // task->lock held, cap->lock not held
+        if (!task->wakeup) waitCondition(&task->cond, &task->lock);
+        cap = task->cap;
+        task->wakeup = rtsFalse;
+        RELEASE_LOCK(&task->lock);
+
+        debugTrace(DEBUG_sched, "woken up on capability %d", cap->no);
+
+        ACQUIRE_LOCK(&cap->lock);
+        if (cap->running_task != NULL) {
+            debugTrace(DEBUG_sched,
+                       "capability %d is owned by another task", cap->no);
+            RELEASE_LOCK(&cap->lock);
+            continue;
+        }
+
+        if (task->cap != cap) {
+            // see Note [migrated bound threads]
+            debugTrace(DEBUG_sched,
+                       "task has been migrated to cap %d", task->cap->no);
+            RELEASE_LOCK(&cap->lock);
+            continue;
+        }
+
+        if (task->incall->tso == NULL) {
+            ASSERT(cap->spare_workers != NULL);
+            // if we're not at the front of the queue, release it
+                // again.  This is unlikely to happen.
+            if (cap->spare_workers != task) {
+                giveCapabilityToTask(cap,cap->spare_workers);
+                RELEASE_LOCK(&cap->lock);
+                continue;
+            }
+            cap->spare_workers = task->next;
+            task->next = NULL;
+            cap->n_spare_workers--;
+        }
+
+        cap->running_task = task;
+        RELEASE_LOCK(&cap->lock);
+        break;
+    }
+
+    return cap;
+}
+
+#endif /* THREADED_RTS */
+
+/* ----------------------------------------------------------------------------
+ * waitForReturnCapability (Task *task)
+ *
+ * The Task should be on the cap->returning_tasks queue of a Capability.  This
+ * function waits for the Task to be woken up, and returns the Capability that
+ * it was woken up on.
+ *
+ * ------------------------------------------------------------------------- */
+
+#if defined(THREADED_RTS)
+
+static Capability * waitForReturnCapability (Task *task)
+{
+    Capability *cap;
+
+    for (;;) {
+        ACQUIRE_LOCK(&task->lock);
+        // task->lock held, cap->lock not held
+        if (!task->wakeup) waitCondition(&task->cond, &task->lock);
+        cap = task->cap;
+        task->wakeup = rtsFalse;
+        RELEASE_LOCK(&task->lock);
+
+        // now check whether we should wake up...
+        ACQUIRE_LOCK(&cap->lock);
+        if (cap->running_task == NULL) {
+            if (cap->returning_tasks_hd != task) {
+                giveCapabilityToTask(cap,cap->returning_tasks_hd);
+                RELEASE_LOCK(&cap->lock);
+                continue;
+            }
+            cap->running_task = task;
+            popReturningTask(cap);
+            RELEASE_LOCK(&cap->lock);
+            break;
+        }
+        RELEASE_LOCK(&cap->lock);
+    }
+
+    return cap;
+}
+
+#endif /* THREADED_RTS */
+
+/* ----------------------------------------------------------------------------
+ * waitForCapability (Capability **pCap, Task *task)
  *
  * Purpose:  when an OS thread returns from an external call,
- * it calls waitForReturnCapability() (via Schedule.resumeThread())
+ * it calls waitForCapability() (via Schedule.resumeThread())
  * to wait for permission to enter the RTS & communicate the
  * result of the external call back to the Haskell thread that
  * made it.
  *
  * ------------------------------------------------------------------------- */
-void
-waitForReturnCapability (Capability **pCap, Task *task)
+
+void waitForCapability (Capability **pCap, Task *task)
 {
 #if !defined(THREADED_RTS)
 
@@ -603,67 +709,42 @@ waitForReturnCapability (Capability **pCap, Task *task)
     Capability *cap = *pCap;
 
     if (cap == NULL) {
-	// Try last_free_capability first
-	cap = last_free_capability;
-	if (cap->running_task) {
-	    nat i;
-	    // otherwise, search for a free capability
+        // Try last_free_capability first
+        cap = last_free_capability;
+        if (cap->running_task) {
+            nat i;
+            // otherwise, search for a free capability
             cap = NULL;
-	    for (i = 0; i < n_capabilities; i++) {
+            for (i = 0; i < n_capabilities; i++) {
                 if (!capabilities[i]->running_task) {
                     cap = capabilities[i];
-		    break;
-		}
-	    }
+                    break;
+                }
+            }
             if (cap == NULL) {
                 // Can't find a free one, use last_free_capability.
                 cap = last_free_capability;
             }
-	}
+        }
 
-	// record the Capability as the one this Task is now assocated with.
-	task->cap = cap;
+        // record the Capability as the one this Task is now assocated with.
+        task->cap = cap;
 
     } else {
-	ASSERT(task->cap == cap);
+        ASSERT(task->cap == cap);
     }
 
-    ACQUIRE_LOCK(&cap->lock);
-
     debugTrace(DEBUG_sched, "returning; I want capability %d", cap->no);
 
+    ACQUIRE_LOCK(&cap->lock);
     if (!cap->running_task) {
-	// It's free; just grab it
-	cap->running_task = task;
-	RELEASE_LOCK(&cap->lock);
+        // It's free; just grab it
+        cap->running_task = task;
+        RELEASE_LOCK(&cap->lock);
     } else {
-	newReturningTask(cap,task);
-	RELEASE_LOCK(&cap->lock);
-
-	for (;;) {
-	    ACQUIRE_LOCK(&task->lock);
-	    // task->lock held, cap->lock not held
-	    if (!task->wakeup) waitCondition(&task->cond, &task->lock);
-	    cap = task->cap;
-	    task->wakeup = rtsFalse;
-	    RELEASE_LOCK(&task->lock);
-
-	    // now check whether we should wake up...
-	    ACQUIRE_LOCK(&cap->lock);
-	    if (cap->running_task == NULL) {
-		if (cap->returning_tasks_hd != task) {
-		    giveCapabilityToTask(cap,cap->returning_tasks_hd);
-		    RELEASE_LOCK(&cap->lock);
-		    continue;
-		}
-		cap->running_task = task;
-		popReturningTask(cap);
-		RELEASE_LOCK(&cap->lock);
-		break;
-	    }
-	    RELEASE_LOCK(&cap->lock);
-	}
-
+        newReturningTask(cap,task);
+        RELEASE_LOCK(&cap->lock);
+        cap = waitForReturnCapability(task);
     }
 
 #ifdef PROFILING
@@ -678,11 +759,30 @@ waitForReturnCapability (Capability **pCap, Task *task)
 #endif
 }
 
-#if defined(THREADED_RTS)
 /* ----------------------------------------------------------------------------
  * yieldCapability
+ *
+ * Give up the Capability, and return when we have it again.  This is called
+ * when either we know that the Capability should be given to another Task, or
+ * there is nothing to do right now.  One of the following is true:
+ *
+ *    - The current Task is a worker, and there's a bound thread at the head of
+ *      the run queue (or vice versa)
+ *
+ *    - The run queue is empty.  We'll be woken up again when there's work to
+ *      do.
+ *
+ *    - Another Task is trying to do parallel GC (pending_sync == SYNC_GC_PAR).
+ *      We should become a GC worker for a while.
+ *
+ *    - Another Task is trying to acquire all the Capabilities (pending_sync !=
+ *      SYNC_GC_PAR), either to do a sequential GC, forkProcess, or
+ *      setNumCapabilities.  We should give up the Capability temporarily.
+ *
  * ------------------------------------------------------------------------- */
 
+#if defined (THREADED_RTS)
+
 /* See Note [GC livelock] in Schedule.c for why we have gcAllowed
    and return the rtsBool */
 rtsBool /* Did we GC? */
@@ -701,63 +801,39 @@ yieldCapability (Capability** pCap, Task *task, rtsBool gcAllowed)
         }
     }
 
-	debugTrace(DEBUG_sched, "giving up capability %d", cap->no);
+    debugTrace(DEBUG_sched, "giving up capability %d", cap->no);
 
-	// We must now release the capability and wait to be woken up
-	// again.
-	task->wakeup = rtsFalse;
-	releaseCapabilityAndQueueWorker(cap);
+    // We must now release the capability and wait to be woken up again.
+    task->wakeup = rtsFalse;
 
-	for (;;) {
-	    ACQUIRE_LOCK(&task->lock);
-	    // task->lock held, cap->lock not held
-	    if (!task->wakeup) waitCondition(&task->cond, &task->lock);
-	    cap = task->cap;
-	    task->wakeup = rtsFalse;
-	    RELEASE_LOCK(&task->lock);
+    ACQUIRE_LOCK(&cap->lock);
 
-	    debugTrace(DEBUG_sched, "woken up on capability %d", cap->no);
+    // If this is a worker thread, put it on the spare_workers queue
+    if (isWorker(task)) {
+        enqueueWorker(cap);
+    }
 
-	    ACQUIRE_LOCK(&cap->lock);
-	    if (cap->running_task != NULL) {
-		debugTrace(DEBUG_sched, 
-			   "capability %d is owned by another task", cap->no);
-		RELEASE_LOCK(&cap->lock);
-		continue;
-	    }
+    releaseCapability_(cap, rtsFalse);
 
-            if (task->cap != cap) {
-                // see Note [migrated bound threads]
-                debugTrace(DEBUG_sched,
-                           "task has been migrated to cap %d", task->cap->no);
-		RELEASE_LOCK(&cap->lock);
-		continue;
-	    }
+    if (isWorker(task) || isBoundTask(task)) {
+        RELEASE_LOCK(&cap->lock);
+        cap = waitForWorkerCapability(task);
+    } else {
+        // Not a worker Task, or a bound Task.  The only way we can be woken up
+        // again is to put ourselves on the returning_tasks queue, so that's
+        // what we do.  We still hold cap->lock at this point
+        // The Task waiting for this Capability does not have it
+        // yet, so we can be sure to be woken up later. (see #10545)
+        newReturningTask(cap,task);
+        RELEASE_LOCK(&cap->lock);
+        cap = waitForReturnCapability(task);
+    }
 
-            if (task->incall->tso == NULL) {
-		ASSERT(cap->spare_workers != NULL);
-		// if we're not at the front of the queue, release it
-		// again.  This is unlikely to happen.
-		if (cap->spare_workers != task) {
-		    giveCapabilityToTask(cap,cap->spare_workers);
-		    RELEASE_LOCK(&cap->lock);
-		    continue;
-		}
-		cap->spare_workers = task->next;
-		task->next = NULL;
-                cap->n_spare_workers--;
-            }
-
-            cap->running_task = task;
-	    RELEASE_LOCK(&cap->lock);
-	    break;
-	}
-
-        debugTrace(DEBUG_sched, "resuming capability %d", cap->no);
-	ASSERT(cap->running_task == task);
+    debugTrace(DEBUG_sched, "resuming capability %d", cap->no);
+    ASSERT(cap->running_task == task);
 
 #ifdef PROFILING
-        cap->r.rCCCS = CCS_SYSTEM;
+    cap->r.rCCCS = CCS_SYSTEM;
 #endif
 
     *pCap = cap;
@@ -767,6 +843,8 @@ yieldCapability (Capability** pCap, Task *task, rtsBool gcAllowed)
     return rtsFalse;
 }
 
+#endif /* THREADED_RTS */
+
 // Note [migrated bound threads]
 //
 // There's a tricky case where:
@@ -798,10 +876,12 @@ yieldCapability (Capability** pCap, Task *task, rtsBool gcAllowed)
 /* ----------------------------------------------------------------------------
  * prodCapability
  *
- * If a Capability is currently idle, wake up a Task on it.  Used to 
+ * If a Capability is currently idle, wake up a Task on it.  Used to
  * get every Capability into the GC.
  * ------------------------------------------------------------------------- */
 
+#if defined (THREADED_RTS)
+
 void
 prodCapability (Capability *cap, Task *task)
 {
@@ -813,6 +893,8 @@ prodCapability (Capability *cap, Task *task)
     RELEASE_LOCK(&cap->lock);
 }
 
+#endif /* THREADED_RTS */
+
 /* ----------------------------------------------------------------------------
  * tryGrabCapability
  *
@@ -820,14 +902,16 @@ prodCapability (Capability *cap, Task *task)
  *
  * ------------------------------------------------------------------------- */
 
+#if defined (THREADED_RTS)
+
 rtsBool
 tryGrabCapability (Capability *cap, Task *task)
 {
     if (cap->running_task != NULL) return rtsFalse;
     ACQUIRE_LOCK(&cap->lock);
     if (cap->running_task != NULL) {
-	RELEASE_LOCK(&cap->lock);
-	return rtsFalse;
+        RELEASE_LOCK(&cap->lock);
+        return rtsFalse;
     }
     task->cap = cap;
     cap->running_task = task;
@@ -872,16 +956,16 @@ shutdownCapability (Capability *cap USED_IF_THREADS,
     for (i = 0; /* i < 50 */; i++) {
         ASSERT(sched_state == SCHED_SHUTTING_DOWN);
 
-	debugTrace(DEBUG_sched, 
-		   "shutting down capability %d, attempt %d", cap->no, i);
-	ACQUIRE_LOCK(&cap->lock);
-	if (cap->running_task) {
-	    RELEASE_LOCK(&cap->lock);
-	    debugTrace(DEBUG_sched, "not owner, yielding");
-	    yieldThread();
-	    continue;
-	}
-	cap->running_task = task;
+        debugTrace(DEBUG_sched,
+                   "shutting down capability %d, attempt %d", cap->no, i);
+        ACQUIRE_LOCK(&cap->lock);
+        if (cap->running_task) {
+            RELEASE_LOCK(&cap->lock);
+            debugTrace(DEBUG_sched, "not owner, yielding");
+            yieldThread();
+            continue;
+        }
+        cap->running_task = task;
 
         if (cap->spare_workers) {
             // Look for workers that have died without removing
@@ -894,7 +978,7 @@ shutdownCapability (Capability *cap USED_IF_THREADS,
             prev = NULL;
             for (t = cap->spare_workers; t != NULL; t = t->next) {
                 if (!osThreadIsAlive(t->id)) {
-                    debugTrace(DEBUG_sched, 
+                    debugTrace(DEBUG_sched,
                                "worker thread %p has died unexpectedly", (void *)(size_t)t->id);
                     cap->n_spare_workers--;
                     if (!prev) {
@@ -907,14 +991,14 @@ shutdownCapability (Capability *cap USED_IF_THREADS,
             }
         }
 
-	if (!emptyRunQueue(cap) || cap->spare_workers) {
-	    debugTrace(DEBUG_sched, 
-		       "runnable threads or workers still alive, yielding");
-	    releaseCapability_(cap,rtsFalse); // this will wake up a worker
-	    RELEASE_LOCK(&cap->lock);
-	    yieldThread();
-	    continue;
-	}
+        if (!emptyRunQueue(cap) || cap->spare_workers) {
+            debugTrace(DEBUG_sched,
+                       "runnable threads or workers still alive, yielding");
+            releaseCapability_(cap,rtsFalse); // this will wake up a worker
+            RELEASE_LOCK(&cap->lock);
+            yieldThread();
+            continue;
+        }
 
         // If "safe", then busy-wait for any threads currently doing
         // foreign calls.  If we're about to unload this DLL, for
@@ -923,10 +1007,10 @@ shutdownCapability (Capability *cap USED_IF_THREADS,
         // We can be a bit more relaxed when this is a standalone
         // program that is about to terminate, and let safe=false.
         if (cap->suspended_ccalls && safe) {
-	    debugTrace(DEBUG_sched, 
-		       "thread(s) are involved in foreign calls, yielding");
+            debugTrace(DEBUG_sched,
+                       "thread(s) are involved in foreign calls, yielding");
             cap->running_task = NULL;
-	    RELEASE_LOCK(&cap->lock);
+            RELEASE_LOCK(&cap->lock);
             // The IO manager thread might have been slow to start up,
             // so the first attempt to kill it might not have
             // succeeded.  Just in case, try again - the kill message
@@ -940,14 +1024,14 @@ shutdownCapability (Capability *cap USED_IF_THREADS,
         }
 
         traceSparkCounters(cap);
-	RELEASE_LOCK(&cap->lock);
-	break;
+        RELEASE_LOCK(&cap->lock);
+        break;
     }
     // we now have the Capability, its run queue and spare workers
     // list are both empty.
 
     // ToDo: we can't drop this mutex, because there might still be
-    // threads performing foreign calls that will eventually try to 
+    // threads performing foreign calls that will eventually try to
     // return via resumeThread() and attempt to grab cap->lock.
     // closeMutex(&cap->lock);
 #endif
@@ -1059,7 +1143,7 @@ rtsBool checkSparkCountInvariant (void)
         sparks.fizzled   += capabilities[i]->spark_stats.fizzled;
         remaining        += sparkPoolSize(capabilities[i]->sparks);
     }
-    
+
     /* The invariant is
      *   created = converted + remaining + gcd + fizzled
      */
@@ -1073,3 +1157,15 @@ rtsBool checkSparkCountInvariant (void)
 
 }
 #endif
+
+#if !defined(mingw32_HOST_OS)
+void setIOManagerControlFd(nat cap_no USED_IF_THREADS, int fd USED_IF_THREADS) {
+#if defined(THREADED_RTS)
+    if (cap_no < n_capabilities) {
+        capabilities[cap_no]->io_manager_control_wr_fd = fd;
+    } else {
+        errorBelch("warning: setIOManagerControlFd called with illegal capability number.");
+    }
+#endif
+}
+#endif
