@@ -144,6 +144,11 @@ typedef struct _ObjectCode {
 extern ObjectCode *objects;
 extern ObjectCode *unloaded_objects;
 
+#ifdef THREADED_RTS
+extern Mutex linker_mutex;
+extern Mutex linker_unloaded_mutex;
+#endif
+
 void exitLinker( void );
 
 void freeObjectCode (ObjectCode *oc);
