@@ -3,10 +3,12 @@
  * (c) The GHC Team, 1994-2000.
  *
  * Heap printer
- * 
+ *
  * ---------------------------------------------------------------------------*/
 
 #include "PosixSource.h"
+#include "ghcconfig.h"
+
 #include "Rts.h"
 #include "rts/Bytecodes.h"  /* for InstrPtr */
 
@@ -33,10 +35,7 @@ static void    insert        ( StgWord value, const char *name );
 #endif
 #if 0 /* unused but might be useful sometime */
 static rtsBool lookup_name   ( char *name, StgWord *result );
-static void    enZcode       ( char *in, char *out );
 #endif
-static char    unZcode       ( char ch );
-static void    printZcoded   ( const char *raw );
 
 /* --------------------------------------------------------------------------
  * Printer
@@ -47,12 +46,13 @@ void printPtr( StgPtr p )
     const char *raw;
     raw = lookupGHCName(p);
     if (raw != NULL) {
-        printZcoded(raw);
+        debugBelch("<%s>", raw);
+        debugBelch("[%p]", p);
     } else {
         debugBelch("%p", p);
     }
 }
-  
+
 void printObj( StgClosure *obj )
 {
     debugBelch("Object "); printPtr((StgPtr)obj); debugBelch(" = ");
@@ -133,11 +133,11 @@ printClosure( StgClosure *obj )
 
             debugBelch("%s(", GET_CON_DESC(con_info));
             for (i = 0; i < info->layout.payload.ptrs; ++i) {
-		if (i != 0) debugBelch(", ");
+                if (i != 0) debugBelch(", ");
                 printPtr((StgPtr)obj->payload[i]);
             }
             for (j = 0; j < info->layout.payload.nptrs; ++j) {
-		if (i != 0 || j != 0) debugBelch(", ");
+                if (i != 0 || j != 0) debugBelch(", ");
                 debugBelch("%p#", obj->payload[i+j]);
             }
             debugBelch(")\n");
@@ -145,28 +145,28 @@ printClosure( StgClosure *obj )
         }
 
     case FUN:
-    case FUN_1_0: case FUN_0_1: 
+    case FUN_1_0: case FUN_0_1:
     case FUN_1_1: case FUN_0_2: case FUN_2_0:
     case FUN_STATIC:
-	debugBelch("FUN/%d(",(int)itbl_to_fun_itbl(info)->f.arity);
-	printPtr((StgPtr)obj->header.info);
+        debugBelch("FUN/%d(",(int)itbl_to_fun_itbl(info)->f.arity);
+        printPtr((StgPtr)obj->header.info);
 #ifdef PROFILING
-	debugBelch(", %s", obj->header.prof.ccs->cc->label);
+        debugBelch(", %s", obj->header.prof.ccs->cc->label);
 #endif
-	printStdObjPayload(obj);
-	break;
+        printStdObjPayload(obj);
+        break;
 
     case PRIM:
-	debugBelch("PRIM(");
-	printPtr((StgPtr)obj->header.info);
-	printStdObjPayload(obj);
-	break;
+        debugBelch("PRIM(");
+        printPtr((StgPtr)obj->header.info);
+        printStdObjPayload(obj);
+        break;
 
     case MUT_PRIM:
         debugBelch("MUT_PRIM(");
-	printPtr((StgPtr)obj->header.info);
-	printStdObjPayload(obj);
-	break;
+        printPtr((StgPtr)obj->header.info);
+        printStdObjPayload(obj);
+        break;
 
     case THUNK:
     case THUNK_1_0: case THUNK_0_1:
@@ -181,9 +181,9 @@ printClosure( StgClosure *obj )
             break;
 
     case THUNK_SELECTOR:
-	printStdObjHdr(obj, "THUNK_SELECTOR");
-	debugBelch(", %p)\n", ((StgSelector *)obj)->selectee);
-	break;
+        printStdObjHdr(obj, "THUNK_SELECTOR");
+        debugBelch(", %p)\n", ((StgSelector *)obj)->selectee);
+        break;
 
     case BCO:
             disassemble( (StgBCO*)obj );
@@ -191,7 +191,7 @@ printClosure( StgClosure *obj )
 
     case AP:
         {
-	    StgAP* ap = (StgAP*)obj;
+            StgAP* ap = (StgAP*)obj;
             StgWord i;
             debugBelch("AP("); printPtr((StgPtr)ap->fun);
             for (i = 0; i < ap->n_args; ++i) {
@@ -204,10 +204,10 @@ printClosure( StgClosure *obj )
 
     case PAP:
         {
-	    StgPAP* pap = (StgPAP*)obj;
+            StgPAP* pap = (StgPAP*)obj;
             StgWord i;
-            debugBelch("PAP/%d(",(int)pap->arity); 
-	    printPtr((StgPtr)pap->fun);
+            debugBelch("PAP/%d(",(int)pap->arity);
+            printPtr((StgPtr)pap->fun);
             for (i = 0; i < pap->n_args; ++i) {
                 debugBelch(", ");
                 printPtr((StgPtr)pap->payload[i]);
@@ -218,7 +218,7 @@ printClosure( StgClosure *obj )
 
     case AP_STACK:
         {
-	    StgAP_STACK* ap = (StgAP_STACK*)obj;
+            StgAP_STACK* ap = (StgAP_STACK*)obj;
             StgWord i;
             debugBelch("AP_STACK("); printPtr((StgPtr)ap->fun);
             for (i = 0; i < ap->size; ++i) {
@@ -230,27 +230,27 @@ printClosure( StgClosure *obj )
         }
 
     case IND:
-            debugBelch("IND("); 
+            debugBelch("IND(");
             printPtr((StgPtr)((StgInd*)obj)->indirectee);
-            debugBelch(")\n"); 
+            debugBelch(")\n");
             break;
 
     case IND_PERM:
-            debugBelch("IND("); 
+            debugBelch("IND(");
             printPtr((StgPtr)((StgInd*)obj)->indirectee);
-            debugBelch(")\n"); 
+            debugBelch(")\n");
             break;
 
     case IND_STATIC:
-            debugBelch("IND_STATIC("); 
+            debugBelch("IND_STATIC(");
             printPtr((StgPtr)((StgInd*)obj)->indirectee);
-            debugBelch(")\n"); 
+            debugBelch(")\n");
             break;
 
     case BLACKHOLE:
-            debugBelch("BLACKHOLE("); 
+            debugBelch("BLACKHOLE(");
             printPtr((StgPtr)((StgInd*)obj)->indirectee);
-            debugBelch(")\n"); 
+            debugBelch(")\n");
             break;
 
     /* Cannot happen -- use default case.
@@ -267,7 +267,7 @@ printClosure( StgClosure *obj )
             printPtr((StgPtr)GET_INFO((StgClosure *)u));
             debugBelch(",");
             printPtr((StgPtr)u->updatee);
-            debugBelch(")\n"); 
+            debugBelch(")\n");
             break;
         }
 
@@ -278,7 +278,7 @@ printClosure( StgClosure *obj )
             printPtr((StgPtr)GET_INFO((StgClosure *)u));
             debugBelch(",");
             printPtr((StgPtr)u->handler);
-            debugBelch(")\n"); 
+            debugBelch(")\n");
             break;
         }
 
@@ -287,7 +287,7 @@ printClosure( StgClosure *obj )
             StgUnderflowFrame* u = (StgUnderflowFrame*)obj;
             debugBelch("UNDERFLOW_FRAME(");
             printPtr((StgPtr)u->next_chunk);
-            debugBelch(")\n"); 
+            debugBelch(")\n");
             break;
         }
 
@@ -296,7 +296,7 @@ printClosure( StgClosure *obj )
             StgStopFrame* u = (StgStopFrame*)obj;
             debugBelch("STOP_FRAME(");
             printPtr((StgPtr)GET_INFO((StgClosure *)u));
-            debugBelch(")\n"); 
+            debugBelch(")\n");
             break;
         }
 
@@ -304,29 +304,44 @@ printClosure( StgClosure *obj )
         {
             StgWord i;
             debugBelch("ARR_WORDS(\"");
-	    for (i=0; i<arr_words_words((StgArrWords *)obj); i++)
-	      debugBelch("%" FMT_Word, (W_)((StgArrWords *)obj)->payload[i]);
+            for (i=0; i<arr_words_words((StgArrWords *)obj); i++)
+              debugBelch("%" FMT_Word, (W_)((StgArrWords *)obj)->payload[i]);
             debugBelch("\")\n");
             break;
         }
 
     case MUT_ARR_PTRS_CLEAN:
-	debugBelch("MUT_ARR_PTRS_CLEAN(size=%" FMT_Word ")\n", (W_)((StgMutArrPtrs *)obj)->ptrs);
-	break;
+        debugBelch("MUT_ARR_PTRS_CLEAN(size=%" FMT_Word ")\n", (W_)((StgMutArrPtrs *)obj)->ptrs);
+        break;
 
     case MUT_ARR_PTRS_DIRTY:
-	debugBelch("MUT_ARR_PTRS_DIRTY(size=%" FMT_Word ")\n", (W_)((StgMutArrPtrs *)obj)->ptrs);
-	break;
+        debugBelch("MUT_ARR_PTRS_DIRTY(size=%" FMT_Word ")\n", (W_)((StgMutArrPtrs *)obj)->ptrs);
+        break;
 
     case MUT_ARR_PTRS_FROZEN:
-	debugBelch("MUT_ARR_PTRS_FROZEN(size=%" FMT_Word ")\n", (W_)((StgMutArrPtrs *)obj)->ptrs);
-	break;
+        debugBelch("MUT_ARR_PTRS_FROZEN(size=%" FMT_Word ")\n", (W_)((StgMutArrPtrs *)obj)->ptrs);
+        break;
+
+    case SMALL_MUT_ARR_PTRS_CLEAN:
+        debugBelch("SMALL_MUT_ARR_PTRS_CLEAN(size=%" FMT_Word ")\n",
+                   (W_)((StgSmallMutArrPtrs *)obj)->ptrs);
+        break;
+
+    case SMALL_MUT_ARR_PTRS_DIRTY:
+        debugBelch("SMALL_MUT_ARR_PTRS_DIRTY(size=%" FMT_Word ")\n",
+                   (W_)((StgSmallMutArrPtrs *)obj)->ptrs);
+        break;
+
+    case SMALL_MUT_ARR_PTRS_FROZEN:
+        debugBelch("SMALL_MUT_ARR_PTRS_FROZEN(size=%" FMT_Word ")\n",
+                   (W_)((StgSmallMutArrPtrs *)obj)->ptrs);
+        break;
 
     case MVAR_CLEAN:
     case MVAR_DIRTY:
         {
-	  StgMVar* mv = (StgMVar*)obj;
-	  debugBelch("MVAR(head=%p, tail=%p, value=%p)\n", mv->head, mv->tail, mv->value);
+          StgMVar* mv = (StgMVar*)obj;
+          debugBelch("MVAR(head=%p, tail=%p, value=%p)\n", mv->head, mv->tail, mv->value);
           break;
         }
 
@@ -339,32 +354,32 @@ printClosure( StgClosure *obj )
 
     case MUT_VAR_CLEAN:
         {
-	  StgMutVar* mv = (StgMutVar*)obj;
-	  debugBelch("MUT_VAR_CLEAN(var=%p)\n", mv->var);
+          StgMutVar* mv = (StgMutVar*)obj;
+          debugBelch("MUT_VAR_CLEAN(var=%p)\n", mv->var);
           break;
         }
 
     case MUT_VAR_DIRTY:
         {
-	  StgMutVar* mv = (StgMutVar*)obj;
-	  debugBelch("MUT_VAR_DIRTY(var=%p)\n", mv->var);
+          StgMutVar* mv = (StgMutVar*)obj;
+          debugBelch("MUT_VAR_DIRTY(var=%p)\n", mv->var);
           break;
         }
 
     case WEAK:
-            debugBelch("WEAK("); 
-	    debugBelch(" key=%p value=%p finalizer=%p", 
-		    (StgPtr)(((StgWeak*)obj)->key),
-		    (StgPtr)(((StgWeak*)obj)->value),
-		    (StgPtr)(((StgWeak*)obj)->finalizer));
-            debugBelch(")\n"); 
-	    /* ToDo: chase 'link' ? */
+            debugBelch("WEAK(");
+            debugBelch(" key=%p value=%p finalizer=%p",
+                    (StgPtr)(((StgWeak*)obj)->key),
+                    (StgPtr)(((StgWeak*)obj)->value),
+                    (StgPtr)(((StgWeak*)obj)->finalizer));
+            debugBelch(")\n");
+            /* ToDo: chase 'link' ? */
             break;
 
     case TSO:
-      debugBelch("TSO("); 
+      debugBelch("TSO(");
       debugBelch("%lu (%p)",(unsigned long)(((StgTSO*)obj)->id), (StgTSO*)obj);
-      debugBelch(")\n"); 
+      debugBelch(")\n");
       break;
 
     case STACK:
@@ -374,9 +389,9 @@ printClosure( StgClosure *obj )
 #if 0
       /* Symptomatic of a problem elsewhere, have it fall-through & fail */
     case EVACUATED:
-      debugBelch("EVACUATED("); 
+      debugBelch("EVACUATED(");
       printClosure((StgEvacuated*)obj->evacuee);
-      debugBelch(")\n"); 
+      debugBelch(")\n");
       break;
 #endif
 
@@ -423,22 +438,22 @@ printStackObj( StgPtr sp )
         printPtr((StgPtr)*sp);
         if (c == (StgClosure*)&stg_ctoi_R1p_info) {
            debugBelch("\t\t\tstg_ctoi_ret_R1p_info\n" );
-	} else
+        } else
         if (c == (StgClosure*)&stg_ctoi_R1n_info) {
            debugBelch("\t\t\tstg_ctoi_ret_R1n_info\n" );
-	} else
+        } else
         if (c == (StgClosure*)&stg_ctoi_F1_info) {
            debugBelch("\t\t\tstg_ctoi_ret_F1_info\n" );
-	} else
+        } else
         if (c == (StgClosure*)&stg_ctoi_D1_info) {
            debugBelch("\t\t\tstg_ctoi_ret_D1_info\n" );
-	} else
+        } else
         if (c == (StgClosure*)&stg_ctoi_V_info) {
            debugBelch("\t\t\tstg_ctoi_ret_V_info\n" );
-	} else
+        } else
         if (get_itbl(c)->type == BCO) {
            debugBelch("\t\t\t");
-           debugBelch("BCO(...)\n"); 
+           debugBelch("BCO(...)\n");
         }
         else {
            debugBelch("\t\t\t");
@@ -447,7 +462,7 @@ printStackObj( StgPtr sp )
         sp += 1;
 
     return sp;
-    
+
 }
 
 static void
@@ -456,13 +471,13 @@ printSmallBitmap( StgPtr spBottom, StgPtr payload, StgWord bitmap, nat size )
     nat i;
 
     for(i = 0; i < size; i++, bitmap >>= 1 ) {
-	debugBelch("   stk[%ld] (%p) = ", (long)(spBottom-(payload+i)), payload+i);
-	if ((bitmap & 1) == 0) {
-	    printPtr((P_)payload[i]);
-	    debugBelch("\n");
-	} else {
-	    debugBelch("Word# %" FMT_Word "\n", (W_)payload[i]);
-	}
+        debugBelch("   stk[%ld] (%p) = ", (long)(spBottom-(payload+i)), payload+i);
+        if ((bitmap & 1) == 0) {
+            printPtr((P_)payload[i]);
+            debugBelch("\n");
+        } else {
+            debugBelch("Word# %" FMT_Word "\n", (W_)payload[i]);
+        }
     }
 }
 
@@ -474,17 +489,17 @@ printLargeBitmap( StgPtr spBottom, StgPtr payload, StgLargeBitmap* large_bitmap,
 
     i = 0;
     for (bmp=0; i < size; bmp++) {
-	StgWord bitmap = large_bitmap->bitmap[bmp];
-	j = 0;
-	for(; i < size && j < BITS_IN(W_); j++, i++, bitmap >>= 1 ) {
-	    debugBelch("   stk[%" FMT_Word "] (%p) = ", (W_)(spBottom-(payload+i)), payload+i);
-	    if ((bitmap & 1) == 0) {
-		printPtr((P_)payload[i]);
-		debugBelch("\n");
-	    } else {
-		debugBelch("Word# %" FMT_Word "\n", (W_)payload[i]);
-	    }
-	}
+        StgWord bitmap = large_bitmap->bitmap[bmp];
+        j = 0;
+        for(; i < size && j < BITS_IN(W_); j++, i++, bitmap >>= 1 ) {
+            debugBelch("   stk[%" FMT_Word "] (%p) = ", (W_)(spBottom-(payload+i)), payload+i);
+            if ((bitmap & 1) == 0) {
+                printPtr((P_)payload[i]);
+                debugBelch("\n");
+            } else {
+                debugBelch("Word# %" FMT_Word "\n", (W_)payload[i]);
+            }
+        }
     }
 }
 
@@ -497,70 +512,70 @@ printStackChunk( StgPtr sp, StgPtr spBottom )
     ASSERT(sp <= spBottom);
     for (; sp < spBottom; sp += stack_frame_sizeW((StgClosure *)sp)) {
 
-	info = get_itbl((StgClosure *)sp);
+        info = get_itbl((StgClosure *)sp);
 
-	switch (info->type) {
-	    
-	case UPDATE_FRAME:
-	case CATCH_FRAME:
+        switch (info->type) {
+
+        case UPDATE_FRAME:
+        case CATCH_FRAME:
         case UNDERFLOW_FRAME:
         case STOP_FRAME:
             printObj((StgClosure*)sp);
-	    continue;
+            continue;
 
         case RET_SMALL:
-	    debugBelch("RET_SMALL (%p)\n", info);
-	    bitmap = info->layout.bitmap;
-	    printSmallBitmap(spBottom, sp+1, 
-			     BITMAP_BITS(bitmap), BITMAP_SIZE(bitmap));
-	    continue;
+            debugBelch("RET_SMALL (%p)\n", info);
+            bitmap = info->layout.bitmap;
+            printSmallBitmap(spBottom, sp+1,
+                             BITMAP_BITS(bitmap), BITMAP_SIZE(bitmap));
+            continue;
 
-	case RET_BCO: {
-	    StgBCO *bco;
-	    
-	    bco = ((StgBCO *)sp[1]);
+        case RET_BCO: {
+            StgBCO *bco;
 
-	    debugBelch("RET_BCO (%p)\n", sp);
-	    printLargeBitmap(spBottom, sp+2,
-			     BCO_BITMAP(bco), BCO_BITMAP_SIZE(bco));
-	    continue;
-	}
+            bco = ((StgBCO *)sp[1]);
 
-	case RET_BIG:
-	    barf("todo");
+            debugBelch("RET_BCO (%p)\n", sp);
+            printLargeBitmap(spBottom, sp+2,
+                             BCO_BITMAP(bco), BCO_BITMAP_SIZE(bco));
+            continue;
+        }
 
-	case RET_FUN:
-	{
-	    StgFunInfoTable *fun_info;
-	    StgRetFun *ret_fun;
+        case RET_BIG:
+            barf("todo");
 
-	    ret_fun = (StgRetFun *)sp;
-	    fun_info = get_fun_itbl(ret_fun->fun);
-	    debugBelch("RET_FUN (%p) (type=%d)\n", ret_fun->fun, (int)fun_info->f.fun_type);
-	    switch (fun_info->f.fun_type) {
-	    case ARG_GEN:
-		printSmallBitmap(spBottom, sp+2,
-				 BITMAP_BITS(fun_info->f.b.bitmap),
-				 BITMAP_SIZE(fun_info->f.b.bitmap));
-		break;
-	    case ARG_GEN_BIG:
-		printLargeBitmap(spBottom, sp+2,
-				 GET_FUN_LARGE_BITMAP(fun_info),
-				 GET_FUN_LARGE_BITMAP(fun_info)->size);
-		break;
-	    default:
-		printSmallBitmap(spBottom, sp+2,
-				 BITMAP_BITS(stg_arg_bitmaps[fun_info->f.fun_type]),
-				 BITMAP_SIZE(stg_arg_bitmaps[fun_info->f.fun_type]));
-		break;
-	    }
-	    continue;
-	}
-	   
-	default:
-	    debugBelch("unknown object %d\n", (int)info->type);
-	    barf("printStackChunk");
-	}
+        case RET_FUN:
+        {
+            StgFunInfoTable *fun_info;
+            StgRetFun *ret_fun;
+
+            ret_fun = (StgRetFun *)sp;
+            fun_info = get_fun_itbl(ret_fun->fun);
+            debugBelch("RET_FUN (%p) (type=%d)\n", ret_fun->fun, (int)fun_info->f.fun_type);
+            switch (fun_info->f.fun_type) {
+            case ARG_GEN:
+                printSmallBitmap(spBottom, sp+2,
+                                 BITMAP_BITS(fun_info->f.b.bitmap),
+                                 BITMAP_SIZE(fun_info->f.b.bitmap));
+                break;
+            case ARG_GEN_BIG:
+                printLargeBitmap(spBottom, sp+2,
+                                 GET_FUN_LARGE_BITMAP(fun_info),
+                                 GET_FUN_LARGE_BITMAP(fun_info)->size);
+                break;
+            default:
+                printSmallBitmap(spBottom, sp+2,
+                                 BITMAP_BITS(stg_arg_bitmaps[fun_info->f.fun_type]),
+                                 BITMAP_SIZE(stg_arg_bitmaps[fun_info->f.fun_type]));
+                break;
+            }
+            continue;
+        }
+
+        default:
+            debugBelch("unknown object %d\n", (int)info->type);
+            barf("printStackChunk");
+        }
     }
 }
 
@@ -631,134 +646,6 @@ static rtsBool lookup_name( char *name, StgWord *result )
 }
 #endif
 
-/* Code from somewhere inside GHC (circa 1994)
- * * Z-escapes:
- *     "std"++xs -> "Zstd"++xs
- *     char_to_c 'Z'  = "ZZ"
- *     char_to_c '&'  = "Za"
- *     char_to_c '|'  = "Zb"
- *     char_to_c ':'  = "Zc"
- *     char_to_c '/'  = "Zd"
- *     char_to_c '='  = "Ze"
- *     char_to_c '>'  = "Zg"
- *     char_to_c '#'  = "Zh"
- *     char_to_c '<'  = "Zl"
- *     char_to_c '-'  = "Zm"
- *     char_to_c '!'  = "Zn"
- *     char_to_c '.'  = "Zo"
- *     char_to_c '+'  = "Zp"
- *     char_to_c '\'' = "Zq"
- *     char_to_c '*'  = "Zt"
- *     char_to_c '_'  = "Zu"
- *     char_to_c c    = "Z" ++ show (ord c)
- */
-static char unZcode( char ch )
-{
-    switch (ch) {
-    case 'a'  : return ('&');
-    case 'b'  : return ('|');
-    case 'c'  : return (':');
-    case 'd'  : return ('/');
-    case 'e'  : return ('=');
-    case 'g'  : return ('>');
-    case 'h'  : return ('#');
-    case 'l'  : return ('<');
-    case 'm'  : return ('-');
-    case 'n'  : return ('!');
-    case 'o'  : return ('.');
-    case 'p'  : return ('+');
-    case 'q'  : return ('\'');
-    case 't'  : return ('*');
-    case 'u'  : return ('_');
-    case 'Z'  :
-    case '\0' : return ('Z');
-    default   : return (ch);
-    }
-}
-
-#if 0
-/* Precondition: out big enough to handle output (about twice length of in) */
-static void enZcode( char *in, char *out )
-{
-    int i, j;
-
-    j = 0;
-    out[ j++ ] = '_';
-    for( i = 0; in[i] != '\0'; ++i ) {
-        switch (in[i]) {
-        case 'Z'  : 
-                out[j++] = 'Z';
-                out[j++] = 'Z';
-                break;
-        case '&'  : 
-                out[j++] = 'Z';
-                out[j++] = 'a';
-                break;
-        case '|'  : 
-                out[j++] = 'Z';
-                out[j++] = 'b';
-                break;
-        case ':'  : 
-                out[j++] = 'Z';
-                out[j++] = 'c';
-                break;
-        case '/'  : 
-                out[j++] = 'Z';
-                out[j++] = 'd';
-                break;
-        case '='  : 
-                out[j++] = 'Z';
-                out[j++] = 'e';
-                break;
-        case '>'  : 
-                out[j++] = 'Z';
-                out[j++] = 'g';
-                break;
-        case '#'  : 
-                out[j++] = 'Z';
-                out[j++] = 'h';
-                break;
-        case '<'  : 
-                out[j++] = 'Z';
-                out[j++] = 'l';
-                break;
-        case '-'  : 
-                out[j++] = 'Z';
-                out[j++] = 'm';
-                break;
-        case '!'  : 
-                out[j++] = 'Z';
-                out[j++] = 'n';
-                break;
-        case '.'  : 
-                out[j++] = 'Z';
-                out[j++] = 'o';
-                break;
-        case '+'  : 
-                out[j++] = 'Z';
-                out[j++] = 'p';
-                break;
-        case '\'' : 
-                out[j++] = 'Z';
-                out[j++] = 'q';
-                break;
-        case '*'  : 
-                out[j++] = 'Z';
-                out[j++] = 't';
-                break;
-        case '_'  : 
-                out[j++] = 'Z';
-                out[j++] = 'u';
-                break;
-        default :
-                out[j++] = in[i];
-                break;
-        }
-    }
-    out[j] = '\0';
-}
-#endif
-
 const char *lookupGHCName( void *addr )
 {
     nat i;
@@ -771,31 +658,24 @@ const char *lookupGHCName( void *addr )
     }
 }
 
-static void printZcoded( const char *raw )
-{
-    nat j = 0;
-    
-    while ( raw[j] != '\0' ) {
-        if (raw[j] == 'Z') {
-            debugBelch("%c", unZcode(raw[j+1]));
-            j = j + 2;
-        } else {
-            debugBelch("%c", unZcode(raw[j+1]));
-            j = j + 1;
-        }
-    }
-}
-
 /* --------------------------------------------------------------------------
  * Symbol table loading
  * ------------------------------------------------------------------------*/
 
 /* Causing linking trouble on Win32 plats, so I'm
-   disabling this for now. 
+   disabling this for now.
 */
 #ifdef USING_LIBBFD
-
-#include <bfd.h>
+#    define PACKAGE 1
+#    define PACKAGE_VERSION 1
+/* Those PACKAGE_* defines are workarounds for bfd:
+ *     https://sourceware.org/bugzilla/show_bug.cgi?id=14243
+ * ghc's build system filter PACKAGE_* values out specifically to avoid clashes
+ * with user's autoconf-based Cabal packages.
+ * It's a shame <bfd.h> checks for unrelated fields instead of actually used
+ * macros.
+ */
+#    include <bfd.h>
 
 /* Fairly ad-hoc piece of code that seems to filter out a lot of
  * rubbish like the obj-splitting symbols
@@ -805,17 +685,17 @@ static rtsBool isReal( flagword flags STG_UNUSED, const char *name )
 {
 #if 0
     /* ToDo: make this work on BFD */
-    int tp = type & N_TYPE;    
+    int tp = type & N_TYPE;
     if (tp == N_TEXT || tp == N_DATA) {
         return (name[0] == '_' && name[1] != '_');
     } else {
         return rtsFalse;
     }
 #else
-    if (*name == '\0'  || 
-	(name[0] == 'g' && name[1] == 'c' && name[2] == 'c') ||
-	(name[0] == 'c' && name[1] == 'c' && name[2] == '.')) {
-	return rtsFalse;
+    if (*name == '\0'  ||
+        (name[0] == 'g' && name[1] == 'c' && name[2] == 'c') ||
+        (name[0] == 'c' && name[1] == 'c' && name[2] == '.')) {
+        return rtsFalse;
     }
     return rtsTrue;
 #endif
@@ -829,53 +709,52 @@ extern void DEBUG_LoadSymbols( char *name )
     bfd_init();
     abfd = bfd_openr(name, "default");
     if (abfd == NULL) {
-	barf("can't open executable %s to get symbol table", name);
+        barf("can't open executable %s to get symbol table", name);
     }
     if (!bfd_check_format_matches (abfd, bfd_object, &matching)) {
-	barf("mismatch");
+        barf("mismatch");
     }
 
     {
-	long storage_needed;
-	asymbol **symbol_table;
-	long number_of_symbols;
+        long storage_needed;
+        asymbol **symbol_table;
+        long number_of_symbols;
         long num_real_syms = 0;
-	long i;
-     
-	storage_needed = bfd_get_symtab_upper_bound (abfd);
-     
-	if (storage_needed < 0) {
-	    barf("can't read symbol table");
-	}     
-#if 0
-	if (storage_needed == 0) {
-	    debugBelch("no storage needed");
-	}
-#endif
-	symbol_table = (asymbol **) stgMallocBytes(storage_needed,"DEBUG_LoadSymbols");
+        long i;
 
-	number_of_symbols = bfd_canonicalize_symtab (abfd, symbol_table);
-     
-	if (number_of_symbols < 0) {
-	    barf("can't canonicalise symbol table");
-	}
+        storage_needed = bfd_get_symtab_upper_bound (abfd);
+
+        if (storage_needed < 0) {
+            barf("can't read symbol table");
+        }
+#if 0
+        if (storage_needed == 0) {
+            debugBelch("no storage needed");
+        }
+#endif
+        symbol_table = (asymbol **) stgMallocBytes(storage_needed,"DEBUG_LoadSymbols");
+
+        number_of_symbols = bfd_canonicalize_symtab (abfd, symbol_table);
+
+        if (number_of_symbols < 0) {
+            barf("can't canonicalise symbol table");
+        }
 
         for( i = 0; i != number_of_symbols; ++i ) {
             symbol_info info;
             bfd_get_symbol_info(abfd,symbol_table[i],&info);
-            /*debugBelch("\t%c\t0x%x      \t%s\n",info.type,(nat)info.value,info.name); */
             if (isReal(info.type, info.name)) {
                 num_real_syms += 1;
             }
         }
-    
+
         IF_DEBUG(interpreter,
-                 debugBelch("Loaded %ld symbols. Of which %ld are real symbols\n", 
+                 debugBelch("Loaded %ld symbols. Of which %ld are real symbols\n",
                          number_of_symbols, num_real_syms)
                  );
 
         reset_table( num_real_syms );
-    
+
         for( i = 0; i != number_of_symbols; ++i ) {
             symbol_info info;
             bfd_get_symbol_info(abfd,symbol_table[i],&info);
@@ -889,16 +768,16 @@ extern void DEBUG_LoadSymbols( char *name )
     prepare_table();
 }
 
-#else /* HAVE_BFD_H */
+#else /* USING_LIBBFD */
 
 extern void DEBUG_LoadSymbols( char *name STG_UNUSED )
 {
   /* nothing, yet */
 }
 
-#endif /* HAVE_BFD_H */
+#endif /* USING_LIBBFD */
 
-void findPtr(P_ p, int);		/* keep gcc -Wall happy */
+void findPtr(P_ p, int);                /* keep gcc -Wall happy */
 
 int searched = 0;
 
@@ -974,7 +853,7 @@ findPtr(P_ p, int follow)
 
    Todo: support for more closure types, and support for non pointer fields in the
    payload.
-*/ 
+*/
 
 void prettyPrintClosure_ (StgClosure *);
 
@@ -992,7 +871,7 @@ void prettyPrintClosure_ (StgClosure *obj)
     /* collapse any indirections */
     unsigned int type;
     type = get_itbl(obj)->type;
-           
+
     while (type == IND ||
            type == IND_STATIC ||
            type == IND_PERM)
@@ -1005,19 +884,19 @@ void prettyPrintClosure_ (StgClosure *obj)
     info = get_itbl(obj);
 
     /* determine what kind of object we have */
-    switch (info->type) 
+    switch (info->type)
     {
         /* full applications of data constructors */
         case CONSTR:
-        case CONSTR_1_0: 
+        case CONSTR_1_0:
         case CONSTR_0_1:
-        case CONSTR_1_1: 
-        case CONSTR_0_2: 
+        case CONSTR_1_1:
+        case CONSTR_0_2:
         case CONSTR_2_0:
         case CONSTR_STATIC:
-        case CONSTR_NOCAF_STATIC: 
+        case CONSTR_NOCAF_STATIC:
         {
-           nat i; 
+           nat i;
            char *descriptor;
 
            /* find the con_info for the constructor */
@@ -1061,7 +940,7 @@ void printPtr( StgPtr p )
 {
     debugBelch("ptr 0x%p (enable -DDEBUG for more info) " , p );
 }
-  
+
 void printObj( StgClosure *obj )
 {
     debugBelch("obj 0x%p (enable -DDEBUG for more info) " , obj );
@@ -1072,7 +951,7 @@ void printObj( StgClosure *obj )
 
 /* -----------------------------------------------------------------------------
    Closure types
-   
+
    NOTE: must be kept in sync with the closure types in includes/ClosureTypes.h
    -------------------------------------------------------------------------- */
 
@@ -1129,7 +1008,7 @@ char *closure_type_names[] = {
  [MUT_VAR_CLEAN]         = "MUT_VAR_CLEAN",
  [MUT_VAR_DIRTY]         = "MUT_VAR_DIRTY",
  [WEAK]                  = "WEAK",
- [PRIM]	                 = "PRIM",
+ [PRIM]                  = "PRIM",
  [MUT_PRIM]              = "MUT_PRIM",
  [TSO]                   = "TSO",
  [STACK]                 = "STACK",
@@ -1141,17 +1020,17 @@ char *closure_type_names[] = {
 };
 
 char *
-info_type(StgClosure *closure){ 
+info_type(StgClosure *closure){
   return closure_type_names[get_itbl(closure)->type];
 }
 
 char *
-info_type_by_ip(StgInfoTable *ip){ 
+info_type_by_ip(StgInfoTable *ip){
   return closure_type_names[ip->type];
 }
 
 void
-info_hdr_type(StgClosure *closure, char *res){ 
+info_hdr_type(StgClosure *closure, char *res){
   strcpy(res,closure_type_names[get_itbl(closure)->type]);
 }
 
