@@ -32,9 +32,6 @@ void      stat_endGC  (Capability *cap, struct gc_thread_ *_gct,
                        W_ live, W_ copied, W_ slop, nat gen,
                        nat n_gc_threads, W_ par_max_copied, W_ par_tot_copied);
 
-void stat_gcWorkerThreadStart (struct gc_thread_ *_gct);
-void stat_gcWorkerThreadDone  (struct gc_thread_ *_gct);
-
 #ifdef PROFILING
 void      stat_startRP(void);
 void      stat_endRP(nat, 
