@@ -22,7 +22,6 @@ import HpcLexer
 import qualified Data.Array as Happy_Data_Array
 import qualified GHC.Exts as Happy_GHC_Exts
 import Control.Applicative(Applicative(..))
-import Control.Monad (ap)
 
 -- parser produced by Happy Version 1.19.4
 
@@ -347,8 +346,8 @@ instance Functor HappyIdentity where
     fmap f (HappyIdentity a) = HappyIdentity (f a)
 
 instance Applicative HappyIdentity where
-    pure  = return
-    (<*>) = ap
+    pure    = return
+    a <*> b = (fmap id a) <*> b
 instance Monad HappyIdentity where
     return = HappyIdentity
     (HappyIdentity p) >>= q = q p
@@ -402,6 +401,7 @@ hpcParser filename = do
 happyError e = error $ show (take 10 e)
 {-# LINE 1 "templates/GenericTemplate.hs" #-}
 {-# LINE 1 "templates/GenericTemplate.hs" #-}
+{-# LINE 1 "<built-in>" #-}
 {-# LINE 1 "<command-line>" #-}
 {-# LINE 8 "<command-line>" #-}
 # 1 "/usr/include/stdc-predef.h" 1 3 4
@@ -421,9 +421,6 @@ happyError e = error $ show (take 10 e)
 
 
 
-# 1 "/usr/include/x86_64-linux-gnu/bits/predefs.h" 1 3 4
-
-# 18 "/usr/include/x86_64-linux-gnu/bits/predefs.h" 3 4
 
 
 
@@ -436,7 +433,14 @@ happyError e = error $ show (take 10 e)
 
 
 
-# 31 "/usr/include/stdc-predef.h" 2 3 4
+
+
+
+
+
+
+
+
 
 
 
